/******************************************************************************
   getdtraces: get demultiplexed e3d traces file

   usage: getdtraces [trace] < e3dTracesFile > demultiplexedTracesFile

      trace: select specific trace and write it as sac file

   examples: getdtraces 5 < e3dTracesFile > sacFile
             getdtraces   < e3dTracesFile > demultiplexedTracesFile

   note: if trace option is used then a fully demultiplexed trace file
         will not be created. Trace number starts at 0.

   author: shawn larsen
   date:   september 1990
   where:  caltech
******************************************************************************/

#include <stdio.h>
#include <math.h>
#include "usage.h"

main(argc, argv)
int    argc;
char **argv;
{
   float dt, *value, **data;
   int i, j, g, n, t, *order, dummy;
   int trace;

   /* check usage */
   if (checkUsage(argc, argv)) exit(0);

   /* get parameters */
   trace=-1;
   if (argc == 2) trace = atoi(argv[1]);
   /*
   setpar(argc, argv);
   getpar("trace", "d", &trace);
   endpar();
   */

   /* read initial header */
   fread(&n,  sizeof(  int), 1, stdin);
   fread(&t,  sizeof(  int), 1, stdin);
   fread(&dt, sizeof(float), 1, stdin);
   if (n  <= 0 ) {fprintf(stderr, "no traces\n"); exit(1);}
   if (t  <= 0 ) {fprintf(stderr, "no data  \n"); exit(1);}
   if (dt <= 0.) {fprintf(stderr, "bad time \n"); exit(1);}

   /* allocate ordering array, value array, and demultiplexed data array */
   order = (int    *) malloc(n*sizeof(int    ));
   value = (float  *) malloc(n*sizeof(float  ));
   data  = (float **) malloc(n*sizeof(float *));
   for(i = 0; i < n; i++) data[i] =  (float *) malloc(t*sizeof(float));

   /* read ordering array */
   g = 0;
   while(g < n) {
      fread(order + g, sizeof(int), 1, stdin);
      if (order[g] < 0) break; 
      g++;
      }
   if (g == n) fread(&dummy, sizeof(int), 1, stdin);

   /* initialize demultiplexed data array */
   for(i = 0; i < n; i++) for(j = 0; j < t; j++) data[i][j] = 0.;

   /* read multiplexed data and store in demultiplexed array */
   for(i = 0; i < t; i++) {
      fread(value, sizeof(float), g, stdin); 
      for(j = 0; j < g; j++) data[order[j]][i] = value[j];
      }

   /* write demultiplexed file */
   if (trace < 0) {
      for(i = 0; i < n; i++) {
         fwrite(data[i], sizeof(float), t, stdout);
         }
      }

   /* write SAC file */
   if (trace >=0 && trace < n) writeSAC(data[trace], t, dt, stdout);

   /* free memory */
   for(i = 0; i < n; i++) free(data[i]);
   free(data );
   free(value);
   free(order);

}




/******************************************************************************
   checkUsage: check usage and print usage message if no input

   usage: checkUsage(argc, argv)

   return 0: correct usage - continue execution
          1: print   usage - stop     execution
******************************************************************************/
checkUsage(argc, argv)
int    argc;
char **argv;
{
   struct stat statbuf;
   int i, error = 1;
 
   /* get input status */
   fstat(0, &statbuf);
 
   /* normal input */
   if (statbuf.st_mode & REGULAR_FILE) error = 0;
   if (statbuf.st_mode & PIPE)         error = 0;

   /* abnormal input */
   if (argc < 1 || argc > 2) error = 1;
 
   /* print usage and return exit */
   if (error) {
      i = 0;
      while(getdtracesUsage[i]) fprintf(stderr, "%s", getdtracesUsage[i++]);
      }
   return(error);
 
}

