/******************************************************************************
   getrle: do rle compression of an e3d velocity file

   usage: getrle < unCompressedFile > CompressedFile

   compile: cc -g -o getrle getrle.o -lm
******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#define REGULAR_FILE 0100000
#define PIPE         0010000

#define BUFFERSIZE 1000000
#define BIGNUMBER  4294967296

main(argc, argv)
int    argc;
char **argv;
{
   float buf[BUFFERSIZE], v;
   int p, size;
   unsigned long int n;

   if (checkUsage(argc, argv)) exit(-1);

   n = 0;
   while((size = fread(buf, sizeof(float), BUFFERSIZE, stdin))) {
      for(p = 0; p < size; p++) {
         if (n == 0) v = buf[p];
         if (v == buf[p]) n++;
         if (v != buf[p]) {
            fwrite(&n, 4, 1, stdout);
            fwrite(&v, 4, 1, stdout);
            v = buf[p];
            n = 1;
            }
         if (n == BIGNUMBER) {
            fwrite(&n, 4, 1, stdout);
            fwrite(&v, 4, 1, stdout);
            n = 0;
            }
         }
      if (n) {
         fwrite(&n, 4, 1, stdout);
         fwrite(&v, 4, 1, stdout);
         n = 0;
         }
      }

}


/******************************************************************************
   checkUsage: check usage and print usage message if no input

   usage: checkUsage(argc, argv)

   return 0: correct usage - continue execution
          1: print   usage - stop     execution
******************************************************************************/
checkUsage(argc, argv)
int    argc;
char **argv;
{
   struct stat statbuf0, statbuf1;
   int i, error = 2;

   /* get input status */
   fstat(0, &statbuf0);
   fstat(1, &statbuf1);

   /* normal input */
   if (statbuf0.st_mode & REGULAR_FILE) error -= 1;
   if (statbuf0.st_mode & PIPE)         error -= 1;
   if (statbuf1.st_mode & REGULAR_FILE) error -= 1;
   if (statbuf1.st_mode & PIPE)         error -= 1;

   /* abnormal input */
   if (argc > 1) error = 1;

   /* print usage and return exit */
   if (error) {
      fprintf(stderr, 
      "usage: getrle < unCompressedFile > CompressedFile\n");
      }
   return(error);

}

