/* 
   getsubvolume 
   convert e3d volume output computed on mpp into serial version 
   usage: getsubvolume < mppvolume > serialvolume 
   nx,ny,nz global dimensions of grid 
   sx,sy,sz dimensions of subgrid 
   x0,y0.z0 offset of subgrid in global grid 
*/

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>

main(int argc, char **argv)
{
   float ***array; 
   int i, j, k, l, m, n, nx, ny, nz, sx, sy, sz, x0, y0, z0, zero; 

   /* read in header from first data subset to get sizes */
   readHeader(&nx, &ny, &nz, &sx, &sy, &sz, &x0, &y0, &z0);

   /* allocate memory */
   array = (float ***) malloc(ny*sizeof(float **));
   for(l = 0; l < ny; l++) 
      array[l] = (float **) malloc(nz*sizeof(float *));
   for(l = 0; l < ny; l++) for(m = 0; m < nz; m++)
      array[l][m] = (float *) malloc(nx*sizeof(float));

   /* read in rest of mpp file, stop at end */
   do {
      for(k = 0; k < sy; k++) for(j = 0; j < sz; j++) for(i = 0; i < sx; i++)
         fread(array[k+y0][j+z0]+(i+x0), sizeof(float), 1, stdin);
      }
   while(readHeader(&nx, &ny, &nz, &sx, &sy, &sz, &x0, &y0, &z0));

   /* write out serial header */
   zero = 0; 
   fwrite(&nx   , sizeof(int), 1, stdout);
   fwrite(&ny   , sizeof(int), 1, stdout);
   fwrite(&nz   , sizeof(int), 1, stdout);
   fwrite(&nx   , sizeof(int), 1, stdout);
   fwrite(&ny   , sizeof(int), 1, stdout);
   fwrite(&nz   , sizeof(int), 1, stdout);
   fwrite(&zero , sizeof(int), 1, stdout);
   fwrite(&zero , sizeof(int), 1, stdout);
   fwrite(&zero , sizeof(int), 1, stdout);
   
   /* write out array */
   for(k = 0; k < ny; k++) for(j = 0; j < nz; j++) 
      fwrite(array[k][j], sizeof(float), nx, stdout);

   /* free memory */
   for(l = 0; l < ny; l++) for(m = 0; m < nz; m++) free(array[l][m]);
   for(l = 0; l < ny; l++) free(array[l]);
   free(array);

}

int
readHeader(int *nx, int *ny, int *nz, 
           int *sx, int *sy, int *sz, 
           int *x0, int *y0, int *z0)
{
   if (!fread(nx, sizeof(int), 1, stdin)) return(0);
   if (!fread(ny, sizeof(int), 1, stdin)) return(0);
   if (!fread(nz, sizeof(int), 1, stdin)) return(0);
   if (!fread(sx, sizeof(int), 1, stdin)) return(0);
   if (!fread(sy, sizeof(int), 1, stdin)) return(0);
   if (!fread(sz, sizeof(int), 1, stdin)) return(0);
   if (!fread(x0, sizeof(int), 1, stdin)) return(0);
   if (!fread(y0, sizeof(int), 1, stdin)) return(0);
   if (!fread(z0, sizeof(int), 1, stdin)) return(0);
   return(1);
}

