/******************************************************************************
   getunrle: get e3d velocity file from compressed velocity file

   usage: getunrle < CompressedFile > unCompressedFile

   compile: cc -g -o getunrle getunrle.o -lm
******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#define REGULAR_FILE 0100000
#define PIPE         0010000

main(argc, argv)
int    argc;
char **argv;
{
   float v;
   unsigned long int n, p;

   if (checkUsage(argc, argv)) exit(-1);

   while(fread(&n, sizeof(float), 1, stdin)) {
      p = fread(&v, sizeof(float), 1, stdin);
      if (p != 1) {
         fprintf(stderr, "Error: read error\n");
         exit(-1);
         }
      for(p = 0; p < n; p++) fwrite(&v, sizeof(float), 1, stdout);
      }

}


/******************************************************************************
   checkUsage: check usage and print usage message if no input

   usage: checkUsage(argc, argv)

   return 0: correct usage - continue execution
          1: print   usage - stop     execution
******************************************************************************/
checkUsage(argc, argv)
int    argc;
char **argv;
{
   struct stat statbuf0, statbuf1;
   int i, error = 2;

   /* get input status */
   fstat(0, &statbuf0);
   fstat(1, &statbuf1);

   /* normal input */
   if (statbuf0.st_mode & REGULAR_FILE) error -= 1;
   if (statbuf0.st_mode & PIPE)         error -= 1;
   if (statbuf1.st_mode & REGULAR_FILE) error -= 1;
   if (statbuf1.st_mode & PIPE)         error -= 1;

   /* abnormal input */
   if (argc > 1) error = 1;

   /* print usage and return exit */
   if (error) {
      fprintf(stderr, 
      "usage: getunrle < CompressedFile > unCompressedFile\n");
      }
   return(error);

}

