/******************************************************************************
   readsac: read sac binary file and convert to ASCII

   usage: readsac < binary.sac > ascii.sac

   compile: cc -g -o readsac readsac.c -lm

   note: header will contain offset dt0 (usually 0), time increment dt,
         number of points n, and error flag (hopefully 0)
******************************************************************************/

#include <stdio.h>

main()
{
   float y[10000], dt0, dt;
   int n, error, i;

   error = rsac(y, &dt0, &dt, &n, stdin);

   printf("dt0 = %f dt = %f n = %d error = %d\n", dt0, dt, n, error);

   for(i = 0; i < n; i++) printf("%10.3e\n", y[i]);

}


/******************************************************************************
   rsac: read sac binary file and extract important information 

   usage: error = rsac(y, &dt0, &dt, &n, in)

   y  : pointer to array containing seismogram (float *) (preallocated)
   dt0: starting time of seismogram (float)
   dt : increment between points (float)
   n  : number of points (float)
   in : pointer to file stream (FILE *)

   error: 0 (successful); -1 (read error)
******************************************************************************/

#define  SACSIZE   (158*4)    /* size of sac header (bytes) */     
#define WORDSIZE        4     /* size of sac word           */
#define   DDELTA        0     /* offset (words) of delta    */
#define       DB        5     /* offset (words) of start    */
#define    DNPTS       79     /* offset (words) of npts     */

int
rsac(y, dt0, dt, n, in)
float *y, *dt0, *dt;
int   *n;
FILE  *in;
{
   int good = 0;
   char header[SACSIZE];
   if (fread(header, SACSIZE, 1, in) != 1) good = -1;
   *dt0 = 0.;
   *dt  = *((float *) &header[WORDSIZE*DDELTA]);
   *dt0 = *((float *) &header[WORDSIZE*DB    ]);
   *n   = *((int   *) &header[WORDSIZE*DNPTS ]);
   if (fread(y, sizeof(float), *n, in) != *n) good = -1;
   return(good);
}
