/******************************************************************************
   sacutil: routines for conversions from/to sac 

   getSACUndefine:   get undefined entries for SAC header
   getSACDefault:    get undefined entries in short SAC header
   putSACParameter:  copy SAC parameters to header
   getSACHeader:     get SAC header

   author: shawn larsen
   date:   june 1992
   where:  llnl
******************************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include "sacutil.h"


/******************************************************************************
   getSACUndefine: get undefined entries for SAC header

   usage: getSACUndefine(h)
      h - complete SAC header
******************************************************************************/
getSACUndefine(h, sacParameter)
char *h;
SACParameter *sacParameter;
{
   float fWord;
   int i, iWord, lWord, wordStart, wordEnd;
   char kWord[9];

   /* initialize undefined values */
   fWord = FVALUE;
   iWord = IVALUE;
   lWord = LVALUE;
   strcpy(kWord, KVALUE);

   /* initialize float entries */
   wordStart = 0;
   wordEnd = wordStart + FWORDS;
   for(i = wordStart; i < wordEnd; i++) *((float *) (h + i*WORDSIZE)) = fWord;
   
   /* initialize integer entries */
   wordStart = FWORDS;
   wordEnd = wordStart + IWORDS;
   for(i = wordStart; i < wordEnd; i++) *((int   *) (h + i*WORDSIZE)) = iWord;
   
   /* initialize logical entries */
   wordStart = FWORDS + IWORDS;
   wordEnd = wordStart + LWORDS;
   for(i = wordStart; i < wordEnd; i++) *((int   *) (h + i*WORDSIZE)) = lWord;

   /* initialize character entries */
   wordStart = FWORDS + IWORDS + LWORDS;
   wordEnd = wordStart + KWORDS;
   for(i = wordStart; i < wordEnd; i += 2) 
      strncpy(h + i*WORDSIZE, kWord, 8);

   /* get special float entries (there are none) */

   /* get special integer entries */
   *((int   *) (h +     DNVHDR*WORDSIZE)) = HVERSION;
   *((int   *) (h + DINTERNAL4*WORDSIZE)) = INTERNAL;
   *((int   *) (h + DINTERNAL5*WORDSIZE)) = INTERNAL;

   /* get special logical entries */
   *((int   *) (h +    DLOVROK*WORDSIZE)) = OVERWRITE;
   *((int   *) (h +     DCALDA*WORDSIZE)) = CALDA;
   
   /* get special character entries (there are none) */

}




/******************************************************************************
   getSACDefault: get default values of SAC header parameters

   usage: getSACDefault(s)
      s - short SAC header
******************************************************************************/
getSACDefault(sacParameter)
SACParameter *sacParameter;
{
   float fWord;
   int iWord, lWord;
   char kWord[9];

   /* determine undefined default values */
   fWord = FVALUE;
   iWord = IVALUE;
   lWord = LVALUE;
   strcpy(kWord, KVALUE);

   /* get undefined default values */
   sacParameter->delta  = fWord;
   sacParameter->depmin = fWord;
   sacParameter->depmax = fWord;
   sacParameter->b      = fWord;
   sacParameter->e      = fWord;
   sacParameter->npts   = iWord;
   sacParameter->iftype = iWord;
   sacParameter->leven  = lWord;

}




/******************************************************************************
   putSACParameter: copy SAC parameters to header

   usage: putSACParameter(h, s)
      h - complete SAC header
      s - short SAC header
******************************************************************************/
putSACParameter(header, sacParameter)
char         *header;
SACParameter *sacParameter;
{
   *((float *) (header +  DDELTA*WORDSIZE)) = sacParameter->delta;
   *((float *) (header + DDEPMIN*WORDSIZE)) = sacParameter->depmin;
   *((float *) (header + DDEPMAX*WORDSIZE)) = sacParameter->depmax;
   *((float *) (header +      DB*WORDSIZE)) = sacParameter->b;
   *((float *) (header +      DE*WORDSIZE)) = sacParameter->e;
   *((int   *) (header +   DNPTS*WORDSIZE)) = sacParameter->npts;
   *((int   *) (header + DIFTYPE*WORDSIZE)) = sacParameter->iftype; 
   *((int   *) (header +  DLEVEN*WORDSIZE)) = sacParameter->leven;
}




/******************************************************************************
   getSACHeader: get SAC header

   usage: getSACHeader(h, s)
      h - complete SAC header
      s - short SAC header
******************************************************************************/
getSACHeader(header, sacParameter)
char         *header;
SACParameter *sacParameter;
{
   getSACUndefine(header, sacParameter);
   putSACParameter(header, sacParameter);
}
