/****************************************************************************
This file contains usage strings for elastic finite difference aux programs

    ================================================= 
   |     Copyright (c) Shawn C. Larsen               |
   |     Seismological Laboratory                    |
   |     California Institute of Technology          |
   |     Pasadena, CA 91125                          |
   |     All Rights Reserved.                        |
    ================================================= 

author: shawn larsen
date:   june 1990
where:  caltech
****************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>

#define REGULAR_FILE 0100000
#define PIPE         0010000


/*****************
USAGE FOR GETIMAGE
*****************/

char *getimageUsage[] = {
   "Usage:  getimage [optional parameters] < outputFieldFile > imageFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   scale  -     scale factor (default 0: autoscale)\n",
   "   ascale - autoscale factor (default 100.)\n",
   "   sample - sampling factor (default 1)\n",
   "   tfile  - topography file (default none)\n",
   "   mode   - 32 bit mode (if mode = 32)\n",
   "   file   - color table file (GMT format)\n",
   "\n",
   "Example: getimage sample=2 scale=2.0 < outfield.1000 > image.1000.ras\n",
   NULL,
   };



/**************
USAGE FOR GETPS
**************/

char *getpsUsage[] = {
   "Usage:  getps [optional parameters] Pfile Sfile > imageFile\n",
   "\n",
   " Parameters\n",
   " ----------\n",
   "   scale  -    scale factor (default 0: autoscale)\n",
   "   pratio -  p scale factor (default 1: autoscale)\n",
   "   sample - sampling factor (default 1)\n",
   "   color  - color scheme (1 = black bg; 2 = white bg) (default 1)\n",
   "   clip   - clipping color value (default 255 color=1; 0 color=2)\n",
   "   mode   - 32 bit mode (if mode = 32)\n",
   "   tfile  - topography file (default none)\n",
   "\n",
   "Example: getps sample=2 image.1000.p image.1000.s > imageps.1000.ras\n",
   NULL,
   };



/***************
USAGE FOR GETXYZ
***************/

char *getxyzUsage[] = {
   "Usage:  getxyz dh [optional parameters] < outputFieldFile > xyzFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   dh     - spatial sampling increment (required)\n",
   "   scale  - scale factor     (default 1.0)\n",
   "   sample - sampling factor  (default 1)\n",
   "   lon0   - origin longitude (default xyz)\n",
   "   lat0   - origin latitude  (default xyz)\n",
   "   angle  - rotation angle (default 0.)\n",
   "   lonmin - minimum longitude (default none)\n",
   "   lonmax - maximum longitude (default none)\n",
   "   latmin - minimum latitude  (default none)\n",
   "   latmax - maximum latitude  (default none)\n",
   "   nx     - points in x direction (default none)\n", 
   "   ny     - points in y direction (default none)\n", 
   "\n",
   "Example: getxyz dh=0.25 scale=2.0 < image.1000.E > E.1000.xyz\n",
   NULL,
   };



/*************
USAGE FOR GETQ
*************/

char *getqUsage[] = {
   "Usage:  getq [parameters] > QFile.sac\n",
   "\n",
   " Parameters\n",
   " ----------\n",
   "   q    - desired Q [required]\n",
   "   f    - frequency at minimum Q [default 1.0 hz]\n",
   "   r    - number of relaxation parameters [default 1]\n",
   "   n    - number of frequency values [default 100]\n",
   "   df   - frequency increment [default 0.1]\n",
   "   mode - mode to select attenuation values [1]\n"
   "          1: Tsig = (sqrt(Q**2 + 1) - 1)/(w*Q)\n"
   "             Teps = (sqrt(Q**2 + 1) + 1)/(w*Q)\n"
   "             [r = 1 only]\n"
   "          2: Tsig = 1/w\n"
   "             Teps = ((2 + Q)/Q)*(1/w)\n"
   "             [r = 1 only]\n"
   "          3: Tau method\n"
   "\n",
   "Example: getq q=10. f=2. > Qfreq.sac\n",
   NULL,
   };



/****************
USAGE FOR GETTIME
****************/

char *gettimeUsage[] = {
   "Usage:  gettime [parameters]\n",
   "\n",
   " Parameters\n",
   " ----------\n",
   "   x    - grid distance in x [required]\n",
   "   y    - grid distance in y [2D if not specified]\n",
   "   z    - grid or subgrid distance in z [required]\n",
   "   v    - velocity or subgrid velocity in z (km/s) [default 1.0]\n",
   "   f    - frequency (hz) [default 1.0]\n",
   "   p    - number of parallel processors [default 1]\n",
   "   s    - single processor speed (MFlops) [default 50]\n",
   "   a    - flag for calculation with active grid\n",
   "   par  - input parameter file\n",
   "\n",
   "Example: gettime par=\"run.par\"\n",
   NULL,
   };

/********************
USAGE FOR GETSUBIMAGE
********************/

char *getsubimageUsage[] = {
   "Usage:  getsubimage [optional parameters] < e3dSubImageFile > imageFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   sample - sampling factor (default autosample)\n",
   "\n",
   "Example: getsubimage sample=1 < subimage.1000 > image.1000\n",
   NULL,
   };

/********************
USAGE FOR GETSUBFIELD
********************/

char *getsubfieldUsage[] = {
   "Usage:  getsubfield [optional parameters] < e3dSubFieldFile > fieldFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   sample - sampling factor (default autosample)\n",
   "\n",
   "Example: getsubfield sample=1 < subfield.1000 > field.1000\n",
   NULL,
   };


/******************
USAGE FOR GETDFAULT
******************/

char *getdfaultUsage[] = {
   "Usage:  getdfault longitude latitude angle dh dt [mode] < in > distFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   longitude - longitude of unrotated bottom-left\n",
   "   latitude  - latitude  of unrotated bottom-left\n",
   "   angle     - angle of grid rotation (ccw from east)\n",
   "   dh        - grid spacing (km)\n",
   "   dt        - timestep increment (sec)\n",
   "   mode      - [optional] 1(def)(node); 2(xyslip); 3(xytime); 4(xyrise)\n",
   "\n",
   "Example: getdfault -121.20 37.40 124. 0.25 0.015 2 < coord > distributed\n",
   "Note (input format): lat lon depth(km) slip(cm) rise(s) time(s) mom(cgs)\n",
   NULL,
   };


/*******************
USAGE FOR GETVMOTION
*******************/

char *getvmotionUsage[] = {
   "Usage:  getvmotion [optional parameters] < e3dVelocityFile > asciiFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   xmin   - minimum x value (km)         [default       0.]\n",
   "   xmax   - minimum x value (km)         [default (n-1)*dx]\n",
   "   ymin   - minimum y value (km)         [default       0.]\n",
   "   ymax   - minimum y value (km)         [default (m-1)*dx]\n",
   "   dx     - grid increment  (km)         [default      1.0]\n",
   "   sample - sampling rate in grid points [default        1]\n",
   "\n",
   "Example: getvmotion xmin=50. xmax=100. dx=0.25 sample=2 < rcrk.V > rcrk.ascii\n",
   "Example: getvmotion par=\"input.par\"                   < rcrk.V > rcrk.ascii\n",
   NULL,
   };


/**************
USAGE FOR DSLIP
**************/

char *dslipUsage[] = {
   "Usage:  dslip [parameters] < inputFile > outputFile\n",
   "\n",
   NULL,
   };



/*****************
USAGE FOR GETFRAME
*****************/

char *getframeUsage[] = {
   "Usage:  getframe [optional parameters] < e3dFrameFile > imageFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   frame  - frame number to extract (start 1; 0 will output header)\n",
   "   sample - sampling factor (default 0 = autosample)\n",
   "\n",
   "Example: getframe frame=5 sample=1 < image.Vx > image.1000.Vx\n",
   NULL,
   };



/******************
USAGE FOR GETFRAMES
******************/

char *getframesUsage[] = {
   "Usage:  getframes [optional parameters] < e3dFrameFile > imageFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   mode   - 1 average       (average at end)\n",
   "          - 2 integrate     (average at end)\n",
   "          - 3 maximum       (absolute) [default]\n",
   "   sample - sampling factor (default 0 = autosample)\n",
   "   time   - number of frames to average at end (default nframes/20)\n",
   "\n",
   "Example: getframes mode=2 sample=1 time=10 < image.Vx > image.Dx\n",
   NULL,
   };


/*****************
USAGE FOR SACFRAME
*****************/

char *sacframeUsage[] = {
   "Usage:  sacframe [parameters] < e3dFrameFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   file   - file to read in sac extraction values\n",
   "   sample - spacial sampling factor (best to use E3D)\n",
   "\n",
   "Example: sacframe file=\"sac.input\" < image.Vx\n",
   NULL,
   };

/*****************
USAGE FOR SEGTRACE
*****************/

char *segtraceUsage[] = {
   "Usage:  segtrace [parameters] < e3dFrameFile > traceFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   file   - file to read in extraction values\n",
   "   sample - spacial sampling factor (best to use E3D)\n",
   "\n",
   "Example: segtrace file=\"seg.input\" < seg.FP > seg.line\n",
   NULL,
   };

/*******************
USAGE FOR GETDTRACES
*******************/

char *getdtracesUsage[] = {
   "Usage:  getdtraces [trace] < e3dTracesFile > demultiplexedTraceFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   trace - trace number to extract as SAC file\n",
   "\n",
   "Example: getdtraces   < e3d.0.TVx > demultiplex.0.Vx\n",
   "Example: getdtraces 5 < e3d.0.TVx > sac5.x\n",
   NULL,
   };

/******************
USAGE FOR GETTIMAGE
******************/

char *gettimageUsage[] = {
   "Usage:  gettimage n m N < e3dTracesFile > e3dImage\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   n - x dimension of image\n",
   "   m - y or z dimension of image\n",
   "   N - image number to extract as E3D image file [start at 1]\n",
   "\n",
   "Example: gettraces 200 100 5 < e3d.0.TVx > image.5.x\n",
   NULL,
   };

/*******************
USAGE FOR GETTIMAGEM
*******************/

char *gettimagemUsage[] = {
   "Usage:  gettimagem n m N headerName extName < e3dTracesFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   n - x dimension of image\n",
   "   m - y or z dimension of image\n",
   "   N - image number sampling\n",
   "   headerName - header for output files\n",
   "   extName    - extension for output files\n",
   "\n",
   "Example: gettraces 200 100 5 image x < e3d.0.TVx\n",
   NULL,
   };

/******************
USAGE FOR GETMIMAGE
******************/

char *getmimageUsage[] = {
   "Usage:  getmimage n m t N headerName extName < e3dTracesFile.Demul\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   n - x dimension of image\n",
   "   m - y or z dimension of image\n",
   "   t - number of timesteps in traces (total images)\n",
   "   N - image number sampling (negative for just one image - (start -1))\n",
   "   headerName - header for output files\n",
   "   extName    - extension for output files\n",
   "\n",
   "Example: getmimage 200 100 2000 5 image x < e3d.0.TVx.demul\n",
   NULL,
   };

/******************
USAGE FOR GETRFRAME
******************/


/******************
USAGE FOR GETRFRAME
******************/

char *getrframeUsage[] = {
   "Usage:  getrframe [optional parameters] < e3dFrameFile > imageFile\n",
   "\n",
   "Parameters\n",
   "----------\n",
   "   sample - sampling factor (best to use e3d sampling factor)\n",
   "   scale  - scale output by this factor [1.0]\n",
   "   ascii  - ascii output mode (0:none;1:ascii;2:asciihead) [0]\n",
   "   T      - period of interest (sec) [10.]\n",
   "   D      - damping factor (fraction) [0.05]\n",
   "   mode   - mode of output (3:displacement) [3]\n",
   "\n",
   "Example: getrframe sample=2 < frames.Vx > response.Vx\n",
   NULL,
   };

