/******************************************************************************
   algorithms: various independent algorithms for 2D/3D finite-difference 

    ================================================================= 
   | (c) Copyright 1996 The Regents of the University of California. |
   |     All Rights Reserved.                                        |
   |                                                                 |
   |     This work was produced under the sponsorship of the         |
   |     United States Department of Energy.                         |
   |     The Government retains certain rights therein.              |
    ================================================================= 

   getPlane       : get plane parameterization from 4 coordinates

   compile: cc -g -c algorithms.c

   note: used  units are mks (m, s, m/s, kg/m3)

   author: shawn larsen
   date  : december 1996
   where : llnl
*****************************************************************************/

#include <stdio.h>
#include <math.h>


/******************************************************************************
   getPlane: get plane parameterization from 4 coordinates

   usage: getPlane(x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4, a, b, c, d)

      xn: x coordinates
      yn: y coordinates
      zn: z coordinates
      a : a plane parameter
      b : b plane parameter
      c : b plane parameter
      d : b plane parameter

   note: plane is given by "ax + by + cz + d = 0"
   note: use Newell's method of Roberts algorithm (P. 209, Procedural Elements 
         for Computer Graphics, David F. Rogers, 1985)
   note: define a, b, c, d such that points in positive z direction give
         positive dot product with Roberts algorithm
******************************************************************************/
getPlaneABCD(x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4, a, b, c, d)
float x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4, *a, *b, *c, *d;
{
   *a = (y1-y2)*(z1+z2) + (y2-y3)*(z2+z3) + (y3-y4)*(z3+z4) + (y4-y1)*(z4+z1);
   *b = (z1-z2)*(x1+x2) + (z2-z3)*(x2+x3) + (z3-z4)*(x3+x4) + (z4-z1)*(x4+x1);
   *c = (x1-x2)*(y1+y2) + (x2-x3)*(y2+y3) + (x3-x4)*(y3+y4) + (x4-x1)*(y4+y1);
   *d = -(*a*x1 + *b*y1 + *c*z1);
}

