/******************************************************************************
   boundary: boundary condition for 2D/3D elastic finite-difference

   compile: cc -g -c boundary.c 

   note: input units are km, s, km/s, g/cm3
   note: used  units are mks (m, s, m/s, kg/m3, N-m)

   author: shawn larsen
   date  : december 1989
   where : caltech
*****************************************************************************/

#include "elas.h"


boundaryV2D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int g;

   if (data->t > 0) getClock(par, grid, 3, 0);

   /* apply side boundary conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bx1 == ABSORBING) absorbLVLa(par, grid, data);
   if (grid->bx2 == ABSORBING) absorbLVR (par, grid, data);
   if (grid->bz1 == ABSORBING) absorbLVT (par, grid, data);
   if (grid->bz2 == ABSORBING) absorbLVBa(par, grid, data);

   /* apply surface conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bz1 == SURFACE   ) surface2VT (par, grid, data); /* wasVT (old) */
   if (grid->bz1 == TOPOGRAPHY) ; /* was write topoV2D */

   /* apply corner conditions */
   for(g = 0; g < par->g; g++) {
      }
   if ((grid->bz1 == SURFACE) || (grid->bz1 == TOPOGRAPHY)) {
      if (grid->bx1 == ABSORBING) cornerVTLa(par, grid, data); /*see old note */
      if (grid->bx2 == ABSORBING) cornerVTR (par, grid, data);
      }
   if (grid->bz1==ABSORBING && grid->bx1==ABSORBING)cornerVTLa(par, grid, data);
   if (grid->bz1==ABSORBING && grid->bx2==ABSORBING)cornerVTR (par, grid, data);
   if (grid->bz2==ABSORBING && grid->bx1==ABSORBING)cornerVBL (par, grid, data);
   if (grid->bz2==ABSORBING && grid->bx2==ABSORBING)cornerVBRa(par, grid, data);

   if (data->t > 0) getClock(par, grid, 3, 1);

   /* apply damping boundary conditions */
   if (par->damp) damp2D(par, grid, data);

   /* apply variable density grid boundaries */
   if (par->g > 1) multiUpdate2D(par, grid, data, 1);

   /* apply parallel message passing (DOESN'T WORK FOR VARIABLE DENSITY GRID) */
   if (data->t > 0) getClock(par, grid, 4, 0);
   if (par->nnode > 1) mpUpdate2D(par, grid, data, 1);
   if (data->t > 0) getClock(par, grid, 4, 1);

}


boundaryT2D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{

   int g;
 
   if (data->t > 0) getClock(par, grid, 3, 0);

   /* apply side boundary conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bx1 == ABSORBING) boundTLa(par, grid, data);
   if (grid->bx2 == ABSORBING) boundTR (par, grid, data);
   if (grid->bz1 == ABSORBING) boundTT (par, grid, data);
   if (grid->bz2 == ABSORBING) boundTBa(par, grid, data);

   /* apply surface conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bz1 == SURFACE   ) surface2TT (par, grid, data); /* wasTT (old) */
   if (grid->bz1 == TOPOGRAPHY) ; /* was topoT2D */

   /* apply corner conditions */
   for(g = 0; g < par->g; g++) {
      }
   if ((grid->bz1 == SURFACE) || (grid->bz1 == TOPOGRAPHY)) {
      if (grid->bx1 == ABSORBING) cornerTTLa(par, grid, data); /*see old note */
      if (grid->bx2 == ABSORBING) cornerTTR (par, grid, data);
      }
   if (grid->bz1==ABSORBING && grid->bx1==ABSORBING)cornerTTLa(par, grid, data);
   if (grid->bz1==ABSORBING && grid->bx2==ABSORBING)cornerTTR (par, grid, data);
   if (grid->bz2==ABSORBING && grid->bx1==ABSORBING)cornerTBL (par, grid, data);
   if (grid->bz2==ABSORBING && grid->bx2==ABSORBING)cornerTBRa(par, grid, data);

   if (data->t > 0) getClock(par, grid, 3, 1);

   /* apply variable density grid boundaries */
   if (par->g > 1) multiUpdate2D(par, grid, data, 2);

   /* apply parallel message passing (DOESN'T WORK FOR VARIABLE DENSITY GRID) */
   if (data->t > 0) getClock(par, grid, 4, 0);
   if (par->nnode > 1) mpUpdate2D(par, grid, data, 2);
   if (data->t > 0) getClock(par, grid, 4, 1);

}




boundaryV3D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int g;

   if (data->t > 0) getClock(par, grid, 3, 0);

   /* apply side boundary conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (par->water == 0 && par->model != 2) {
      if (grid->bx1 == ABSORBING) vabsorbLVL(par, grid, data);
      if (grid->bx2 == ABSORBING) vabsorbLVR(par, grid, data);
      if (grid->by1 == ABSORBING) vabsorbLVF(par, grid, data);
      if (grid->by2 == ABSORBING) vabsorbLVK(par, grid, data);
      if (grid->bz2 == ABSORBING) vabsorbLVB(par, grid, data);
      if (grid->bz1 == ABSORBING) vabsorbLVT(par, grid, data);
      }
   else if (par->water == 1 && par->model != 2) {
      if (grid->bx1 == ABSORBING) vabsorbLVLa(par, grid, data);
      if (grid->bx2 == ABSORBING) vabsorbLVR (par, grid, data);
      if (grid->by1 == ABSORBING) vabsorbLVF (par, grid, data);
      if (grid->by2 == ABSORBING) vabsorbLVKa(par, grid, data);
      if (grid->bz2 == ABSORBING) vabsorbLVBa(par, grid, data);
      if (grid->bz1 == ABSORBING) vabsorbLVT (par, grid, data);
      /****
      if (grid->bx1 == ABSORBING) vabsorbLVL (par, grid, data);
      if (grid->bx1 == ABSORBING) vabsorbLVLa(par, grid, data);
      if (grid->bx2 == ABSORBING) vabsorbLVR (par, grid, data);
      if (grid->by1 == ABSORBING) dabsorbLVF (par, grid, data);
      if (grid->by1 == ABSORBING) vabsorbLVF (par, grid, data);
      if (grid->by2 == ABSORBING) vabsorbLVK (par, grid, data);
      if (grid->by2 == ABSORBING) vabsorbLVKa(par, grid, data);
      if (grid->bz2 == ABSORBING) dabsorbLVBa(par, grid, data);
      if (grid->bz1 == ABSORBING) vabsorbLVT (par, grid, data);
      ****/
      }
   else if (par->model == 2) {
      if (grid->bx1 == ABSORBING) aabsorbLVLa(par, grid, data);
      if (grid->bx2 == ABSORBING) aabsorbLVR (par, grid, data);
      if (grid->by1 == ABSORBING) aabsorbLVF (par, grid, data);
      if (grid->by2 == ABSORBING) aabsorbLVKa(par, grid, data);
      if (grid->bz2 == ABSORBING) aabsorbLVBa(par, grid, data);
      }

   /* apply surface conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bz1 == SURFACE   ) {
      if (par->model != 2) surface3VTv(par, grid, data);
      if (par->model == 2) surface3VTa(par, grid, data);
      }
   if (grid->bz1 == TOPOGRAPHY) ; /* was write topoV3D */

   /* apply edge conditions */
   for(g = 0; g < par->g; g++) {
      }
   /*
   if ((grid->bz1 == SURFACE) || (grid->bz1 == TOPOGRAPHY)) {
   */
   if (grid->b == SURFACE || grid->b == TOPOGRAPHY || grid->b == ABSORBING) {
      if (par->water == 0 && par->model != 2) {
         dedgeVBL(par, grid, data);
         dedgeVBR(par, grid, data);
         dedgeVTL(par, grid, data);
         dedgeVTR(par, grid, data);
         dedgeVFL(par, grid, data);
         dedgeVFR(par, grid, data);
         dedgeVKL(par, grid, data);
         dedgeVKR(par, grid, data);
         dedgeVBF(par, grid, data);
         dedgeVTF(par, grid, data);
         dedgeVBK(par, grid, data);
         dedgeVTK(par, grid, data);
         }
      else if (par->water == 1 && par->model != 2) {
         dedgeVBR (par, grid, data);
         dedgeVTL (par, grid, data);
         dedgeVTR (par, grid, data);
         dedgeVFL (par, grid, data);
         dedgeVFR (par, grid, data);
         dedgeVKR (par, grid, data);
         dedgeVBF (par, grid, data);
         dedgeVTF (par, grid, data);
         dedgeVTK (par, grid, data);
         /****
         dedgeVBLa(par, grid, data);
         dedgeVBRa(par, grid, data);
         dedgeVTL (par, grid, data);
         dedgeVTR (par, grid, data);
         dedgeVFL (par, grid, data);
         dedgeVFR (par, grid, data);
         dedgeVKL (par, grid, data);
         dedgeVKR (par, grid, data);
         dedgeVBF (par, grid, data);
         dedgeVTF (par, grid, data);
         dedgeVBK (par, grid, data);
         dedgeVTK (par, grid, data);
         ****/
         }
      else if (par->model == 2) {
         aedgeVBR (par, grid, data);
         aedgeVTL (par, grid, data);
         aedgeVTR (par, grid, data);
         aedgeVFL (par, grid, data);
         aedgeVFR (par, grid, data);
         aedgeVKR (par, grid, data);
         aedgeVBF (par, grid, data);
         aedgeVTF (par, grid, data);
         aedgeVTK (par, grid, data);
         }
      }

   /* apply corner conditions */
   for(g = 0; g < par->g; g++) {
      }
   /*
   if ((grid->bz1 == SURFACE) || (grid->bz1 == TOPOGRAPHY)) {
   */
   if (grid->b == SURFACE || grid->b == TOPOGRAPHY || grid->b == ABSORBING) {
      if (par->model != 2) {
         dcornerVBLF(par,grid,data);
         dcornerVBRF(par,grid,data);
         dcornerVTLF(par,grid,data);
         dcornerVTRF(par,grid,data);
         dcornerVBLK(par,grid,data);
         dcornerVBRK(par,grid,data);
         dcornerVTLK(par,grid,data);
         dcornerVTRK(par,grid,data);
         }
      else if (par->model == 2) {
         acornerVBLF(par,grid,data);
         acornerVBRF(par,grid,data);
         acornerVTLF(par,grid,data);
         acornerVTRF(par,grid,data);
         acornerVBLK(par,grid,data);
         acornerVBRK(par,grid,data);
         acornerVTLK(par,grid,data);
         acornerVTRK(par,grid,data);
         }
      }

   if (data->t > 0) getClock(par, grid, 3, 1);

   /* amplitude reduction boundary condition */
   if (par->damp) damp3D(par, grid, data);

   /* apply variable density grid boundaries (DOESN'T WORK FOR PARALLEL) */
   if (par->g > 1) multiUpdate3D(par, grid, data, 1);
   /*
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bz1 == MULTIGRID) {
      multiGetAboveV(par, grid, data);
      if (grid->bx1 == ABSORBING) multiGetAboveVL(par, grid, data);
      if (grid->bx2 == ABSORBING) multiGetAboveVR(par, grid, data);
      }
   if (grid->bz2 == MULTIGRID) {
      multiGetBelowV(par, grid, data);
      if (grid->bx1 == ABSORBING) multiGetBelowVL(par, grid, data);
      if (grid->bx2 == ABSORBING) multiGetBelowVR(par, grid, data);
      }
   */

   /* apply parallel message passing (DOESN'T WORK FOR VARIABLE DENSITY GRID) */
   if (data->t > 0) getClock(par, grid, 4, 0);
   if (par->nnode > 1) mpUpdate3D(par, grid, data, 1);
   if (data->t > 0) getClock(par, grid, 4, 1);
}



boundaryT3D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{

   int g;
 
   if (data->t > 0) getClock(par, grid, 3, 0);

   /* apply side boundary conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (par->water == 0 && par->model != 2) {
      if (grid->bx1 == ABSORBING) vboundTL(par, grid, data);
      if (grid->bx2 == ABSORBING) vboundTR(par, grid, data);
      if (grid->by1 == ABSORBING) vboundTF(par, grid, data);
      if (grid->by2 == ABSORBING) vboundTK(par, grid, data);
      if (grid->bz2 == ABSORBING) vboundTB(par, grid, data);
      if (grid->bz1 == ABSORBING) vboundTT(par, grid, data);
      }
   else if (par->water == 1 && par->model != 2) {
      if (grid->bx1 == ABSORBING) vboundTLa(par, grid, data);
      if (grid->bx2 == ABSORBING) vboundTR (par, grid, data);
      if (grid->by1 == ABSORBING) vboundTF (par, grid, data);
      if (grid->by2 == ABSORBING) vboundTKa(par, grid, data);
      if (grid->bz2 == ABSORBING) vboundTBa(par, grid, data);
      if (grid->bz1 == ABSORBING) vboundTT(par, grid, data);
      /****
      if (grid->bx1 == ABSORBING) vboundTL (par, grid, data);
      if (grid->bx1 == ABSORBING) vboundTLa(par, grid, data);
      if (grid->bx2 == ABSORBING) vboundTR (par, grid, data);
      if (grid->by1 == ABSORBING) dboundTF (par, grid, data);
      if (grid->by1 == ABSORBING) vboundTF (par, grid, data);
      if (grid->by2 == ABSORBING) vboundTK (par, grid, data);
      if (grid->by2 == ABSORBING) vboundTKa(par, grid, data);
      if (grid->bz2 == ABSORBING) vboundTB (par, grid, data);
      if (grid->bz2 == ABSORBING) dboundTBa(par, grid, data);
      if (grid->bz1 == ABSORBING) vboundTT(par, grid, data);
      ****/
      }
   else if (par->model == 2) {
      if (grid->bx1 == ABSORBING) aboundTLa(par, grid, data);
      if (grid->bx2 == ABSORBING) aboundTR (par, grid, data);
      if (grid->by1 == ABSORBING) aboundTF (par, grid, data);
      if (grid->by2 == ABSORBING) aboundTKa(par, grid, data);
      if (grid->bz2 == ABSORBING) aboundTBa(par, grid, data);
      }

   /* apply surface conditions */
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bz1 == SURFACE   ) {
      if (par->model != 2) surface3TTv(par, grid, data);
      if (par->model == 2) surface3TTa(par, grid, data);
      }
   if (grid->bz1 == TOPOGRAPHY) ; /* was topoT2D */

   /* apply edge conditions */
   for(g = 0; g < par->g; g++) {
      }
   /*
   if ((grid->bz1 == SURFACE) || (grid->bz1 == TOPOGRAPHY)) {
   */
   if (grid->b == SURFACE || grid->b == TOPOGRAPHY || grid->b == ABSORBING) {
      if (par->water == 0 && par->model != 2) {
         dedgeTBL(par, grid, data);
         dedgeTBR(par, grid, data);
         dedgeTTL(par, grid, data);
         dedgeTTR(par, grid, data);
         dedgeTFL(par, grid, data);
         dedgeTFR(par, grid, data);
         dedgeTKL(par, grid, data);
         dedgeTKR(par, grid, data);
         dedgeTBF(par, grid, data);
         dedgeTTF(par, grid, data);
         dedgeTBK(par, grid, data);
         dedgeTTK(par, grid, data);
         }
      else if (par->water == 1 && par->model != 2) {
         dedgeTBR (par, grid, data);
         dedgeTTL (par, grid, data);
         dedgeTTR (par, grid, data);
         dedgeTFL (par, grid, data);
         dedgeTFR (par, grid, data);
         dedgeTKR (par, grid, data);
         dedgeTBF (par, grid, data);
         dedgeTTF (par, grid, data);
         dedgeTTK (par, grid, data);
         /****
         dedgeTBLa(par, grid, data);
         dedgeTBRa(par, grid, data);
         dedgeTTL (par, grid, data);
         dedgeTTR (par, grid, data);
         dedgeTFL (par, grid, data);
         dedgeTFR (par, grid, data);
         dedgeTKL (par, grid, data);
         dedgeTKR (par, grid, data);
         dedgeTBF (par, grid, data);
         dedgeTTF (par, grid, data);
         dedgeTBK (par, grid, data);
         dedgeTTK (par, grid, data);
         ****/
         }
      else if (par->model == 2) {
         aedgeTBR (par, grid, data);
         aedgeTTL (par, grid, data);
         aedgeTTR (par, grid, data);
         aedgeTFL (par, grid, data);
         aedgeTFR (par, grid, data);
         aedgeTKR (par, grid, data);
         aedgeTBF (par, grid, data);
         aedgeTTF (par, grid, data);
         aedgeTTK (par, grid, data);
         }
      }

   /* apply corner conditions */
   for(g = 0; g < par->g; g++) {
      }
   /*
   if ((grid->bz1 == SURFACE) || (grid->bz1 == TOPOGRAPHY)) {
   */
   if (grid->b == SURFACE || grid->b == TOPOGRAPHY || grid->b == ABSORBING) {
      if (par->model != 2) { 
         dcornerTBLF(par,grid,data);
         dcornerTBRF(par,grid,data);
         dcornerTTLF(par,grid,data);
         dcornerTTRF(par,grid,data);
         dcornerTBLK(par,grid,data);
         dcornerTBRK(par,grid,data);
         dcornerTTLK(par,grid,data);
         dcornerTTRK(par,grid,data);
         }
      else if (par->model == 2) {
         acornerTBLF(par,grid,data);
         acornerTBRF(par,grid,data);
         acornerTTLF(par,grid,data);
         acornerTTRF(par,grid,data);
         acornerTBLK(par,grid,data);
         acornerTBRK(par,grid,data);
         acornerTTLK(par,grid,data);
         acornerTTRK(par,grid,data);
         }
      }

   if (data->t > 0) getClock(par, grid, 3, 1);

   /* apply variable density grid boundaries (DOESN'T WORK FOR PARALLEL) */
   if (par->g > 1) multiUpdate3D(par, grid, data, 2);
   /*
   for(g = 0; g < par->g; g++) {
      }
   if (grid->bz1 == MULTIGRID) {
      multiGetAboveT(par, grid, data);
      if (grid->bx1 == ABSORBING) multiGetAboveTL(par, grid, data);
      if (grid->bx2 == ABSORBING) multiGetAboveTR(par, grid, data);
      }
   if (grid->bz2 == MULTIGRID) {
      multiGetBelowT(par, grid, data);
      if (grid->bx1 == ABSORBING) multiGetBelowTL(par, grid, data);
      if (grid->bx2 == ABSORBING) multiGetBelowTR(par, grid, data);
      }
   */

   /* apply parallel message passing (DOESN'T WORK FOR VARIABLE DENSITY GRID) */
   if (data->t > 0) getClock(par, grid, 4, 0);
   if (par->nnode > 1) mpUpdate3D(par, grid, data, 2);
   if (data->t > 0) getClock(par, grid, 4, 1);

}

