/*******************************************************************************
   init: initialization functions for 2D/3D finite-difference 

    ================================================= 
   |     Copyright (c) Shawn C. Larsen               |
   |     Seismological Laboratory                    |
   |     California Institute of Technology          |
   |     Pasadena, CA 91125                          |
   |     All Rights Reserved.                        |
    ================================================= 

   usage: initGrid(par, grid, data, image, sac)

   compile: cc -g -c init.c 

   note: input units are km, s, km/s, g/cm3, dyne-cm
   note: used  units are mks (m, s, m/s, kg/m3, N-m)

   author: shawn larsen
   date  : december 1990
   where : caltech
*****************************************************************************/

#include "elas.h"

initGrid(par, grid, data, image, sac)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
Image          *image;
Seismogram     *sac;
{
   int i, j, g, l, m, n, p, t;

   /* initialize grid */
   if (par->model != 2) {
      for(g = 0; g < par->g; g++) {
         for(l = -grid[g].fnt; l < grid[g].l + grid[g].bck; l++) {
            for(m = -grid[g].top; m < grid[g].m + grid[g].bot; m++) {
               for(n = -grid[g].lhs; n < grid[g].n + grid[g].rhs; n++) {
                  data[g].Vx [l][m][n] = 0.;
                  data[g].Vz [l][m][n] = 0.;
                  data[g].Txx[l][m][n] = 0.;
                  data[g].Tzz[l][m][n] = 0.;
                  data[g].Txz[l][m][n] = 0.;
                  if (par->d == 3) {
                     data[g].Vy [l][m][n] = 0.;
                     data[g].Txy[l][m][n] = 0.;
                     data[g].Tyy[l][m][n] = 0.;
                     data[g].Tyz[l][m][n] = 0.;
                     }
                  }
               }
            }
         data[g].t = 0;
         data[g].lmin = 0;
         data[g].lmax = grid[g].l;
         data[g].mmin = 0;
         data[g].mmax = grid[g].m;
         data[g].nmin = 0;
         data[g].nmax = grid[g].n;
         if (grid[g].bz1 == SURFACE || grid[g].bz1 == TOPOGRAPHY) data[g].mmin = 1;
         }
      }
   if (par->model == 2) {
      for(g = 0; g < par->g; g++) {
         for(l = -grid[g].fnt; l < grid[g].l + grid[g].bck; l++) {
            for(m = -grid[g].top; m < grid[g].m + grid[g].bot; m++) {
               for(n = -grid[g].lhs; n < grid[g].n + grid[g].rhs; n++) {
                  data[g].Vx [l][m][n] = 0.;
                  data[g].Vz [l][m][n] = 0.;
                  data[g].T  [l][m][n] = 0.;
                  if (par->d == 3) {
                     data[g].Vy [l][m][n] = 0.;
                     }
                  }
               }
            }
         data[g].t = 0;
         data[g].lmin = 0;
         data[g].lmax = grid[g].l;
         data[g].mmin = 0;
         data[g].mmax = grid[g].m;
         data[g].nmin = 0;
         data[g].nmax = grid[g].n;
         if (grid[g].bz1 == SURFACE || grid[g].bz1 == TOPOGRAPHY) data[g].mmin = 1;
         }
      }

   /* initialize seismograms */
   for(i = 0; i < par->s; i++) {
      /*
      if (sac[i].x >= 0. && sac[i].y >= 0. && sac[i].z >= 0.){
      */
         for(t = 0; t < sac[i].t; t++) {
            if (sac[i].mode == SACS) {
               sac[i].Vx [t] = 0.;
               sac[i].Vy [t] = 0.;
               sac[i].Vz [t] = 0.;
               sac[i].P  [t] = 0.;
               sac[i].S  [t] = 0.;
               }
            else if (sac[i].mode == SACM) {
               sac[i].Mxx[t] = 0.;
               sac[i].Myy[t] = 0.;
               sac[i].Mzz[t] = 0.;
               sac[i].Mxy[t] = 0.;
               sac[i].Mxz[t] = 0.;
               sac[i].Myz[t] = 0.;
               }
            }
         /*
         }
         */
      }

   /* initialize field variables */
   for(i=0; i < par->p; i++) for(j=0; j < image[i].p; j++)image[i].v[j] = 0.;

   /* initialize memory variables */
   if (par->q) {
      for(g = 0; g < par->g; g++) {
         for(l = -grid[g].fnt; l < grid[g].l + grid[g].bck; l++) {
            for(m = -grid[g].top; m < grid[g].m + grid[g].bot; m++) {
               for(n = -grid[g].lhs; n < grid[g].n + grid[g].rhs; n++) {
                  data[g].q1[l][m][n] = 0.;
                  data[g].q2[l][m][n] = 0.;
                  data[g].q3[l][m][n] = 0.;
                  if (par->d == 3) {
                     data[g].q4[l][m][n] = 0.;
                     data[g].q5[l][m][n] = 0.;
                     data[g].q6[l][m][n] = 0.;
                     }
                  }
               }
            }
         }
      }

   /* read dump grid */
   /* restart if requested (no multigrid) */
   if (par->restart) getRestart(par, grid, data);
   
}

/* no multigrid, 3D only */
getRestart(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int l, m, n, l0, m0, n0, l1, m1, n1;
   FILE *file, *getRFile();

   /* initialize coordinates */
   l0 = -grid->fnt;
   m0 = -grid->top;
   n0 = -grid->lhs;
   l1 = grid->l + grid->bck;
   m1 = grid->m + grid->bot;
   n1 = grid->n + grid->rhs;

   /* get restart file to read */
   file = getRFile(par, par->restart);

   /* read data values */
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Vx[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Vy[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Vz[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Txx[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Tyy[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Tzz[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Txy[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Txz[l][m]+n0, sizeof(float), (n1-n0), file);
   for(l = l0; l < l1; l++) for(m = m0; m < m1; m++)
      fread(data->Tyz[l][m]+n0, sizeof(float), (n1-n0), file);

   /* set other parameters and values */
   data->t = par->restart;

   /* close restart file */
   fclose(file);

}

FILE *
getRFile(par, t)
Parameter *par;
int          t;
{
   char filename[82];
   FILE *file;
   sprintf(filename, "%s.%d.%d.E3D_RESTART\0", par->rfile, par->node, t);
   if ((file = fopen(filename, "r")) == NULL) {
      fprintf(stderr, "ERROR: can't open restart file \"%s\"\n", filename);
      exit(1);
      }
   return(file);
}

