/******************************************************************************
   join: 2D finite difference joining program for hybrid implementation

    ================================================================= 
   | (c) Copyright 1992 The Regents of the University of California. |
   |     All Rights Reserved.                                        |
   |                                                                 |
   |     This work was produced under the sponsorship of the         |
   |     United States Department of Energy.                         |
   |     The Government retains certain rights therein.              |
    ================================================================= 

   usage: elas < inputFile > outputFile

   compile: cc -g -o elas elas.c -lm -lget

   note: input units are km, s, km/s; g/cm3; dyne-cm;
   note: used  units are mks (m, s, m/s, kg/m3; N-m)

   author: craig schultz
   date  : december 1995
   where : llnl
*****************************************************************************/

#include "elas.h"

addJoinV(par, grid, data, t)
Parameter        *par;
FdGrid          *grid;
VelocityStress  *data;
int                 t;
{
  float Vx, Vz;
  float VxL0,VxR0,VzL0,VzR0;  
  float VxL,VxR,VzL,VzR;
  float lammu;
  int m,n,np1,mmin;
  float **matrix();
  static FILE *file;

  mmin = data->mmin;
  n = par->xjoin;
  np1 = n + 1;

  if (t == 0) {
    if (par->join == 1) file = fopen("join.data",   "r");
    if (par->join == 2) file = fopen("join.dataFF", "r");
    if (par->join == 3) file = fopen("join.data",   "w");
    if (par->join == 4) file = fopen("join.dataFF", "w");

    /* allocate memory and initialize displacement array */
    if(par->join == 2){
      printf("allocating join memory\n");
      data->Ux = (float **) matrix(mmin-1,data->mmax+1,0,1);
      data->Uz = (float **) matrix(mmin-1,data->mmax+1,0,1);
      for(m = mmin-1; m < data->mmax+1; m++) { 
   data->Ux[m][0]=0.0;    data->Ux[m][1]=0.0;
   data->Uz[m][0]=0.0;   data->Uz[m][1]=0.0;
      }
    }
  }

  /* case: velocities drive grid with hard boundary conditions */
  if (par->join == 1) {
    fread (&par->Vx0 , 4, 1, file);
    fread (&par->Vz0 , 4, 1, file);

    /* update velocity up to boundary, then read in boundary values */
    for(m = mmin; m < data->mmax; m++) { 
      update2Vx2d(par, grid, data->Vx[0], data->Txx[0], data->Txz[0], m, np1);
      update2Vz2d(par, grid, data->Vz[0], data->Txz[0], data->Tzz[0], m, np1); 
      
      fread (&Vx , 4, 1, file);
      fread (&Vz , 4, 1, file);
      data->Vx[0][m][n] = Vx;
      data->Vz[0][m][n] = Vz;
    }
  }

  /* case: forcing functions in velocity and stress drive grid */
  if (par->join == 2) {
    /* read in surface values */
if (mmin != 0) {
}
    fread (&VxL0 , 4, 1, file);
    fread (&VxR0 , 4, 1, file);
    fread (&VzL0 , 4, 1, file);
    fread (&VzR0 , 4, 1, file);

    /* calculate u_i's to get displacements above the surface */
if (mmin != 0) {
}
    data->Ux[mmin-1][0] += par->dt*VxL0;
    data->Ux[mmin-1][1] += par->dt*VxR0;
    data->Uz[mmin-1][0] += par->dt*VzL0;
    data->Uz[mmin-1][1] += par->dt*VzR0;
/*
if (mmin == 0) {
data->Ux[-1][0] = 0.;
data->Ux[-1][1] = 0.;
data->Uz[-1][0] = 0.;
data->Uz[-1][1] = 0.;
}
*/

    for(m = mmin; m < data->mmax; m++) {
      /* read in velocities from a data file */
      fread (&VxL , 4, 1, file);
      fread (&VxR , 4, 1, file);
      fread (&VzL , 4, 1, file);
      fread (&VzR , 4, 1, file);

/*
{
FILE *fd1, *fd2; 
if (m == 15) {
   fd1 = fopen("x15_0", "a");
   fd2 = fopen("z15_0", "a");
   fprintf(fd1, "%e\n", VxL);
   fprintf(fd2, "%e\n", VzR);
   fclose(fd1);
   fclose(fd2);
   }
if (m == 30) {
   fd1 = fopen("x30_0", "a");
   fd2 = fopen("z30_0", "a");
   fprintf(fd1, "%e\n", VxL);
   fprintf(fd2, "%e\n", VzR);
   fclose(fd1);
   fclose(fd2);
   }
if (m == 30) {
   fd1 = fopen("x30_p", "a");
   fd2 = fopen("z30_m", "a");
   fprintf(fd1, "%e\n", VxR);
   fprintf(fd2, "%e\n", VzL);
   fclose(fd1);
   fclose(fd2);
   }
if (m == 45) {
   fd1 = fopen("x45_p", "a");
   fd2 = fopen("z45_m", "a");
   fprintf(fd1, "%e\n", VxR);
   fprintf(fd2, "%e\n", VzL);
   fclose(fd1);
   fclose(fd2);
   }
}
*/

      /* calculate u_i's to be used for t_ij calculation in Vi function*/
      data->Ux[m][0] += par->dt*VxL;
      data->Ux[m][1] += par->dt*VxR;
      data->Uz[m][0] += par->dt*VzL;
      data->Uz[m][1] += par->dt*VzR;

      /* assign proper moments and apply at stress time step              */
      /* note: grid->dx scaling must be used to match amplitudes to input */
      lammu = grid->lamda[0][m][n] + 2.*grid->mu[0][m][n];
      data->Txx[0][m][n] -= par->dt*(         lammu         )*VxL / grid->dx;
      data->Tzz[0][m][n] -= par->dt*(   grid->lamda[0][m][n])*VxL / grid->dx;
      data->Txz[0][m][n] -= par->dt*(   grid->mu   [0][m][n])*VzR / grid->dx; 
    }
  }

  /* output a velocity file which can be read in with par->join == 1 */
  /* Vx and Vz in their grid located spots are printed */
  if (par->join == 3) {
    for(m = mmin-1; m < data->mmax; m++) { 
      Vx  = data->Vx[0][m][n]; 
      Vz  = data->Vz[0][m][n]; 
      fwrite(&Vx , 4, 1, file);
      fwrite(&Vz , 4, 1, file);
    }
  }

  /* output a velocity file which can be read in with par->join == 2 */
  /* Vx(left) and Vx(right) for depth (i-1)*dx                       */
  /* Vz(left) and Vz(right) for depth (i-1)*dx+ 0.5*dx               */
  if (par->join == 4) {
    for(m = mmin-1; m < data->mmax; m++) { 
      VxL  = data->Vx[0][m][n]; 
      VxR  = 0.5 * (data->Vx[0][m][n] + data->Vx[0][m][n+1]); 
      VzL  = 0.5 * (data->Vz[0][m][n-1] + data->Vz[0][m][n]); 
      VzR  = data->Vz[0][m][n]; 
      fwrite(&VxL , 4, 1, file);
      fwrite(&VxR , 4, 1, file);
      fwrite(&VzL , 4, 1, file);
      fwrite(&VzR , 4, 1, file);
    }
  }

  if (t == par->t - 1) {
    fclose(file);
  }
}

addJoinT(par, grid, data, t)
Parameter        *par;
FdGrid          *grid;
VelocityStress  *data;
int                 t;
{
  float TxxJn,TxzJn;
  int m, n;
  int  np1, mmin;

  n = par->xjoin;
  np1 = n + 1;
  mmin = data->mmin;

  if (par->join == 1) {
    /* update stress at proper points adjacent to boundary */
    for(m = mmin; m < data->mmax; m++) {
      update2Txx2d(par, grid, data->Txx[0], data->Vx[0], data->Vz[0], m, np1);
      update2Tzz2d(par, grid, data->Tzz[0], data->Vx[0], data->Vz[0], m, np1);
      update2Txz2d(par, grid, data->Txz[0], data->Vx[0], data->Vz[0], m, np1);
      update2Txx2d(par, grid, data->Txx[0], data->Vx[0], data->Vz[0], m, n);
      update2Tzz2d(par, grid, data->Tzz[0], data->Vx[0], data->Vz[0], m, n);
    }
  }

  if (par->join == 2) {
    for(m = mmin; m < data->mmax; m++) { 
      /* use u_i's to calculate proper Tii's along source array */
      /* this allows for the removal of stresses as input       */
      update2TxxJn(par, grid, data, &TxxJn, m, n);
      update2TxzJn(par, grid, data, &TxzJn, m, n);

      /* apply these stresses as point forces */
      data->Vx[0][m][n] -= par->dt*(1.*grid->irho[0][m][n])*TxxJn / grid->dx;
      data->Vz[0][m][n] -= par->dt*(1.*grid->irho[0][m][n])*TxzJn / grid->dx;
    }
  }

}

surface2VTjn(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
  int n, m, np1, njn;
  njn = par->xjoin;
  np1 = njn + 1;
  m = -1;

  /* velocities should be inputed from the driving function file at njn */
  n = njn;
  data->Vx[0][0][n] = par->Vx0;
  data->Vz[0][0][n] = par->Vz0;

  n = np1;
  update2Vx2d(par, grid, data->Vx[0], data->Txx[0], data->Txz[0], 0, n);
  update2Vz2d(par, grid, data->Vz[0], data->Txz[0], data->Tzz[0], 0, n);

  for(n = njn; n <= np1; n++) {  
    aboveSurfaceVz2d(par, grid, data->Vx[0], data->Vz[0], m, n);
    /* note: actually only needed at np1 */
    aboveSurfaceVx2d(par, grid, data->Vx[0], data->Vz[0], m, n);
  }

}

surface2TTjn(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
  int n, m, np1, njn;
  njn = par->xjoin;
  np1 = njn + 1;
  m = -1;

  for(n = njn; n <= np1; n++) {  
    update2Tzz2d(par, grid, data->Tzz[0], data->Vx[0], data->Vz[0], 0, n);
    update2Txx2d(par, grid, data->Txx[0], data->Vx[0], data->Vz[0], 0, n);
    /* note: actually only needed at np1 */
    update2Txz2d(par, grid, data->Txz[0], data->Vx[0], data->Vz[0], 0, n);
  }
  
  for(n = njn; n <= np1; n++) {
    data->Tzz[0][ 0][n] =               0.;
    data->Tzz[0][-1][n] = -data->Tzz[0][1][n];
    data->Tzz[0][-2][n] = -data->Tzz[0][2][n];
    /* note: Txz's actually only needed at np1 */
    data->Txz[0][-1][n] = -data->Txz[0][0][n];
    data->Txz[0][-2][n] = -data->Txz[0][1][n];
    update2Txx2d(par, grid, data->Txx[0], data->Vx[0], data->Vz[0], m, n);
  }
}


update2TxxJn(par, grid, data, TxxJn, m, n)
Parameter  *par;
FdGrid    *grid;
VelocityStress *data;
float     *TxxJn;
int m, n;
{
   float  lamda, mu;
   lamda    = grid->lamda[0][m][n];
   mu       = grid->mu[0][m][n];

   (*TxxJn) = ((data->Ux[m][1] - data->Ux[m][0])*(lamda + 2.* mu)/(0.5*grid->dx)  +
       (data->Uz[m][1] - data->Uz[m-1][1])*(lamda)/grid->dx);
}

update2TxzJn(par, grid, data, TxzJn, m, n)
Parameter *par;
FdGrid    *grid;
VelocityStress *data;
float *TxzJn;
int m, n;
{
   float mu;
   mu       = grid->mu[0][m][n];
   (*TxzJn) = ((data->Ux[m+1][0] - data->Ux[m][0])/grid->dx +
      (data->Uz[m][1] - data->Uz[m][0])/(0.5*grid->dx))*mu;
           
}



float **matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
   int i;
   float **m;

   m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
   if (!m) printf("allocation failure 1 in matrix()\n");
   m -= nrl;

   for(i=nrl;i<=nrh;i++) {
      m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
      if (!m[i]) printf("allocation failure 2 in matrix()\n");
      m[i] -= ncl;
   }
   return m;
}
