#include "elas.h"
#include <memory.h>


int
mpInit(argc, argv) 
int    *argc;
char ***argv;
{
   mpsc_init(argc, argv);
   return(mynode());
}


mpClose()
{
   mpsc_fini();
}


int
mpNode()
{
   return(mynode());
}


int
mpN()
{
   return(numnodes());
}


int
mpNodeOrder(mode)
int mode;
{
   int i, node, nnode;
   nnode = numnodes();
   node  = mynode();
   if (mode == 0) for(i =    0; i <=  node; i++) gsync();
   if (mode == 1) for(i = node; i <  nnode; i++) gsync();
   return(0);
}


mpGridLimit(par, grid)
Parameter      *par;
FdGrid         *grid;
{
   /* specifically for floats */
   float work;
   gslow (&(grid->minP ), 1, &work);
   gshigh(&(grid->maxP ), 1, &work);
   gslow (&(grid->minS ), 1, &work);
   gshigh(&(grid->maxS ), 1, &work);
   gslow (&(grid->minR ), 1, &work);
   gshigh(&(grid->maxR ), 1, &work);
   gslow (&(grid->minQp), 1, &work);
   gshigh(&(grid->maxQp), 1, &work);
   gslow (&(grid->minQs), 1, &work);
   gshigh(&(grid->maxQs), 1, &work);
   gslow (&(grid->minQw), 1, &work);
   gshigh(&(grid->maxQw), 1, &work);
}


mpWindow(node, w)
int node;
unsigned long *w;
{
   if (node == 0)
      csend(1, (void *) w, sizeof(unsigned long), -1, 0);
   else
      crecv(1, (void *) w, sizeof(unsigned long));
}


mpByteImage(par, grid, pflag, m0, n0, m, n, bytePS, node)
Parameter      *par;
FdGrid         *grid; /*Image          *visual;*/
int             pflag, m0, n0, m, n, node;
{
   float value[5];
   int p;
   unsigned char *bytes;

   gsync();
   if (par->node != 0 && par->node != node) return(0);

   if (par->node == node) {
      value[0] = m;
      value[1] = n;
      value[2] = m0;
      value[3] = n0;
      value[4] = pflag;
      csend(1, (void *) value, 5*sizeof(float), 0, 0);
      }
   else {
      crecv(1, (void *) value, 5*sizeof(float));
      m     = value[0];
      n     = value[1];
      m0    = value[2];
      n0    = value[3];
      pflag = value[4];
      }

   if (par->node == node && pflag) {
      csend(1, (void *) bytePS,             m*n, 0, 0);
      }
   else if (pflag) {
      p = m*n;
      bytes = (unsigned char *) malloc(p*sizeof(unsigned char));
      crecv(1, (void *) bytes, p);
      window(bytes, m0, n0, m, n);
      free(bytes);
      }

}


mpBoundary(par, grid) 
Parameter      *par;
FdGrid         *grid;
{ 
   int nx, ny, nz, node; 
   nx   = par->nx; 
   ny   = par->ny;
   nz   = par->nz;
   node = par->node; 
   grid->px1 = ( node%nx          ==        0) ? (-1) : (node -     1);
   grid->px2 = ( node%nx          == (nx - 1)) ? (-1) : (node +     1);
   grid->pz1 = ((node%(nx*nz))/nx ==        0) ? (-1) : (node -    nx);
   grid->pz2 = ((node%(nx*nz))/nx == (nz - 1)) ? (-1) : (node +    nx);
   grid->py1 = ( node/(nx*nz)     ==        0) ? (-1) : (node - nx*nz);
   grid->py2 = ( node/(nx*nz)     == (ny - 1)) ? (-1) : (node + nx*nz);
} 


mpDecomposition(par, grid)
Parameter      *par;
FdGrid         *grid;
{ 
   int l, m, n, nx, ny, nz, node, nnode, mnode, rnode;

   /* NOTE: THIS ONLY WORKS FOR NON VDENSITY GRID */

   /* get grid decomposition from file */
   if ((int) par->pfile[0]) {mpDecompositionFile(par, grid); return(0);}

   /* initialize */
   nx      = par->nx; 
   ny      = par->ny;
   nz      = par->nz;
   l       = grid->l;
   m       = grid->m;
   n       = grid->n;
   node    = par->node;

   /* x decomposition */
   rnode     = node%nx;
   mnode     = (n%nx);
   nnode     = (n/nx);
   grid->n   = (rnode < mnode) ? (nnode + 1) : (nnode);
   grid->n0  = rnode*nnode;
   grid->n0 += (rnode < mnode) ? (rnode) : (mnode);
   grid->x   = ((float) grid->n - 1)*grid->dx;
   grid->x0  = ((float) grid->n0   )*grid->dx;

   /* z decomposition */
   rnode     = (node%(nx*nz))/nx;
   mnode     = (m%nz);
   nnode     = (m/nz);
   grid->m   = (rnode < mnode) ? (nnode + 1) : (nnode);
   grid->m0  = rnode*nnode;
   grid->m0 += (rnode < mnode) ? (rnode) : (mnode);
   grid->z   = ((float) grid->m - 1)*grid->dz;
   grid->z0  = ((float) grid->m0   )*grid->dz;

   /* y decomposition */
   rnode     = node/(nx*nz);
   mnode     = (l%ny);
   nnode     = (l/ny);
   grid->l   = (rnode < mnode) ? (nnode + 1) : (nnode);
   grid->l0  = rnode*nnode;
   grid->l0 += (rnode < mnode) ? (rnode) : (mnode);
   grid->y   = ((float) grid->l - 1)*grid->dy;
   grid->y0  = ((float) grid->l0   )*grid->dy;

}


mpDecompositionFile(par, grid)
Parameter      *par;
FdGrid         *grid;
{
   int i, node, nodes, rnode, l, m, n, nx, ny, nz;
/*
   int x[MAXNODE], y[MAXNODE], z[MAXNODE];
*/
   int *x, *y, *z;
   char line[258];
   FILE *file;

   /* NOTE: THIS ONLY WORKS FOR NON VDENSITY GRID */

   /* initialize */
   nx = par->nx; ny = par->ny; nz = par->nz;
   x = (int *) malloc(nx*sizeof(int));
   y = (int *) malloc(ny*sizeof(int));
   z = (int *) malloc(nz*sizeof(int));
   nodes = par->nnode;
   if (nx*ny*nz != nodes) {
      fprintf(stderr, "Error: bad number of nodes\n");
      exit(-1);
      }
/*
   if (nx > MAXNODE || ny > MAXNODE || nz > MAXNODE) {
      fprintf(stderr, "Error: MAXNODE exceeded\n");
      exit(-1);
      }
*/
   for(i = 0; i < nx; i++) x[i] = -1;
   for(i = 0; i < ny; i++) y[i] = -1;
   for(i = 0; i < nz; i++) z[i] = -1;
/*
   for(i = 0; i < MAXNODE; i++) {x[i] = -1; y[i] = -1; z[i] = -1;}
*/
   node  = par->node;

   /* open grid decomposition file */
   file = fopen(par->pfile, "r");
   if (file == NULL) {
      fprintf(stderr, "Error: can't open mpp file \"%s\"\n", par->pfile);
      exit(-1);
      }

   /* read grid decomposition file */
   for(i = 0; i < nx; i++) {
      fgets(line, 256, file);
      sscanf(line, "%d", x+i);
      }
   for(i = 0; i < ny; i++) {
      fgets(line, 256, file);
      sscanf(line, "%d", y+i);
      }
   for(i = 0; i < nz; i++) {
      fgets(line, 256, file);
      sscanf(line, "%d", z+i);
      }
   l = 0; m = 0; n = 0;
   for(i = 0; i < nx; i++) n += x[i];
   for(i = 0; i < ny; i++) l += y[i];
   for(i = 0; i < nz; i++) m += z[i];
   if (l != par->l || m != par->m || n != par->n) {
      fprintf(stderr, "Error: bad mpp file sums \"%s\"\n", par->pfile);
      exit(-1);
      }

   /* get x decomposition */
   rnode    = node%nx;
   grid->n  = x[rnode];
   grid->n0 = 0;
   for(i = 0; i < rnode; i++) grid->n0 += x[i];
   grid->x  = ((float) grid->n - 1)*grid->dx;
   grid->x0 = ((float) grid->n0   )*grid->dx;

   /* get z decomposition */
   rnode    = (node%(nx*nz))/nx;
   grid->m  = z[rnode];
   grid->m0 = 0;
   for(i = 0; i < rnode; i++) grid->m0 += z[i];
   grid->z  = ((float) grid->m - 1)*grid->dz;
   grid->z0 = ((float) grid->m0   )*grid->dz;

   /* get y decomposition */
   rnode     = node/(nx*nz);
   grid->l   = y[rnode];
   grid->l0  = 0;
   for(i = 0; i < rnode; i++) grid->l0 += y[i];
   grid->y   = ((float) grid->l - 1)*grid->dy;
   grid->y0  = ((float) grid->l0   )*grid->dy;

   /* close file */
   free(x); free(y); free(z);
   fclose(file);

}


mpUpdate()
{
}


mpUpdate2D(par, grid, data, mode)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
int             mode;
{
   int nx0, nx1, nz0, nz1, m, n, b;

   /* initialize */
   nx0 = grid->px1;
   nx1 = grid->px2;
   nz0 = grid->pz1;
   nz1 = grid->pz2;
   m   = grid->m;
   n   = grid->n;
   b   = 2;

   /* make sure nodes are synchronized */
   gsync();
   
   /* update velocities */
   if (mode == 1) {
      mpGrid2D(data->Vx [0], nx0, nx1, nz0, nz1, m, n, b);
      mpGrid2D(data->Vz [0], nx0, nx1, nz0, nz1, m, n, b);
      }

   /* update stresses */
   if (mode == 2) {
      mpGrid2D(data->Txx[0], nx0, nx1, nz0, nz1, m, n, b);
      mpGrid2D(data->Tzz[0], nx0, nx1, nz0, nz1, m, n, b);
      mpGrid2D(data->Txz[0], nx0, nx1, nz0, nz1, m, n, b);
      }

   /* update medium parameters */
   if (mode == 3) {
      mpGrid2D(grid->lamda[0], nx0, nx1, nz0, nz1, m, n, b);
      mpGrid2D(grid->mu   [0], nx0, nx1, nz0, nz1, m, n, b);
      mpGrid2D(grid->rho  [0], nx0, nx1, nz0, nz1, m, n, b);
      mpGrid2D(grid->irho [0], nx0, nx1, nz0, nz1, m, n, b);
      }

   /* update topography parameters (could do b = 1) */
   if (mode == 4) {
      mpGrid2D(grid->rho  [0], nx0, nx1, nz0, nz1, m, n, b);
      }

}

mpUpdate3D(par, grid, data, mode)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
int             mode;
{
   int nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b;

   /* initialize */
   nx0 = grid->px1;
   nx1 = grid->px2;
   ny0 = grid->py1;
   ny1 = grid->py2;
   nz0 = grid->pz1;
   nz1 = grid->pz2;
   l   = grid->l;
   m   = grid->m;
   n   = grid->n;
   b   = 2;
   
   /* make sure nodes are synchronized */
   gsync();
   
   /* update velocities */
   if (mode == 1) {
      mpGrid3D(data->Vx , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Vy , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Vz , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      }

   /* update stresses */
   if (mode == 2) {
      if (par->model != 2) {
      mpGrid3D(data->Txx, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Tyy, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Tzz, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Txy, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Txz, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(data->Tyz, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      }
      if (par->model == 2) {
      mpGrid3D(data->T  , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      }
      }

   /* update medium parameters */
   if (mode == 3) {
      mpGrid3D(grid->lamda, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      if (par->model != 2)
      mpGrid3D(grid->mu   , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(grid->rho  , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      mpGrid3D(grid->irho , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      }

   /* update topography parameters (could do b = 1 ?) */
   if (mode == 4) {
      mpGrid3D(grid->rho  , nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b);
      }


}

mpGrid2D(array, nx0, nx1, nz0, nz1, m, n, b)
float **array;
int nx0, nx1, nz0, nz1, m, n, b;
{
   float *line1, *line2;
   int size, i, j, p, index;

   /* get side boundary condition number */
   p = b;

   /* update slowest varying sides */
   size = (n + 2*p)*sizeof(float);
   for(j = -b; j < 0; j++)
      mysendrecv((void *) (array[j+m]-p), (void *) (array[j]-p), size,nz1,nz0);
   for(j = m; j < m+b; j++)
      mysendrecv((void *) (array[j-m]-p), (void *) (array[j]-p), size,nz0,nz1);

   /* update fastest varying sides */
   line1 = (float *) malloc((m+2*p)*b*sizeof(float));
   line2 = (float *) malloc((m+2*p)*b*sizeof(float));
   size = (m + 2*p)*b*sizeof(float);
   index = 0;
   if (nx1 >= 0) for(j = -p; j < m+p; j++) for(i = -b; i < 0; i++)
      line1[index++] = array[j][i+n];
   mysendrecv((void *) line1, (void *) line2, size, nx1, nx0);
   index = 0;
   if (nx0 >= 0) for(j = -p; j < m+p; j++) for(i = -b; i < 0; i++) 
      array[j][i] = line2[index++];
   index = 0;
   if (nx0 >= 0)
   for(j = -p; j < m+p; j++) for(i = n; i < n+b; i++)
      line1[index++] = array[j][i-n];
   mysendrecv((void *) line1, (void *) line2, size, nx0, nx1);
   index = 0;
   if (nx1 >= 0) for(j = -p; j < m+p; j++) for(i = n; i < n+b; i++)
      array[j][i] = line2[index++];
   free(line1);
   free(line2);
         
}


mpGrid3D(array, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b)
float ***array;
int nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b;
{
   float *plane1, *plane2;
   int size, i, j, k, p, index;
   int sizen;

   /* get side boundary condition number */
   p = b;

   /* update slowest varying sides */
   plane1 = (float *) malloc((m+2*p)*(n+2*p)*b*sizeof(float));
   plane2 = (float *) malloc((m+2*p)*(n+2*p)*b*sizeof(float));
   size  = (m+2*p)*(n+2*p)*b*sizeof(float);
   sizen = (n+2*p)*sizeof(float);
   index = 0;
   if (ny1 >= 0) 
      for(k = -b; k < 0; k++) for(j = -p; j < m+p; j++) {
         memcpy((char *) (plane1 + index), (char *) (array[k+l][j]-p), sizen);
         index += (n + 2*p);
         }
   mysendrecv((void *) plane1, (void *) plane2, size, ny1, ny0);
   index = 0;
   if (ny0 >= 0)
      for(k = -b; k < 0; k++) for(j = -p; j < m+p; j++) {
         memcpy((char *) (array[k][j]-p), (char *) (plane2 + index), sizen);
         index += (n + 2*p);
         }
   index = 0;
   if (ny0 >= 0)
      for(k = l; k < l+b; k++) for(j = -p; j < m+p; j++) {
         memcpy((char *) (plane1 + index), (char *) (array[k-l][j]-p), sizen);
         index += (n + 2*p);
         }
   mysendrecv((void *) plane1, (void *) plane2, size, ny0, ny1);
   index = 0;
   if (ny1 >= 0)
      for(k = l; k < l+b; k++) for(j = -p; j < m+p; j++) {
         memcpy((char *) (array[k][j]-p), (char *) (plane2 + index), sizen);
         index += (n + 2*p);
         }
   free(plane1);
   free(plane2);

   /* update medium  varying sides */
   plane1 = (float *) malloc((l+2*p)*(n+2*p)*b*sizeof(float));
   plane2 = (float *) malloc((l+2*p)*(n+2*p)*b*sizeof(float));
   size  = (l+2*p)*(n+2*p)*b*sizeof(float);
   sizen = (n+2*p)*sizeof(float);
   index = 0;
   if (nz1 >= 0)
      for(k = -p; k < l+p; k++) for(j = -b; j < 0; j++) {
         memcpy((char *) (plane1 + index), (char *) (array[k][j+m]-p), sizen);
         index += (n + 2*p);
         }
   mysendrecv((void *) plane1, (void *) plane2, size, nz1, nz0);
   index = 0;
   if (nz0 >= 0)
      for(k = -p; k < l+p; k++) for(j = -b; j < 0; j++) {
         memcpy((char *) (array[k][j]-p), (char *) (plane2 + index), sizen);
         index += (n + 2*p);
         }
   index = 0;
   if (nz0 >= 0)
      for(k = -p; k < l+p; k++) for(j = m; j < m+b; j++) {
         memcpy((char *) (plane1 + index), (char *) (array[k][j-m]-p), sizen);
         index += (n + 2*p);
         }
   mysendrecv((void *) plane1, (void *) plane2, size, nz0, nz1);
   index = 0;
   if (nz1 >= 0)
      for(k = -p; k < l+p; k++) for(j = m; j < m+b; j++) {
         memcpy((char *) (array[k][j]-p), (char *) (plane2 + index), sizen);
         index += (n + 2*p);
         }
   free(plane1);
   free(plane2);

   /* update fastest varying sides */
   plane1 = (float *) malloc((l+2*p)*(m+2*p)*b*sizeof(float));
   plane2 = (float *) malloc((l+2*p)*(m+2*p)*b*sizeof(float));
   size  = (l+2*p)*(m+2*p)*b*sizeof(float);
   index = 0;
   if (nx1 >= 0)
      for(k = -p; k < l+p; k++) for(j = -p; j < m+p; j++) for(i = -b; i<0; i++)
         plane1[index++] = array[k][j][i+n];
   mysendrecv((void *) plane1, (void *) plane2, size, nx1, nx0);
   index = 0;
   if (nx0 >= 0)
      for(k = -p; k < l+p; k++) for(j = -p; j < m+p; j++) for(i = -b; i<0; i++) 
         array[k][j][i] = plane2[index++];
   index = 0;
   if (nx0 >= 0)
      for(k = -p; k < l+p; k++) for(j = -p; j < m+p; j++) for(i = n; i<n+b; i++)
         plane1[index++] = array[k][j][i-n];
   mysendrecv((void *) plane1, (void *) plane2, size, nx0, nx1);
   index = 0;
   if (nx1 >= 0)
      for(k = -p; k < l+p; k++) for(j = -p; j < m+p; j++) for(i = n; i<n+b; i++)
         array[k][j][i] = plane2[index++];
   free(plane1);
   free(plane2);

}



mysendrecv(data_send, data_recv, length, to, from)
void *data_send, *data_recv;
int length, to, from;
{
   int msgid;
   if (from >= 0) msgid = irecv(1, data_recv, length);
   gsync();
   if (to   >= 0) csend(1, data_send, length, to, 0);
   if (from >= 0) msgwait(msgid);
   return(0);
}


mpSurfaceUpdate(par, grid, Vx, Vy, Vz, mode)
Parameter  *par;
FdGrid     *grid;
float    ***Vx, ***Vy, ***Vz;
int mode;
{
   float value1[2], value2[2], *line;
   int l, m, n, p, nx0, nx1, ny0, ny1;

   /* update selected Vx for 2D surface condition */
   if (mode == 1 && par->d == 2) {
      /*
      p = sizeof(float);
      if (grid->px1 >= 0) value1[0] = Vx[0][0][0];
      mysendrecv((void *) value1, (void *) value2, p, grid->px1, grid->px2);
      if (grid->px2 >= 0) Vx[0][0][grid->n] = value2[0];
      */
      p = sizeof(float);
      if (grid->px1 >= 0) {
         value1[0] = Vx[0][0][0];
         csend(11, (void *) value1, p, grid->px1, 0);
         }
      if (grid->px2 >= 0) {
         crecv(11, (void *) value2, p);
         Vx[0][0][grid->n] = value2[0];
         }
      }

   /* update selected Vz for 2D surface condition */
   if (mode == 3 && par->d == 2) {
      /*
      p = 2*sizeof(float);
      if (grid->px2 >= 0) value1[0] = Vz[0][ 0][grid->n-1];
      if (grid->px2 >= 0) value1[1] = Vz[0][-1][grid->n-1];
      mysendrecv((void *) value1, (void *) value2, p, grid->px2, grid->px1);
      if (grid->px1 >= 0) Vz[0][ 0][-1] = value2[0];
      if (grid->px1 >= 0) Vz[0][-1][-1] = value2[1];
      */
      p = 2*sizeof(float);
      if (grid->px2 >= 0) {
         value1[0] = Vz[0][ 0][grid->n-1];
         value1[1] = Vz[0][-1][grid->n-1];
         csend(13, (void *) value1, p, grid->px2, 0);
         }
      if (grid->px1 >= 0) {
         crecv(13, (void *) value2, p);
         Vz[0][ 0][-1] = value2[0];
         Vz[0][-1][-1] = value2[1];
         }
      }

   /* update selected Vx for 3D surface condition */
   if (mode == 1 && par->d == 3) {
      p = grid->l*sizeof(float);
      line = (float *) malloc(grid->l*sizeof(float));
      if (grid->px1 >= 0) {
         for(l = 0; l < grid->l; l++) line[l] = Vx[l][0][0];
         csend(11, (void *) line, p, grid->px1, 0);
         }
      if (grid->px2 >= 0) {
         crecv(11, (void *) line, p);
         for(l = 0; l < grid->l; l++) Vx[l][0][grid->n] = line[l];
         }
      free(line);
      }

   /* update selected Vy for 3D surface condition */
   if (mode == 2 && par->d == 3) {
      p = grid->n*sizeof(float);
      line = (float *) malloc(grid->n*sizeof(float));
      if (grid->py2 >= 0) {
         for(n = 0; n < grid->n; n++) line[n] = Vy[grid->l-1][0][n];
         csend(12, (void *) line, p, grid->py2, 0);
         }
      if (grid->py1 >= 0) {
         crecv(12, (void *) line, p);
         for(n = 0; n < grid->n; n++) Vy[-1][0][n] = line[n];
         }
      free(line);
      }

   /* update selected Vz for 3D surface condition */
   if (mode == 3 && par->d == 3) {
      p = 2*grid->l*sizeof(float);
      line = (float *) malloc(2*grid->l*sizeof(float));
      if (grid->px2 >= 0) {
         n = grid->n - 1;
         for(l =       0; l <   grid->l; l++) line[l] = Vz[l        ][ 0][n];
         for(l = grid->l; l < 2*grid->l; l++) line[l] = Vz[l-grid->l][-1][n];
         csend(13, (void *) line, p, grid->px2, 0);
         }
      if (grid->px1 >= 0) {
         crecv(13, (void *) line, p);
         for(l =       0; l <   grid->l; l++) Vz[l        ][ 0][-1] = line[l];
         for(l = grid->l; l < 2*grid->l; l++) Vz[l-grid->l][-1][-1] = line[l];
         }
      free(line);
      p = 2*grid->n*sizeof(float);
      line = (float *) malloc(2*grid->n*sizeof(float));
      if (grid->py1 >= 0) {
         for(n =       0; n <   grid->n; n++) line[n] = Vz[0][ 0][n        ];
         for(n = grid->n; n < 2*grid->n; n++) line[n] = Vz[0][-1][n-grid->n];
         csend(14, (void *) line, p, grid->py1, 0);
         }
      if (grid->py2 >= 0) {
         l = grid->l;
         crecv(14, (void *) line, p);
         for(n =       0; n <   grid->n; n++) Vz[l][ 0][n        ] = line[n];
         for(n = grid->n; n < 2*grid->n; n++) Vz[l][-1][n-grid->n] = line[n];
         }
      free(line);
      }

}






/*
#define L  100
#define M  100
#define N  100
#define B  2

#define TIMESTEP   10
#define SVAR       9
*/

/*
main(argc, argv) 
int    argc;
char **argv;
{
   
   float ***array, sum, sum0, time;
   int b, i, j, k, l, m, n, p1, p2, node, time1, time2;
   int nx0, nx1, ny0, ny1, nz0, nz1;
   int nx, ny, nz;

   mpsc_init();
*/

   /* initialize */
/*
   if (argc != 4) {fprintf(stderr, "arg error\n"); exit(-1);} 
   b    =        B;
   l    =        L;
   m    =        M;
   n    =        N;
   nx   =       atoi(argv[1]);
   ny   =       atoi(argv[2]);
   nz   =       atoi(argv[3]);
   if (numnodes() != nx*ny*nz) {fprintf(stderr, "dimensional error\n"); exit(-1);}
   node = mynode();
*/

   /* determine boundary nodes */
/*
   nx0 = ( node%nx          ==        0) ? (-1) : (node -     1);
   nx1 = ( node%nx          == (nx - 1)) ? (-1) : (node +     1);
   nz0 = ((node%(nx*nz))/nx ==        0) ? (-1) : (node -    nx);
   nz1 = ((node%(nx*nz))/nx == (nz - 1)) ? (-1) : (node +    nx);
   ny0 = ( node/(nx*nz)     ==        0) ? (-1) : (node - nx*nz);
   ny1 = ( node/(nx*nz)     == (ny - 1)) ? (-1) : (node + nx*nz);
*/

   /* print boundary nodes */
   /*
   for(i = 0; i < numnodes(); i++) {
      if (node == i) printf("node = %2d; %3d %3d  %3d %3d  %3d %3d\n", 
                                   node, nx0, nx1, ny0, ny1, nz0, nz1);
      gsync();
      }
   */

   /* allocate memory */
/*
   array = (float ***) malloc((l + 2*b)*sizeof(float **)) + b;
   for(k = -b; k < l + b; k++)
      array[k] = (float **) malloc((m + 2*b)*sizeof(float *)) + b;
   for(k = -b; k < l + b; k++) for(j = -b; j < m + b; j++)
      array[k][j] = (float *) malloc((n + 2*b)*sizeof(float *)) + b;
*/

   /* initialize arrays */
/*
   for(k = -b; k < l+b; k++) for(j = -b; j < m+b; j++) for(i = -b; i < n+b; i++)
      array[k][j][i] = 0.;
   for(k = 0; k < l; k++) for(j = 0; j < m; j++) for(i = 0; i < n; i++)
      array[k][j][i] = (float) node;
*/

   /* update along boundary */
/*
   time1 = mclock();
   for(p1 = 0; p1 < TIMESTEP; p1++) {
      for(p2 = 0; p2 < SVAR; p2++) {
         update3dp(array, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b,nx,ny,nz,node);
         }
      time2 = mclock();
      time  = ((float) (time2 - time1))/1000.;
      time  = time/(p1 + 1);
      if (node == 0) printf("%3d; time = %7.3f\n", p1+1, time);
      }
   time2 = mclock();
   time  = ((float) (time2 - time1))/1000.;
   time  = time/TIMESTEP;
*/
   /*
   if (node == 0) printf("\n");
   for(i = 0; i < numnodes(); i++) {
      if (node == i) printf("node = %3d; time = %7.3f\n", node, time);
      gsync();
      }
   */

   /* print information */
/*
   sum  = 0.;
   sum0 = 0.;
   for(k = -b; k < 0; k++) for(j = 0; j < m; j++) for(i = 0; i < n; i++)
      sum0 += array[k][j][i];
   sum += sum0/(b*m*n);
   sum0 = 0.;
   for(k = l; k < l + b; k++) for(j = 0; j < m; j++) for(i = 0; i < n; i++)
      sum0 += array[k][j][i];
   sum += sum0/(b*m*n);
   sum0 = 0.;
   for(k = 0; k < l; k++) for(j = -b; j < 0; j++) for(i = 0; i < n; i++)
      sum0 += array[k][j][i];
   sum += sum0/(l*b*n);
   sum0 = 0.;
   for(k = 0; k < l; k++) for(j = m; j < m + b; j++) for(i = 0; i < n; i++)
      sum0 += array[k][j][i];
   sum += sum0/(l*b*n);
   sum0 = 0.;
   for(k = 0; k < l; k++) for(j = 0; j < m; j++) for(i = -b; i < 0; i++)
      sum0 += array[k][j][i];
   sum += sum0/(l*m*b);
   sum0 = 0.;
   for(k = 0; k < l; k++) for(j = 0; j < m; j++) for(i = n; i < n + b; i++)
      sum0 += array[k][j][i];
   sum += sum0/(l*m*b);
*/
   /*
   for(i = 0; i < numnodes(); i++) {
      if (node == i) printf("node = %2d; sum = %8.2f\n", node, sum);
      gsync();
      }
   */

   /* free memory */
/*
   for(k = -b; k < l + b; k++) for(j = -b; j < m + b; j++) free(array[k][j]);
   for(k = -b; k < l + b; k++) free(array[k]);
   free(array);

   mpsc_fini();

}
*/


/*
update3d(array, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b, nx, ny, nz, node)
*/

mpGrid3DL(array, nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b, nz, node)
float ***array;
int nx0, nx1, ny0, ny1, nz0, nz1, l, m, n, b;
{
   float *line1, *line2;
   int size, i, j, k;

   /* initialize and allocate memory */
   line1 = (float *) malloc(m*sizeof(float));
   line2 = (float *) malloc(m*sizeof(float));

   /* update slowest varying sides */
   size = 4*n;
   for(k = -b; k < 0; k++) for(j = 0; j < m; j++) 
      mysendrecv((void *) array[k+l][j], (void *) array[k][j], size, ny1, ny0);
   for(k = l; k < l+b; k++) for(j = 0; j < m; j++) 
      mysendrecv((void *) array[k-l][j], (void *) array[k][j], size, ny0, ny1);

   /* update medium  varying sides */
   size = 4*n;
   for(j = -b; j < 0; j++) for(k = 0; k < l; k++) 
      mysendrecv((void *) array[k][j+m], (void *) array[k][j], size, nz1, nz0);
   for(j = m; j < m+b; j++) for(k = 0; k < l; k++) 
      mysendrecv((void *) array[k][j-m], (void *) array[k][j], size, nz0, nz1);

   /* update fastest varying sides (could speed up ?) */
   size = 4*m;
   for(i = -b; i < 0; i++) for(k = 0; k < l; k++) {
      if (nx1 >= 0) for(j = 0; j < m; j++) line1[j] = array[k][j][i+n];
      mysendrecv((void *) line1, (void *) line2, size, nx1, nx0);
      if (nx0 >= 0) for(j = 0; j < m; j++) array[k][j][i] = line2[j];
      }
   for(i = n; i < n+b; i++) for(k = 0; k < l; k++) {
      if (nx0 >= 0) for(j = 0; j < m; j++) line1[j] = array[k][j][i-n];
      mysendrecv((void *) line1, (void *) line2, size, nx0, nx1);
      if (nx1 >= 0) for(j = 0; j < m; j++) array[k][j][i] = line2[j];
      }

   /* free memory */
   free(line1);
   free(line2);

}

