/******************************************************************************
   multi2D: multi (variable density) grid functions for 2D/3D finite-difference

    ================================================= 
   |     Copyright (c) Shawn C. Larsen               |
   |     Seismological Laboratory                    |
   |     California Institute of Technology          |
   |     Pasadena, CA 91125                          |
   |     All Rights Reserved.                        |
    ================================================= 

    =================================================================
   | (c) Copyright 1996 The Regents of the University of California. |
   |     All Rights Reserved.                                        |
   |                                                                 |
   |     This work was produced under the sponsorship of the         |
   |     United States Department of Energy.                         |
   |     The Government retains certain rights therein.              |
    =================================================================

   compile: cc -g -c multi2D.c

   note: input units are km, s, km/s, g/cm3
   note: used  units are mks (m, s, m/s, kg/m3, N-m)
   NOTE: I believe GETABOVE updates need to be called before GETBELOW updates
   NOTE: This must be done in earlier call to multiUpdate

   author: shawn larsen
   date  : march 1992 (from caltech) (reorganized december 1996)
   where : llnl
*****************************************************************************/

#include "elas.h"

multiUpdate2D(par, grid, data, mode)
Parameter       *par;
FdGrid          *grid;
VelocityStress  *data;
{
   int m, n;

   /* initialize */
   m = grid->m;
   n = grid->n;

   /* update velocities */
   if (mode == 1) {
      if (grid->bz1 == MULTIGRID) {
         multiGetAboveV(par, grid, data);
         if (grid->bx1 == ABSORBING) multiGetAboveVL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetAboveVR(par, grid, data);
         }
      if (grid->bz2 == MULTIGRID) {
         multiGetBelowV(par, grid, data);
         if (grid->bx1 == ABSORBING) multiGetBelowVL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetBelowVR(par, grid, data);
         }
      }

   /* update stresses */
   if (mode == 2) {
      if (grid->bz1 == MULTIGRID) {
         multiGetAboveT(par, grid, data);
         if (grid->bx1 == ABSORBING) multiGetAboveTL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetAboveTR(par, grid, data);
         }
      if (grid->bz2 == MULTIGRID) {
         multiGetBelowT(par, grid, data);
         if (grid->bx1 == ABSORBING) multiGetBelowTL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetBelowTR(par, grid, data);
         }
      }

   /* update grid medium */
   if (mode == 3) {
      }

}

/*
multiGridVx2D(array1, array2, m, n)
{
*/
   /* get Vx for points above */ 
/*
   for(n = 0; n < grid[0].n; n++) {
      data[0].Vx[0][-1][n] = data[1].Vx[0][m-2][2*n];
      }
}
*/



/******************************************************************************
   test case for multi grid
******************************************************************************/

multiGetAboveV(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int m, n;

   /* get row number above */
   m = grid[1].m;

   /* get Vx for points above */ 
   for(n = 0; n < grid[0].n; n++) {
      data[0].Vx[0][-1][n] = data[1].Vx[0][m-2][2*n];
      }

   /* get Vz for points above */
   for(n = 0; n < grid[0].n; n++) {
      data[0].Vz[0][-1][n] = (data[1].Vz[0][m-1][2*n] + data[1].Vz[0][m-1][2*n+1] +
                           data[1].Vz[0][m-2][2*n] + data[1].Vz[0][m-2][2*n+1])/4.;
      data[0].Vz[0][-2][n] = (data[1].Vz[0][m-3][2*n] + data[1].Vz[0][m-3][2*n+1] +
                           data[1].Vz[0][m-4][2*n] + data[1].Vz[0][m-4][2*n+1])/4.;
      }

}



multiGetAboveVL(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float VzLast, beta, xS;
   int m, n;

   /* get row number above */
   m = grid[1].m;

   /* get Vx for points above */ 
   n = -1;
   data[0].Vx[0][-1][n] = data[1].Vx[0][m-2][2*n];

   /* get Vz for points above */
   n = -1;
   VzLast = data[0].Vz[0][-1][n];
/*
   beta = sqrt(grid[0].mu[0][m][n]*grid[0].irho[0][m][n]);
*/
   beta = sqrt(grid[1].mu[0][m][n]*grid[1].irho[0][m][n]);
   xS = (grid[0].dx - beta*par->dt)/(grid[0].dx + beta*par->dt);
   data[0].Vz[0][-1][n  ] = (data[1].Vz[0][m-1][2*n] + data[1].Vz[0][m-1][2*n+1] +
                          data[1].Vz[0][m-2][2*n] + data[1].Vz[0][m-2][2*n+1])/4.;
   data[0].Vz[0][-2][n  ] = (data[1].Vz[0][m-3][2*n] + data[1].Vz[0][m-3][2*n+1] +
                          data[1].Vz[0][m-4][2*n] + data[1].Vz[0][m-4][2*n+1])/4.;
   data[0].Vz[0][-1][n-1] = VzLast + xS*(data[0].Vz[0][-1][n-1] - data[0].Vz[0][-1][n]);

}



multiGetAboveVR(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int m, n;

   /* get row number above */
   m = grid[1].m;

   /* get Vx for points above */ 
   n = grid[0].n;
   data[0].Vx[0][-1][n] = data[1].Vx[0][m-2][2*n];

   /* get Vz for points above */
   n = grid[0].n;
   data[0].Vz[0][-1][n] = (data[1].Vz[0][m-1][2*n] + data[1].Vz[0][m-1][2*n+1] +
                        data[1].Vz[0][m-2][2*n] + data[1].Vz[0][m-2][2*n+1])/4.;

}



multiGetBelowV(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float v1, v2, v3, v4;
   int m, n;

   /* get row number */
   m = grid[0].m;

   /* get Vx below */
   for(n = 0; n < grid[-1].n; n++) {
      data[0].Vx[0][m  ][2*n  ] =  data[-1].Vx[0][0][n];
      data[0].Vx[0][m  ][2*n+1] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][0][n+1])/2.;
      data[0].Vx[0][m+1][2*n  ] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][1][n  ])/2.;
      data[0].Vx[0][m+1][2*n+1] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][0][n+1] +
                                data[-1].Vx[0][1][n] + data[-1].Vx[0][1][n+1])/4.;
      }

   /* get Vz below */
   for(n = 0; n < grid[-1].n; n++) {
      v1 = 0.75*data[-1].Vz[0][0][n] + 0.25*data[-1].Vz[0][0][n-1];
      v2 = 0.75*data[-1].Vz[0][1][n] + 0.25*data[-1].Vz[0][1][n-1];
      v3 = 0.75*data[-1].Vz[0][0][n] + 0.25*data[-1].Vz[0][0][n+1];
      v4 = 0.75*data[-1].Vz[0][1][n] + 0.25*data[-1].Vz[0][1][n+1];
      data[0].Vz[0][m][2*n  ] = (2./3.)*v1 + (1./3.)*data[0].Vz[0][m-1][2*n  ];
      data[0].Vz[0][m][2*n+1] = (2./3.)*v3 + (1./3.)*data[0].Vz[0][m-1][2*n+1];
      }

}



multiGetBelowVL(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float v3;
   int m, n;

   /* get row number */
   m = grid[0].m;

   /* get Vx below */
   n = -1;
   data[0].Vx[0][m  ][2*n  ] =  data[-1].Vx[0][0][n];
   data[0].Vx[0][m  ][2*n+1] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][0][n+1])/2.;
   data[0].Vx[0][m+1][2*n+1] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][0][n+1] +
                             data[-1].Vx[0][1][n] + data[-1].Vx[0][1][n+1])/4.;

   /* get Vz below */
   n = -1;
   v3 = 0.75*data[-1].Vz[0][0][n] + 0.25*data[-1].Vz[0][0][n+1];
   data[0].Vz[0][m][2*n+1] = (2./3.)*v3 + (1./3.)*data[0].Vz[0][m-1][2*n+1];

}



multiGetBelowVR(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int m, n;

   /* get row number */
   m = grid[0].m;

   /* get Vx below */
   n = grid[-1].n;
   data[0].Vx[0][m  ][2*n  ] =  data[-1].Vx[0][0][n];
   data[0].Vx[0][m  ][2*n+1] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][0][n+1])/2.;
   data[0].Vx[0][m+1][2*n  ] = (data[-1].Vx[0][0][n] + data[-1].Vx[0][1][n  ])/2.;

   /* get Vz below */
   n = grid[-1].n;

}



multiGetAboveT(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int m, n;

   /* get row number above */
   m = grid[1].m;

   /* get Txx for points above */ 
   for(n = 0; n < grid[0].n; n++) {
      data[0].Txx[0][-1][n] = (data[1].Txx[0][m-2][2*n] + data[1].Txx[0][m-2][2*n+1])/2.;
      }

   /* get Tzz for points above */
   for(n = 0; n < grid[0].n; n++) {
      data[0].Tzz[0][-1][n] = (data[1].Tzz[0][m-2][2*n] + data[1].Tzz[0][m-2][2*n+1])/2.;
      }

   /* get Txz for points above */
   for(n = 0; n < grid[0].n; n++) {
      data[0].Txz[0][-1][n] = (data[1].Txz[0][m-1][2*n] + data[1].Txz[0][m-2][2*n])/2.;
      data[0].Txz[0][-2][n] = (data[1].Txz[0][m-3][2*n] + data[1].Txz[0][m-4][2*n])/2.;
      }

}



multiGetAboveTL(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int m, n;

   /* get row number above */
   m = grid[1].m;

   /* get Txx for points above */ 
   n = -1;
   data[0].Txx[0][-1][n] = (data[1].Txx[0][m-2][2*n] + data[1].Txx[0][m-2][2*n+1])/2.;

   /* get Tzz for points above */
   n = -1;
   data[0].Tzz[0][-1][n] = (data[1].Tzz[0][m-2][2*n] + data[1].Tzz[0][m-2][2*n+1])/2.;

   /* get Txz for points above */
   n = -1;
   data[0].Txz[0][-1][n] = (data[1].Txz[0][m-1][2*n] + data[1].Txz[0][m-2][2*n])/2.;

}



multiGetAboveTR(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int m, n;

   /* get row number above */
   m = grid[1].m;

   /* get Txx for points above */ 

   /* get Tzz for points above */

   /* get Txz for points above */
   n = grid[0].n;
   data[0].Txz[0][-1][n] = (data[1].Txz[0][m-1][2*n] + data[1].Txz[0][m-2][2*n])/2.;

}



multiGetBelowT(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float t1, t2, t3, t4;
   int m, n;

   /* get row number */
   m = grid[0].m;

   /* get Txx below */
   for(n = 0; n < grid[-1].n; n++) {
      t1 = 0.75*data[-1].Txx[0][0][n] + 0.25*data[-1].Txx[0][0][n-1];
      t2 = 0.75*data[-1].Txx[0][1][n] + 0.25*data[-1].Txx[0][1][n-1];
      t3 = 0.75*data[-1].Txx[0][0][n] + 0.25*data[-1].Txx[0][0][n+1];
      t4 = 0.75*data[-1].Txx[0][1][n] + 0.25*data[-1].Txx[0][1][n+1];
      data[0].Txx[0][m  ][2*n  ] = t1;
      data[0].Txx[0][m  ][2*n+1] = t3;
      data[0].Txx[0][m+1][2*n  ] = 0.5*t1 + 0.5*t2;
      data[0].Txx[0][m+1][2*n+1] = 0.5*t3 + 0.5*t4;
      }

   /* get Tzz below */
   for(n = 0; n < grid[-1].n; n++) {
      t1 = 0.75*data[-1].Tzz[0][0][n] + 0.25*data[-1].Tzz[0][0][n-1];
      t2 = 0.75*data[-1].Tzz[0][1][n] + 0.25*data[-1].Tzz[0][1][n-1];
      t3 = 0.75*data[-1].Tzz[0][0][n] + 0.25*data[-1].Tzz[0][0][n+1];
      t4 = 0.75*data[-1].Tzz[0][1][n] + 0.25*data[-1].Tzz[0][1][n+1];
      data[0].Tzz[0][m  ][2*n  ] = t1;
      data[0].Tzz[0][m  ][2*n+1] = t3;
      data[0].Tzz[0][m+1][2*n  ] = 0.5*t1 + 0.5*t2;
      data[0].Tzz[0][m+1][2*n+1] = 0.5*t3 + 0.5*t4;
      }

   /* get Txz below */
   for(n = 0; n < grid[-1].n; n++) {
      t1 =  data[-1].Txz[0][0][n];
      t3 = (data[-1].Txz[0][0][n] + data[-1].Txz[0][0][n+1])/2.;
      data[0].Txz[0][m][2*n  ] = (2./3.)*t1 + (1./3.)*data[0].Txz[0][m-1][2*n  ];
      data[0].Txz[0][m][2*n+1] = (2./3.)*t3 + (1./3.)*data[0].Txz[0][m-1][2*n+1];
      }

}



multiGetBelowTL(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float t1, t2, t3, t4;
   int m, n;

   /* get row number */
   m = grid[0].m;

   /* get Txx below */
   n = -1;

   /* get Tzz below */
   n = -1;
   t1 = 0.75*data[-1].Tzz[0][0][n] + 0.25*data[-1].Tzz[0][0][n-1];
   t3 = 0.75*data[-1].Tzz[0][0][n] + 0.25*data[-1].Tzz[0][0][n+1];
   t4 = 0.75*data[-1].Tzz[0][1][n] + 0.25*data[-1].Tzz[0][1][n+1];
   data[0].Tzz[0][m  ][2*n  ] = t1;
   data[0].Tzz[0][m  ][2*n+1] = t3;
   data[0].Tzz[0][m+1][2*n+1] = 0.5*t3 + 0.5*t4;

   /* get Txz below */
   n = -1;

}



multiGetBelowTR(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float t1;
   int m, n;

   /* get row number */
   m = grid[0].m;

   /* get Txx below */
   n = grid[-1].n;

   /* get Tzz below */
   n = grid[-1].n;
   t1 = 0.75*data[-1].Tzz[0][0][n] + 0.25*data[-1].Tzz[0][0][n-1];
   data[0].Tzz[0][m  ][2*n  ] = t1;


   /* get Txz below */
   n = grid[-1].n;

}




multi1(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float v1, v2, v3, v4, t1, t2, t3, t4;
   int m, n;

   m = grid[0].m;

   /* get Vx (course) */ 
   for(n = 0; n < grid[1].n; n++) {
      data[1].Vx[0][-1][n] = data[0].Vx[0][m-2][2*n];
      }

   /* get Vx (fine) */
   for(n = 0; n < grid[1].n; n++) {           /* check all right grid */
      data[0].Vx[0][m  ][2*n  ] =  data[1].Vx[0][0][n];
      data[0].Vx[0][m  ][2*n+1] = (data[1].Vx[0][0][n] + data[1].Vx[0][0][n+1])/2.;
      data[0].Vx[0][m+1][2*n  ] = (data[1].Vx[0][0][n] + data[1].Vx[0][1][n  ])/2.;
      data[0].Vx[0][m+1][2*n+1] = (data[1].Vx[0][0][n] + data[1].Vx[0][0][n+1] +
                                data[1].Vx[0][1][n] + data[1].Vx[0][1][n+1])/4.;
      }

   /* Vz (course) */
   for(n = 0; n < grid[1].n; n++) {
      data[1].Vz[0][-1][n] = (data[0].Vz[0][m-1][2*n] + data[0].Vz[0][m-1][2*n+1] +
                           data[0].Vz[0][m-2][2*n] + data[0].Vz[0][m-2][2*n+1])/4.;
      data[1].Vz[0][-2][n] = (data[0].Vz[0][m-3][2*n] + data[0].Vz[0][m-3][2*n+1] +
                           data[0].Vz[0][m-4][2*n] + data[0].Vz[0][m-4][2*n+1])/4.;
      }

   /* Vz (fine) */
   for(n = 0; n < grid[1].n; n++) {
      v1 = 0.75*data[1].Vz[0][0][n] + 0.25*data[1].Vz[0][0][n-1];
      v2 = 0.75*data[1].Vz[0][1][n] + 0.25*data[1].Vz[0][1][n-1];
      v3 = 0.75*data[1].Vz[0][0][n] + 0.25*data[1].Vz[0][0][n+1];
      v4 = 0.75*data[1].Vz[0][1][n] + 0.25*data[1].Vz[0][1][n+1];
      data[0].Vz[0][m][2*n  ] = (2./3.)*v1 + (1./3.)*data[0].Vz[0][m-1][2*n  ];
      data[0].Vz[0][m][2*n+1] = (2./3.)*v3 + (1./3.)*data[0].Vz[0][m-1][2*n+1];
      }

}


multi2(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float v1, v2, v3, v4, t1, t2, t3, t4;
   int m, n;

   m = grid[0].m;

   /* Txx (course) */
   for(n = 0; n < grid[1].n; n++) {
      data[1].Txx[0][-1][n] = (data[0].Txx[0][m-2][2*n] + data[0].Txx[0][m-2][2*n+1])/2.;
      }

   /* Txx (fine) */
   for(n = 0; n < grid[1].n; n++) {
      t1 = 0.75*data[1].Txx[0][0][n] + 0.25*data[1].Txx[0][0][n-1];
      t2 = 0.75*data[1].Txx[0][1][n] + 0.25*data[1].Txx[0][1][n-1];
      t3 = 0.75*data[1].Txx[0][0][n] + 0.25*data[1].Txx[0][0][n+1];
      t4 = 0.75*data[1].Txx[0][1][n] + 0.25*data[1].Txx[0][1][n+1];
      data[0].Txx[0][m  ][2*n  ] = t1;
      data[0].Txx[0][m  ][2*n+1] = t3;
      data[0].Txx[0][m+1][2*n  ] = 0.5*t1 + 0.5*t2;
      data[0].Txx[0][m+1][2*n+1] = 0.5*t3 + 0.5*t4;
      }

   /* Tzz (course) */
   for(n = 0; n < grid[1].n; n++) {
      data[1].Tzz[0][-1][n] = (data[0].Tzz[0][m-2][2*n] + data[0].Tzz[0][m-2][2*n+1])/2.;
      }

   /* Tzz (fine) */
   for(n = 0; n < grid[1].n; n++) {
      t1 = 0.75*data[1].Tzz[0][0][n] + 0.25*data[1].Tzz[0][0][n-1];
      t2 = 0.75*data[1].Tzz[0][1][n] + 0.25*data[1].Tzz[0][1][n-1];
      t3 = 0.75*data[1].Tzz[0][0][n] + 0.25*data[1].Tzz[0][0][n+1];
      t4 = 0.75*data[1].Tzz[0][1][n] + 0.25*data[1].Tzz[0][1][n+1];
      data[0].Tzz[0][m  ][2*n  ] = t1;
      data[0].Tzz[0][m  ][2*n+1] = t3;
      data[0].Tzz[0][m+1][2*n  ] = 0.5*t1 + 0.5*t2;
      data[0].Tzz[0][m+1][2*n+1] = 0.5*t3 + 0.5*t4;
      }

   /* Txz (course) */
   for(n = 0; n < grid[1].n; n++) {
      data[1].Txz[0][-1][n] = (data[0].Txz[0][m-1][2*n] + data[0].Txz[0][m-2][2*n])/2.;
      data[1].Txz[0][-2][n] = (data[0].Txz[0][m-3][2*n] + data[0].Txz[0][m-4][2*n])/2.;
      }

   /* Txz (fine) */
   for(n = 0; n < grid[1].n; n++) {
      t1 =  data[1].Txz[0][0][n];
      t3 = (data[1].Txz[0][0][n] + data[1].Txz[0][0][n+1])/2.;
      data[0].Txz[0][m][2*n  ] = (2./3.)*t1 + (1./3.)*data[0].Txz[0][m-1][2*n  ];
      data[0].Txz[0][m][2*n+1] = (2./3.)*t3 + (1./3.)*data[0].Txz[0][m-1][2*n+1];
      }

}
