/******************************************************************
* FILE          : multi.h
* PROGRAMMED BY : Ted Bakowsky
* DATE          : April 1996
* MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted
*                 Bakowsky
******************************************************************/

/******************************************************************************
   multi3D: multi (variable density) grid functions for 2D/3D finite-difference

   compile: cc -g -c multi3D.c

   note: input units are km, s, km/s, g/cm3
   note: used  units are mks (m, s, m/s, kg/m3, N-m)
   NOTE: I believe GETABOVE updates need to be called before GETBELOW updates
   NOTE: This must be done in earlier call to multiUpdate

   author: shawn larsen
   date  : january 1997 (driving routine)
   where : llnl
*****************************************************************************/

#include "elas.h"

#define RC2  0.50
#define RC4  0.25
#define RC8  0.125
#define RC16 0.0625
#define RC64 0.015625

multiUpdate3D(par, grid, data, mode)
Parameter       *par;
FdGrid          *grid;
VelocityStress  *data;
{
   int m, n;

   /* initialize */
   m = grid->m;
   n = grid->n;

   /* update velocities */
   if (mode == 1) {
      if (grid->bz1 == MULTIGRID) {
         multiGetAboveV3D(par, grid, data);
         /*
         if (grid->bx1 == ABSORBING) multiGetAboveVL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetAboveVR(par, grid, data);
         */
         }
      if (grid->bz2 == MULTIGRID) {
         multiGetBelowV3D(par, grid, data);
         /*
         if (grid->bx1 == ABSORBING) multiGetBelowVL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetBelowVR(par, grid, data);
         */
         }
      }

   /* update stresses */
   if (mode == 2) {
      if (grid->bz1 == MULTIGRID) {
         multiGetAboveT3D(par, grid, data);
         /*
         if (grid->bx1 == ABSORBING) multiGetAboveTL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetAboveTR(par, grid, data);
         */
         }
      if (grid->bz2 == MULTIGRID) {
         multiGetBelowT3D(par, grid, data);
         /*
         if (grid->bx1 == ABSORBING) multiGetBelowTL(par, grid, data);
         if (grid->bx2 == ABSORBING) multiGetBelowTR(par, grid, data);
         */
         }
      }

   /* update grid medium */
   if (mode == 3) {
      }

}



 /*
  *****************************************************************
  * FILE          : new_multiGetAboveV3D.c
  * PROGRAMMED BY : Ted Bakowsky
  * DATE          : March 1996
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */


 static
   multiGetAboveVx() ;

 static
   multiGetAboveVy() ;

 static
   multiGetAboveVz() ;



 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveVx
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveVx (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  )
 int             m_max_bel ;
 int             l_max_abv ;
 int             n_max_abv ;
 VelocityStress *vel_str_bel ;
 VelocityStress *vel_str_abv ;
 {
  float ***vx_bel,
        ***vx_abv ;

  int      l_abv,
           n_abv,
           m_bel_m2,
           n_bel,
           l_bel ;

  vx_bel = vel_str_bel->Vx ;
  vx_abv = vel_str_abv->Vx ;

  m_bel_m2 = m_max_bel - 2 ;
  l_bel    = 0 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel = 0 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        vx_abv[l_abv][-1][n_abv] = vx_bel[l_bel][m_bel_m2][n_bel] ;

        n_bel += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel += 2 ;

    } /* end of for ( l_abv = 0; ... */

 } /* end of function: multiGetAboveVx() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveVy
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveVy (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  float ***vy_bel,
        ***vy_abv ;

  float    vy1, vy2, vy3, vy4 ;

  int      l_abv,
           n_abv,
           n_bel,
           n_bel_p1,
           l_bel,
           l_bel_p1,
           m_bel_m2
            ;

  vy_bel = vel_str_bel->Vy ;
  vy_abv = vel_str_abv->Vy ;

  m_bel_m2 = m_max_bel - 2 ;

  l_bel    = 0 ;
  l_bel_p1 = 1 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel    = 0 ;
     n_bel_p1 = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        vy1 = vy_bel[l_bel   ][m_bel_m2][n_bel   ] ;
        vy2 = vy_bel[l_bel   ][m_bel_m2][n_bel_p1] ;
        vy3 = vy_bel[l_bel_p1][m_bel_m2][n_bel_p1] ;
        vy4 = vy_bel[l_bel_p1][m_bel_m2][n_bel   ] ;

        vy_abv[l_abv][-1][n_abv] = RC4 * ( vy1 + vy2 + vy3 + vy4 ) ;

        n_bel    += 2 ;
        n_bel_p1 += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel    += 2 ;
     l_bel_p1 += 2 ;

    } /* end of for ( l_abv = 0; ... */

 } /* end of function: multiGetAboveVy() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveVz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveVz (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  float          ***vz_bel,
                 ***vz_abv,

                    vz_bel_1,
                    vz_bel_2,
                    vz_bel_3,
                    vz_bel_4
                      ;

  int               n_abv, l_abv,
                    m_bel_m1,
                    m_bel_m2,
                    m_bel_m3,
                    m_bel_m4,
                    n_bel,
                    n_bel_p1,
                    l_bel
                     ;


  vz_bel      = vel_str_bel->Vz ;
  vz_abv      = vel_str_abv->Vz ;

  m_bel_m1    = m_max_bel - 1 ;
  m_bel_m2    = m_max_bel - 2 ;
  m_bel_m3    = m_max_bel - 3 ;
  m_bel_m4    = m_max_bel - 4 ;

  l_bel       = 0 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel    = 0 ;
     n_bel_p1 = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        /*
         * Interpole Vz
         */

        vz_bel_1 = vz_bel[l_bel][m_bel_m1][n_bel   ] ;
        vz_bel_2 = vz_bel[l_bel][m_bel_m1][n_bel_p1] ;
        vz_bel_3 = vz_bel[l_bel][m_bel_m2][n_bel_p1] ;
        vz_bel_4 = vz_bel[l_bel][m_bel_m2][n_bel   ] ;

        vz_abv[l_abv][-1][n_abv] = RC4 * (
                                          vz_bel_1 +
                                          vz_bel_2 +
                                          vz_bel_3 +
                                          vz_bel_4
                                         ) ;


        vz_bel_1 = vz_bel[l_bel][m_bel_m3][n_bel   ] ;
        vz_bel_2 = vz_bel[l_bel][m_bel_m3][n_bel_p1] ;
        vz_bel_3 = vz_bel[l_bel][m_bel_m4][n_bel_p1] ;
        vz_bel_4 = vz_bel[l_bel][m_bel_m4][n_bel   ] ;

        vz_abv[l_abv][-2][n_abv] = RC4 * (
                                          vz_bel_1 +
                                          vz_bel_2 +
                                          vz_bel_3 +
                                          vz_bel_4
                                         ) ;

        n_bel    += 2 ;
        n_bel_p1 += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel+= 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of function: static multiGetAboveVz() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveV3D
  * STORAGE CLASS : extern
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 multiGetAboveV3D(
                  par,
                  grid,
                  data
                 )
 Parameter      *par ;
 FdGrid         *grid ;
 VelocityStress *data ;
 {
  FdGrid           *grid_abv,     /* coarse grid                       */
                   *grid_bel ;    /* fine grid                         */

  VelocityStress   *vel_str_abv,  /* coarse grid velocity-stress array */
                   *vel_str_bel ; /* fine grid velocity-stress array   */

  int               m_max_bel,
                    n_max_abv,
                    l_max_abv
                     ;


  grid_bel    = &grid[1] ;
  grid_abv    = &grid[0] ;

  m_max_bel   = grid_bel->m ;
  l_max_abv   = grid_abv->l ;
  n_max_abv   = grid_abv->n ;

  vel_str_bel = &data[1] ;
  vel_str_abv = &data[0] ;

  multiGetAboveVx (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  ) ;

  multiGetAboveVy (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  ) ;

  multiGetAboveVz (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  ) ;

 } /* end of function: multiGetAboveV3D() ... */


/***********************************************************************/



 /*
  *****************************************************************
  * FILE          : new_multiGetBelowV3D
  * PROGRAMMED BY : Ted Bakowsky
  * DATE          : March 1996
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted
  *                 Bakowsky
  *****************************************************************
  */



 static
  multiGetBelowVx() ;

 static
  multiGetBelowVy() ;

 static
  multiGetBelowVz() ;



 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowVx
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowVx (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Vx in overlap area
   */

  float ***vx_bel,
        ***vx_abv ;

  float    vx1l,    vx2l,  vx3l,  vx4l,
           vx1lp,   vx2lp, vx3lp, vx4lp,
           vx_1234, vx ;

  int      l_bel, l_bel_p,
           m_bel, m_bel_p,
           n_bel, n_bel_p,
           l_abv, l_abv_p,
           n_abv, n_abv_m, n_abv_p
            ;

  m_bel   = m_max_bel ;
  m_bel_p = m_max_bel + 1 ;

  vx_bel  = vel_str_bel->Vx ;
  vx_abv  = vel_str_abv->Vx ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;
   
  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     l_abv_p = l_abv + 1 ;

     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_m = n_abv - 1 ;
        n_abv_p = n_abv + 1 ;

        vx1l  = vx_abv [l_abv  ] [1] [n_abv  ] ;
        vx2l  = vx_abv [l_abv  ] [1] [n_abv_p] ;
        vx3l  = vx_abv [l_abv  ] [0] [n_abv_p] ;
        vx4l  = vx_abv [l_abv  ] [0] [n_abv  ] ;

        vx1lp = vx_abv [l_abv_p] [1] [n_abv  ] ;
        vx2lp = vx_abv [l_abv_p] [1] [n_abv_p] ;
        vx3lp = vx_abv [l_abv_p] [0] [n_abv_p] ;
        vx4lp = vx_abv [l_abv_p] [0] [n_abv  ] ;

        /*
         * Get Vx [l_bel] [m_bel] [n_bel]
         */

        vx_bel [l_bel][m_bel][n_bel] = vx4l ;


        /*
         * Get Vx [l_bel+1] [m_bel] [n_bel]
         */

        vx = 0.5 * ( vx4l + vx4lp ) ;
        vx_bel [l_bel_p][m_bel][n_bel] = vx ;


        /*
         * Get Vx [l_bel] [m_bel] [n_bel+1]
         */

        vx = 0.5 * ( vx3l + vx4l ) ;
        vx_bel [l_bel][m_bel][n_bel_p] = vx ;


        /*
         * Get Vx [l_bel+1] [m_bel] [n_bel+1]
         */

        vx = 0.25 * ( vx3l + vx4l + vx3lp + vx4lp ) ;
        vx_bel [l_bel_p] [m_bel] [n_bel_p] = vx ;


        /*
         ***** Now do the (m_bel+1) plane (4-points). *****
         */

        /*
         * Get Vx [l_bel] [m_bel+1] [n_bel]
         */

        vx = 0.5 * ( vx1l + vx4l ) ;
        vx_bel [l_bel] [m_bel_p] [n_bel] = vx ;


        /*
         * Get Vx [l_bel+1] [m_bel+1] [n_bel]
         */

        vx = 0.25 * ( vx1l + vx4l + vx1lp + vx4lp ) ;
        vx_bel [l_bel_p] [m_bel_p] [n_bel] = vx ;


        /*
         * Get Vx [l_bel] [m_bel+1] [n_bel+1]
         */

        vx_1234 = vx1l + vx2l + vx3l + vx4l  ;
        vx      = 0.25 * vx_1234 ;
        vx_bel [l_bel] [m_bel_p] [n_bel_p] = vx ;


        /*
         * Get Vx [l_bel+1] [m_bel+1] [n_bel+1]
         */

        vx = 0.125 * ( vx_1234 + vx1lp + vx2lp + vx3lp + vx4lp ) ;       
        vx_bel [l_bel_p] [m_bel_p] [n_bel_p] = vx ;


        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function void multiGetBelowVx ( ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowVy
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowVy (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  )
 int             m_max_bel ;
 int             l_max_abv ;
 int             n_max_abv ;
 VelocityStress *vel_str_bel ;
 VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Vy in overlap area
   */

  float ***vy_bel,
        ***vy_abv ;

  int      l_bel, l_bel_p,
           m_bel, m_bel_p,
           n_bel, n_bel_p,
           l_abv, l_abv_m, l_abv_p,
           n_abv, n_abv_m, n_abv_p
            ;

  float    vy4lm, vy5lm, vy6lm,
           vy4l,  vy5l,  vy6l,
           vy4lp, vy5lp, vy6lp,

           vy1lm, vy2lm, vy3lm,
           vy1l,  vy2l,  vy3l,
           vy1lp, vy2lp, vy3lp,

           vy ;


  m_bel   = m_max_bel ;
  m_bel_p = m_max_bel + 1 ;

  vy_bel = vel_str_bel->Vy ;
  vy_abv = vel_str_abv->Vy ;


  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;
   
  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     l_abv_m = l_abv - 1 ;
     l_abv_p = l_abv + 1 ;

     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_m = n_abv - 1 ;
        n_abv_p = n_abv + 1 ;

        vy1lm = vy_abv [l_abv_m] [ 1] [n_abv_m] ;
        vy2lm = vy_abv [l_abv_m] [ 1] [n_abv  ] ;
        vy3lm = vy_abv [l_abv_m] [ 1] [n_abv_p] ;
        vy4lm = vy_abv [l_abv_m] [ 0] [n_abv_m] ;
        vy5lm = vy_abv [l_abv_m] [ 0] [n_abv  ] ;
        vy6lm = vy_abv [l_abv_m] [ 0] [n_abv_p] ;

        vy1l  = vy_abv [l_abv  ] [ 1] [n_abv_m] ;
        vy2l  = vy_abv [l_abv  ] [ 1] [n_abv  ] ;
        vy3l  = vy_abv [l_abv  ] [ 1] [n_abv_p] ;
        vy4l  = vy_abv [l_abv  ] [ 0] [n_abv_m] ;
        vy5l  = vy_abv [l_abv  ] [ 0] [n_abv  ] ;
        vy6l  = vy_abv [l_abv  ] [ 0] [n_abv_p] ;

        vy1lp = vy_abv [l_abv_p] [ 1] [n_abv_m] ;
        vy2lp = vy_abv [l_abv_p] [ 1] [n_abv  ] ;
        vy3lp = vy_abv [l_abv_p] [ 1] [n_abv_p] ;
        vy4lp = vy_abv [l_abv_p] [ 0] [n_abv_m] ;
        vy5lp = vy_abv [l_abv_p] [ 0] [n_abv  ] ;
        vy6lp = vy_abv [l_abv_p] [ 0] [n_abv_p] ;


        /*
         * Get Vy [l_bel] [m_bel] [n_bel]     (1)
         */

        vy  = (
                 vy5lm * 3.0
               + vy4lm
               + vy5l  * 9.0
               + vy4l  * 3.0

              ) * RC16 ;

        vy_bel [l_bel] [m_bel] [n_bel] = vy ;


        /*
         * Get Vy [l_bel+1] [m_bel] [n_bel]     (2)
         */

        vy  = (
                 vy5l  * 9.0
               + vy4l  * 3.0 
               + vy5lp * 3.0
               + vy4lp

              ) * RC16 ;

        vy_bel [l_bel_p] [m_bel] [n_bel] = vy ;


        /*
         * Get Vy [l_bel] [m_bel] [n_bel+1]    (3)
         */

        vy  = (
                  vy6lm
                + vy5lm * 3.0
                + vy6l  * 3.0
                + vy5l  * 9.0

              ) * RC16 ;


        vy_bel [l_bel] [m_bel] [n_bel_p] = vy ;


        /*
         * Get Vy [l_bel+1] [m_bel] [n_bel+1]     (4)
         */

        vy  = (
                  vy6l  * 3.0
                + vy5l  * 9.0
                + vy6lp
                + vy5lp * 3.0

              ) * RC16 ;

        vy_bel [l_bel_p] [m_bel] [n_bel_p] = vy ;


        /*
         * Get Vy [l_bel] [m_bel+1] [n_bel]     (5)
         */


        vy  = (
                  vy1lm *  2.0 
                + vy2lm *  6.0
                + vy5lm *  6.0
                + vy4lm *  2.0

                + vy1l  *  6.0
                + vy2l  * 18.0
                + vy5l  * 18.0
                + vy4l  *  6.0

              ) * RC64 ;

        vy_bel [l_bel] [m_bel_p] [n_bel] = vy ; 


        /*
         * Get Vy [l_bel+1] [m_bel+1] [n_bel]     (6)
         */

        vy  = (
                  vy1l  *  6.0
                + vy2l  * 18.0
                + vy5l  * 18.0
                + vy4l  *  6.0

                + vy1lp *  2.0
                + vy2lp *  6.0
                + vy5lp *  6.0
                + vy4lp *  2.0

              ) * RC64 ;

        vy_bel [l_bel_p] [m_bel_p] [n_bel] = vy ;


        /*
         * Get Vy [l_bel] [m_bel+1] [n_bel+1]     (7)
         */

        vy  = (
                  vy2lm *  6.0
                + vy3lm *  2.0
                + vy6lm *  2.0
                + vy5lm *  6.0

                + vy2l  * 18.0
                + vy3l  *  6.0
                + vy6l  *  6.0
                + vy5l  * 18.0

              ) * RC64 ;

        vy_bel [l_bel] [m_bel_p] [n_bel_p] = vy ;


        /*
         * Get Vy [l_bel+1] [m_bel+1] [n_bel+1]     (8)
         */

        vy  = (
                  vy2l  * 18.0
                + vy3l  *  6.0
                + vy6l  *  6.0
                + vy5l  * 18.0

                + vy2lp *  6.0
                + vy3lp *  2.0
                + vy6lp *  2.0
                + vy5lp *  6.0

              ) * RC64 ;

        vy_bel [l_bel_p] [m_bel_p] [n_bel_p] = vy ;

       
        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function multiGetBelowVy() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowVz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowVz (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Vz in overlap region
   */

  float ***vz_bel,
        ***vz_abv ;

  float    vz1l,  vz2l,  vz3l,  vz4l,  vz5l,  vz6l,
           vz1lp, vz2lp, vz3lp, vz4lp, vz5lp, vz6lp,
           vz ;

  int      l_bel, l_bel_p,
           n_bel, n_bel_p,
           l_abv, l_abv_p,
           n_abv, n_abv_m, n_abv_p,
           m_bel, m_bel_p
            ;


  m_bel   = m_max_bel ;
  m_bel_p = m_max_bel + 1 ;

  vz_bel = vel_str_bel->Vz ;
  vz_abv = vel_str_abv->Vz ;


  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;
   
  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (y-direction)
      */

     l_abv_p = l_abv + 1 ;

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_m = n_abv - 1 ;
        n_abv_p = n_abv + 1 ;

        vz1l  = vz_abv [l_abv  ] [ 0] [n_abv_m] ;
        vz2l  = vz_abv [l_abv  ] [ 0] [n_abv  ] ;
        vz3l  = vz_abv [l_abv  ] [ 0] [n_abv_p] ;
        vz4l  = vz_abv [l_abv  ] [-1] [n_abv_m] ;
        vz5l  = vz_abv [l_abv  ] [-1] [n_abv  ] ;
        vz6l  = vz_abv [l_abv  ] [-1] [n_abv_p] ;

        vz1lp = vz_abv [l_abv_p] [ 0] [n_abv_m] ;
        vz2lp = vz_abv [l_abv_p] [ 0] [n_abv  ] ;
        vz3lp = vz_abv [l_abv_p] [ 0] [n_abv_p] ;
        vz4lp = vz_abv [l_abv_p] [-1] [n_abv_m] ;
        vz5lp = vz_abv [l_abv_p] [-1] [n_abv  ] ;
        vz6lp = vz_abv [l_abv_p] [-1] [n_abv_p] ;


        /*
         * Get vz_bel [l_bel] [m_bel] [n_bel]
         */

        vz  = (
                 vz1l * 3.0
               + vz2l * 9.0
               + vz5l * 3.0
               + vz4l

              ) * RC16 ;

        vz_bel [l_bel][m_bel][n_bel] = vz ;


        /*
         *  Get Vz [l_bel+1] [m_bel] [n_bel] ;
         */

        vz  = (
                 vz1l  *  6.0
               + vz2l  * 18.0
               + vz5l  *  6.0
               + vz4l  *  2.0

               + vz1lp *  6.0
               + vz2lp * 18.0
               + vz5lp *  6.0
               + vz4lp *  2.0

              ) * RC64 ;


        vz_bel [l_bel_p] [m_bel] [n_bel] = vz ;


        /*
         *  Get Vz [l_bel] [m_bel] [n_bel+1] ;
         */

        vz = (
                vz2l * 9.0
              + vz3l * 3.0
              + vz6l
              + vz5l * 3.0

             ) * RC16 ;

        vz_bel [l_bel][m_bel][n_bel_p] = vz ;


        /*
         *  Get Vz [l_bel+1] [m_bel] [n_bel+1] ;
         */

        vz = (
                vz2l  * 18.0
              + vz3l  *  6.0
              + vz6l  *  2.0
              + vz5l  *  6.0

              + vz2lp * 18.0
              + vz3lp *  6.0
              + vz6lp *  2.0
              + vz5lp *  6.0

             ) * RC64 ;

        vz_bel [l_bel_p][m_bel][n_bel_p] = vz ;


        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function multiGetBelowVz() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowV3D
  * STORAGE CLASS : extern
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted
  *                 Bakowsky
  *****************************************************************
  */

 multiGetBelowV3D (
                   par,
                   grid,
                   data
                  )
 Parameter      *par ;
 FdGrid         *grid ;
 VelocityStress *data ;
 {
  FdGrid         *grid_abv,         /* coarse grid                       */
                 *grid_bel ;        /* fine   grid                       */

  VelocityStress *vel_str_abv,      /* coarse grid velocity-stress array */
                 *vel_str_bel ;     /* fine   grid velocity-stress array */

  int             m_max_bel,
                  n_max_abv,
                  l_max_abv
                    ;

  grid_bel = &grid[ 0] ;
  grid_abv = &grid[-1] ;

  m_max_bel = grid_bel->m ;

  l_max_abv = grid_abv->l ;
  n_max_abv = grid_abv->n ;

  vel_str_bel = &data[ 0] ;
  vel_str_abv = &data[-1] ;

  multiGetBelowVx (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  ) ;

  multiGetBelowVy (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  ) ;

  multiGetBelowVz (
                   m_max_bel,
                   l_max_abv,
                   n_max_abv,
                   vel_str_bel,
                   vel_str_abv
                  ) ;
                  

 } /* end of function: multiGetBelowV3D() ... */




/***********************************************************************/




 /*
  *****************************************************************
  * FILE          : multiGetAboveT3D.c
  * PROGRAMMED BY : Ted Bakowsky
  * DATE          : March 1996
  * MODIFIED      : Converted to pre-ANSI C, July 1996 by 
  *               : Ted Bakowsky
  *****************************************************************
  */



 static
   multiGetAboveTxz() ;

 static
   multiGetAboveTxxTyyTzz() ;

 static
   multiGetAboveTyz() ;

 static
   multiGetAboveTxy() ;




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveTxz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996 by 
  *                 Ted Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveTxz (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  float ***txz_bel,
        ***txz_abv,
           txz1, txz2 ;

  int      l_abv,
           n_abv,
           m_bel_m1,
           m_bel_m2,
           m_bel_m3,
           m_bel_m4,
           n_bel,
           l_bel ;

  txz_bel = vel_str_bel->Txz ;
  txz_abv = vel_str_abv->Txz ;

  m_bel_m1 = m_max_bel - 1 ;
  m_bel_m2 = m_max_bel - 2 ;
  m_bel_m3 = m_max_bel - 3 ;
  m_bel_m4 = m_max_bel - 4 ;

  l_bel    = 0 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel = 0 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        txz1 = txz_bel [l_bel][m_bel_m1][n_bel] ;
        txz2 = txz_bel [l_bel][m_bel_m2][n_bel] ;

        txz_abv[l_abv][-1][n_abv] = RC2 * ( txz1 + txz2 ) ;

        txz1 = txz_bel [l_bel][m_bel_m3][n_bel] ;
        txz2 = txz_bel [l_bel][m_bel_m4][n_bel] ;

        txz_abv[l_abv][-2][n_abv] = RC2 * ( txz1 + txz2 ) ;

        n_bel += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel += 2 ;

    } /* end of for ( l_abv = 0; ... */

 } /* end of function: multiGetAboveTxz () ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveTxxTyyTzz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996 by 
  *                 Ted Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveTxxTyyTzz (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  float ***txx_bel,
        ***txx_abv,
        ***tyy_bel,
        ***tyy_abv,
        ***tzz_bel,
        ***tzz_abv,
           txx1, txx2,
           tyy1, tyy2,
           tzz1, tzz2 ;

  int      l_abv,
           n_abv,
           m_bel_m2,
           n_bel,
           n_bel_p1,
           l_bel ;

  txx_bel = vel_str_bel->Txx ;
  txx_abv = vel_str_abv->Txx ;

  tyy_bel = vel_str_bel->Tyy ;
  tyy_abv = vel_str_abv->Tyy ;

  tzz_bel = vel_str_bel->Tzz ;
  tzz_abv = vel_str_abv->Tzz ;

  m_bel_m2 = m_max_bel - 2 ;
  l_bel    = 0 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel    = 0 ;
     n_bel_p1 = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        txx1 = txx_bel [l_bel][m_bel_m2][n_bel   ] ;
        tyy1 = tyy_bel [l_bel][m_bel_m2][n_bel   ] ;
        tzz1 = tzz_bel [l_bel][m_bel_m2][n_bel   ] ;

        txx2 = txx_bel [l_bel][m_bel_m2][n_bel_p1] ;
        tyy2 = tyy_bel [l_bel][m_bel_m2][n_bel_p1] ;
        tzz2 = tzz_bel [l_bel][m_bel_m2][n_bel_p1] ;

        txx_abv [l_abv][-1][n_abv] = RC2 * ( txx1 + txx2 ) ;
        tyy_abv [l_abv][-1][n_abv] = RC2 * ( tyy1 + tyy2 ) ;
        tzz_abv [l_abv][-1][n_abv] = RC2 * ( tzz1 + tzz2 ) ;

        n_bel    += 2 ;
        n_bel_p1 += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel += 2 ;

    } /* end of for ( l_abv = 0; ... */

 } /* end of function: multiGetAboveTxxTyyTzz () ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveTyz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996 by 
  *                 Ted Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveTyz (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  float ***tyz_bel,
        ***tyz_abv,
           tyz1, tyz2, tyz3, tyz4,
           tyz5, tyz6, tyz7, tyz8
            ;

  int      l_abv,
           n_abv,
           m_bel_m1,
           m_bel_m2,
           m_bel_m3,
           m_bel_m4,
           n_bel,
           n_bel_p1,
           l_bel,
           l_bel_p1 ;

  tyz_bel = vel_str_bel->Tyz ;
  tyz_abv = vel_str_abv->Tyz ;

  m_bel_m1 = m_max_bel - 1 ;
  m_bel_m2 = m_max_bel - 2 ;
  m_bel_m3 = m_max_bel - 3 ;
  m_bel_m4 = m_max_bel - 4 ;

  l_bel    = 0 ;
  l_bel_p1 = 1 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel    = 0 ;
     n_bel_p1 = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        /*
         * First interpolate onto the M = -1 plane
         */

        tyz1 = tyz_bel [l_bel][m_bel_m1][n_bel   ] ;
        tyz2 = tyz_bel [l_bel][m_bel_m1][n_bel_p1] ;
        tyz3 = tyz_bel [l_bel][m_bel_m2][n_bel_p1] ;
        tyz4 = tyz_bel [l_bel][m_bel_m2][n_bel   ] ;

        tyz5 = tyz_bel [l_bel_p1][m_bel_m1][n_bel   ] ;
        tyz6 = tyz_bel [l_bel_p1][m_bel_m1][n_bel_p1] ;
        tyz7 = tyz_bel [l_bel_p1][m_bel_m2][n_bel_p1] ;
        tyz8 = tyz_bel [l_bel_p1][m_bel_m2][n_bel   ] ;

        tyz_abv[l_abv][-1][n_abv] = RC8 * (
                                           tyz1 + tyz2 + tyz3 + tyz4 +
                                           tyz5 + tyz6 + tyz7 + tyz8
                                          ) ;

        /*
         * Next interpolate onto the M = -2 plane
         */

        tyz1 = tyz_bel [l_bel][m_bel_m3][n_bel   ] ;
        tyz2 = tyz_bel [l_bel][m_bel_m3][n_bel_p1] ;
        tyz3 = tyz_bel [l_bel][m_bel_m4][n_bel_p1] ;
        tyz4 = tyz_bel [l_bel][m_bel_m4][n_bel   ] ;

        tyz5 = tyz_bel [l_bel_p1][m_bel_m3][n_bel   ] ;
        tyz6 = tyz_bel [l_bel_p1][m_bel_m3][n_bel_p1] ;
        tyz7 = tyz_bel [l_bel_p1][m_bel_m4][n_bel_p1] ;
        tyz8 = tyz_bel [l_bel_p1][m_bel_m4][n_bel   ] ;

        tyz_abv[l_abv][-2][n_abv] = RC8 * (
                                           tyz1 + tyz2 + tyz3 + tyz4 +
                                           tyz5 + tyz6 + tyz7 + tyz8
                                          ) ;

        n_bel    += 2 ;
        n_bel_p1 += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel    += 2 ;
     l_bel_p1 += 2 ;

    } /* end of for ( l_abv = 0; ... */

 } /* end of function: multiGetAboveTyz () ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveTxy
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996 by 
  *                 Ted Bakowsky
  *****************************************************************
  */

 static
  multiGetAboveTxy (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  float ***txy_bel,
        ***txy_abv,
           txy1, txy2
            ;

  int      l_abv,
           n_abv,
           m_bel_m1,
           m_bel_m2,
           n_bel,
           l_bel,
           l_bel_p1 ;

  txy_bel = vel_str_bel->Txy ;
  txy_abv = vel_str_abv->Txy ;

  m_bel_m1 = m_max_bel - 1 ;
  m_bel_m2 = m_max_bel - 2 ;

  l_bel    = 0 ;
  l_bel_p1 = 1 ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel = 0 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        txy1 = txy_bel[l_bel   ][m_bel_m2][n_bel] ;
        txy2 = txy_bel[l_bel_p1][m_bel_m2][n_bel] ;

        txy_abv[l_abv][-1][n_abv] = RC2 * ( txy1 + txy2 ) ;

        n_bel += 2 ;

       } /* end of for ( n_abv=0; ... */

     l_bel    += 2 ;
     l_bel_p1 += 2 ;

    } /* end of for ( l_abv = 0; ... */

 } /* end of function: multiGetAboveTxy() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetAboveT3D
  * STORAGE CLASS : extern
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996 by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 multiGetAboveT3D (
                   par,
                   grid,
                   data
                  )
 Parameter      *par ;
 FdGrid         *grid ;
 VelocityStress *data ;
 {
  FdGrid           *grid_abv,     /* coarse grid                       */
                   *grid_bel ;    /* fine grid                         */

  VelocityStress   *vel_str_abv,  /* coarse grid velocity-stress array */
                   *vel_str_bel ; /* fine grid velocity-stress array   */

  int               m_max_bel,
                    n_max_abv,
                    l_max_abv
                     ;


  grid_bel    = &grid[1] ;
  grid_abv    = &grid[0] ;

  m_max_bel   = grid_bel->m ;
  n_max_abv   = grid_abv->n ;
  l_max_abv   = grid_abv->l ;

  vel_str_bel = &data[1] ;
  vel_str_abv = &data[0] ;

  multiGetAboveTxz       (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         ) ;

  multiGetAboveTxxTyyTzz (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         ) ;

  multiGetAboveTyz       (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         ) ;

  multiGetAboveTxy       (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         ) ;

 } /* end of function: multiGetAboveT3D() ... */



/******************************************************************/



 /*
  *****************************************************************
  * FILE          : multiGetBelowT3D.c
  * PROGRAMMED BY : Ted Bakowsky
  * DATE          : March 1996
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */



 static
  multiGetBelowTxz() ;

 static
  multiGetBelowTxxTyyTzz() ;

 static
  multiGetBelowTyz() ;

 static
  multiGetBelowTxy() ;




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowTxz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowTxz (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Txz in overlap area
   */

  int      l_bel, l_bel_p,
           n_bel, n_bel_p,
           m_bel,
           l_abv, l_abv_p,
           n_abv, n_abv_p
            ;

  float ***txz_bel,
        ***txz_abv ;

  float    txz1l,  txz2l,  txz3l,  txz4l,
           txz1lp, txz2lp, txz3lp, txz4lp,
           txz ;


  m_bel   = m_max_bel ;

  txz_bel = vel_str_bel->Txz ;
  txz_abv = vel_str_abv->Txz ;


  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;
   
  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     l_abv_p = l_abv + 1 ;

     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_p = n_abv + 1 ;

        txz1l  = txz_abv [l_abv  ] [ 0] [n_abv  ] ;
        txz2l  = txz_abv [l_abv  ] [ 0] [n_abv_p] ;
        txz3l  = txz_abv [l_abv  ] [-1] [n_abv_p] ;
        txz4l  = txz_abv [l_abv  ] [-1] [n_abv  ] ;

        txz1lp = txz_abv [l_abv_p] [ 0] [n_abv  ] ;
        txz2lp = txz_abv [l_abv_p] [ 0] [n_abv_p] ;
        txz3lp = txz_abv [l_abv_p] [-1] [n_abv_p] ;
        txz4lp = txz_abv [l_abv_p] [-1] [n_abv  ] ;

        /*
         * Get Txz [l_bel] [m_bel] [n_bel]
         */

        txz = (
                 txz1l * 3.0
               + txz4l

              ) * RC4 ;

        txz_bel [l_bel] [m_bel] [n_bel] = txz ;


        /*
         * Get Txz [l_bel+1] [m_bel] [n_bel]
         */

        txz  = (
                  txz1l  * 6.0
                + txz4l  * 2.0
                + txz1lp * 6.0
                + txz4lp * 2.0

               ) * RC16 ;

        txz_bel [l_bel_p] [m_bel] [n_bel] = txz ;


        /*
         * Get Txz [l_bel] [m_bel] [n_bel+1]
         */

        txz = (
                 txz1l * 6.0
               + txz2l * 6.0
               + txz3l * 2.0
               + txz4l * 2.0

              ) * RC16 ;

        txz_bel [l_bel] [m_bel] [n_bel_p] = txz ;


        /*
         * Get Txz [l_bel+1] [m_bel] [n_bel+1]
         */

        txz  = (
                  txz1l  * 12.0 
                + txz2l  * 12.0
                + txz3l  *  4.0
                + txz4l  *  4.0

                + txz1lp * 12.0
                + txz2lp * 12.0
                + txz3lp *  4.0
                + txz4lp *  4.0

               ) * RC64 ;

        txz_bel [l_bel_p] [m_bel] [n_bel_p] = txz ;
        

        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function multiGetBelowTxz() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowTxxTyyTzz
  * STORAGE CLASS : static
  * RETURN VALUE  : void
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowTxxTyyTzz (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Txx,Tyy & Tzz
   * in overlap region.
   */

  float ***txx_bel,
        ***tyy_bel,
        ***tzz_bel,

        ***txx_abv,
        ***tyy_abv,
        ***tzz_abv ;

  float    txx1l,  txx2l,  txx3l,  txx4l,  txx5l,  txx6l,
           txx1lp, txx2lp, txx3lp, txx4lp, txx5lp, txx6lp,

           tyy1l,  tyy2l,  tyy3l,  tyy4l,  tyy5l,  tyy6l,
           tyy1lp, tyy2lp, tyy3lp, tyy4lp, tyy5lp, tyy6lp,

           tzz1l,  tzz2l,  tzz3l,  tzz4l,  tzz5l,  tzz6l,
           tzz1lp, tzz2lp, tzz3lp, tzz4lp, tzz5lp, tzz6lp,
           txx,    tyy,    tzz ;


  int      l_bel, l_bel_p,
           n_bel, n_bel_p,
           m_bel, m_bel_p,
           l_abv, l_abv_p,
           n_abv, n_abv_m, n_abv_p
            ;

  m_bel   = m_max_bel ;
  m_bel_p = m_max_bel+1 ;

  txx_bel = vel_str_bel->Txx ;
  tyy_bel = vel_str_bel->Tyy ;
  tzz_bel = vel_str_bel->Tzz ;

  txx_abv = vel_str_abv->Txx ;
  tyy_abv = vel_str_abv->Tyy ;
  tzz_abv = vel_str_abv->Tzz ;

  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;

  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     l_abv_p = l_abv + 1 ;

     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_m = n_abv - 1 ;
        n_abv_p = n_abv + 1 ;

        txx1l  = txx_abv [l_abv  ] [1] [n_abv_m] ;
        txx2l  = txx_abv [l_abv  ] [1] [n_abv  ] ;
        txx3l  = txx_abv [l_abv  ] [1] [n_abv_p] ;
        txx4l  = txx_abv [l_abv  ] [0] [n_abv_m] ;
        txx5l  = txx_abv [l_abv  ] [0] [n_abv  ] ;
        txx6l  = txx_abv [l_abv  ] [0] [n_abv_p] ;

        txx1lp = txx_abv [l_abv_p] [1] [n_abv_m] ;
        txx2lp = txx_abv [l_abv_p] [1] [n_abv  ] ;
        txx3lp = txx_abv [l_abv_p] [1] [n_abv_p] ;
        txx4lp = txx_abv [l_abv_p] [0] [n_abv_m] ;
        txx5lp = txx_abv [l_abv_p] [0] [n_abv  ] ;
        txx6lp = txx_abv [l_abv_p] [0] [n_abv_p] ;

        tyy1l  = tyy_abv [l_abv  ] [1] [n_abv_m] ;
        tyy2l  = tyy_abv [l_abv  ] [1] [n_abv  ] ;
        tyy3l  = tyy_abv [l_abv  ] [1] [n_abv_p] ;
        tyy4l  = tyy_abv [l_abv  ] [0] [n_abv_m] ;
        tyy5l  = tyy_abv [l_abv  ] [0] [n_abv  ] ;
        tyy6l  = tyy_abv [l_abv  ] [0] [n_abv_p] ;

        tyy1lp = tyy_abv [l_abv_p] [1] [n_abv_m] ;
        tyy2lp = tyy_abv [l_abv_p] [1] [n_abv  ] ;
        tyy3lp = tyy_abv [l_abv_p] [1] [n_abv_p] ;
        tyy4lp = tyy_abv [l_abv_p] [0] [n_abv_m] ;
        tyy5lp = tyy_abv [l_abv_p] [0] [n_abv  ] ;
        tyy6lp = tyy_abv [l_abv_p] [0] [n_abv_p] ;

        tzz1l  = tzz_abv [l_abv  ] [1] [n_abv_m] ;
        tzz2l  = tzz_abv [l_abv  ] [1] [n_abv  ] ;
        tzz3l  = tzz_abv [l_abv  ] [1] [n_abv_p] ;
        tzz4l  = tzz_abv [l_abv  ] [0] [n_abv_m] ;
        tzz5l  = tzz_abv [l_abv  ] [0] [n_abv  ] ;
        tzz6l  = tzz_abv [l_abv  ] [0] [n_abv_p] ;

        tzz1lp = tzz_abv [l_abv_p] [1] [n_abv_m] ;
        tzz2lp = tzz_abv [l_abv_p] [1] [n_abv  ] ;
        tzz3lp = tzz_abv [l_abv_p] [1] [n_abv_p] ;
        tzz4lp = tzz_abv [l_abv_p] [0] [n_abv_m] ;
        tzz5lp = tzz_abv [l_abv_p] [0] [n_abv  ] ;
        tzz6lp = tzz_abv [l_abv_p] [0] [n_abv_p] ;


        /*
         * Get Txx(yy)(zz) [l_bel] [m_bel] [n_bel]
         */

        txx = (
                 txx5l * 3.0
               + txx4l

              ) * RC4 ;

        tyy = (
                 tyy5l * 3.0
               + tyy4l

              ) * RC4 ;

        tzz = (
                 tzz5l * 3.0
               + tzz4l

              ) * RC4 ;

        txx_bel [l_bel] [m_bel] [n_bel] = txx ;
        tyy_bel [l_bel] [m_bel] [n_bel] = tyy ;
        tzz_bel [l_bel] [m_bel] [n_bel] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel+1] [m_bel] [n_bel]
         */

        txx = (
                 txx5l  * 6.0
               + txx4l  * 2.0
               + txx5lp * 6.0
               + txx4lp * 2.0

              ) * RC16 ;

        tyy = (
                 tyy5l  * 6.0
               + tyy4l  * 2.0
               + tyy5lp * 6.0
               + tyy4lp * 2.0

              ) * RC16 ;

        tzz = (
                 tzz5l  * 6.0
               + tzz4l  * 2.0
               + tzz5lp * 6.0
               + tzz4lp * 2.0

              ) * RC16 ;

        txx_bel [l_bel_p] [m_bel] [n_bel] = txx ;
        tyy_bel [l_bel_p] [m_bel] [n_bel] = tyy ;
        tzz_bel [l_bel_p] [m_bel] [n_bel] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel] [m_bel] [n_bel+1]
         */

        txx = (
                 txx6l
               + txx5l * 3.0

              ) * RC4 ;

        tyy = 
              (  tyy6l
               + tyy5l * 3.0

              ) * RC4 ;

        tzz = (
                 tzz6l
               + tzz5l * 3.0

              ) * RC4 ;

        txx_bel [l_bel] [m_bel] [n_bel_p] = txx ;
        tyy_bel [l_bel] [m_bel] [n_bel_p] = tyy ;
        tzz_bel [l_bel] [m_bel] [n_bel_p] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel+1] [m_bel] [n_bel+1]
         */

        txx = (
                 txx6l  * 2.0 
               + txx5l  * 6.0
               + txx6lp * 2.0
               + txx5lp * 6.0

              ) * RC16 ;

        tyy = (
                 tyy6l  * 2.0
               + tyy5l  * 6.0
               + tyy6lp * 2.0
               + tyy5lp * 6.0

              ) * RC16 ;

        tzz = 
              (  tzz6l  * 2.0
               + tzz5l  * 6.0
               + tzz6lp * 2.0
               + tzz5lp * 6.0

              ) * RC16 ;

        txx_bel [l_bel_p] [m_bel] [n_bel_p] = txx ;
        tyy_bel [l_bel_p] [m_bel] [n_bel_p] = tyy ;
        tzz_bel [l_bel_p] [m_bel] [n_bel_p] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel] [m_bel+1] [n_bel]
         */

        txx = (
                 txx1l * 2.0
               + txx2l * 6.0
               + txx5l * 6.0
               + txx4l * 2.0

              ) * RC16 ;

        tyy = (
                 tyy1l * 2.0
               + tyy2l * 6.0
               + tyy5l * 6.0
               + tyy4l * 2.0

              ) * RC16 ;

        tzz = (
                 tzz1l * 2.0
               + tzz2l * 6.0
               + tzz5l * 6.0
               + tzz4l * 2.0

              ) * RC16 ;

        txx_bel [l_bel] [m_bel_p] [n_bel] = txx ;
        tyy_bel [l_bel] [m_bel_p] [n_bel] = tyy ;
        tzz_bel [l_bel] [m_bel_p] [n_bel] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel_p] [m_bel+1] [n_bel]
         */

        txx = (
                 txx1l  *  4.0
               + txx2l  * 12.0
               + txx5l  * 12.0
               + txx4l  *  4.0
               + txx1lp *  4.0
               + txx2lp * 12.0
               + txx5lp * 12.0
               + txx4lp *  4.0

              ) * RC64 ;

        tyy = (
                 tyy1l  *  4.0
               + tyy2l  * 12.0
               + tyy5l  * 12.0
               + tyy4l  *  4.0
               + tyy1lp *  4.0
               + tyy2lp * 12.0
               + tyy5lp * 12.0
               + tyy4lp *  4.0

              ) * RC64 ;

        tzz = (
                 tzz1l  *  4.0
               + tzz2l  * 12.0
               + tzz5l  * 12.0
               + tzz4l  *  4.0
               + tzz1lp *  4.0
               + tzz2lp * 12.0
               + tzz5lp * 12.0
               + tzz4lp *  4.0

              ) * RC64 ;

        txx_bel [l_bel_p] [m_bel_p] [n_bel] = txx ;
        tyy_bel [l_bel_p] [m_bel_p] [n_bel] = tyy ;
        tzz_bel [l_bel_p] [m_bel_p] [n_bel] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel] [m_bel+1] [n_bel_p]
         */

        txx = (
                 txx2l * 6.0
               + txx3l * 2.0
               + txx6l * 2.0
               + txx5l * 6.0

              ) * RC16 ;

        tyy = (
                 tyy2l * 6.0
               + tyy3l * 2.0
               + tyy6l * 2.0
               + tyy5l * 6.0

              ) * RC16 ;

        tzz = (
                 tzz2l * 6.0
               + tzz3l * 2.0
               + tzz6l * 2.0
               + tzz5l * 6.0

              ) * RC16 ;

        txx_bel [l_bel] [m_bel_p] [n_bel_p] = txx ;
        tyy_bel [l_bel] [m_bel_p] [n_bel_p] = tyy ;
        tzz_bel [l_bel] [m_bel_p] [n_bel_p] = tzz ;


        /*
         * Get Txx(yy)(zz) [l_bel_p] [m_bel+1] [n_bel_p]
         */

        txx = (
                 txx2l  * 12.0
               + txx3l  *  4.0
               + txx6l  *  4.0
               + txx5l  * 12.0
               + txx2lp * 12.0
               + txx3lp *  4.0
               + txx6lp *  4.0
               + txx5lp * 12.0

              ) * RC64 ;

        tyy = (
                 tyy2l  * 12.0
               + tyy3l  *  4.0
               + tyy6l  *  4.0
               + tyy5l  * 12.0
               + tyy2lp * 12.0
               + tyy3lp *  4.0
               + tyy6lp *  4.0
               + tyy5lp * 12.0

              ) * RC64 ;

        tzz = (
                 tzz2l  * 12.0
               + tzz3l  *  4.0
               + tzz6l  *  4.0
               + tzz5l  * 12.0
               + tzz2lp * 12.0
               + tzz3lp *  4.0
               + tzz6lp *  4.0
               + tzz5lp * 12.0

              ) * RC64 ;

        txx_bel [l_bel_p] [m_bel_p] [n_bel_p] = txx ;
        tyy_bel [l_bel_p] [m_bel_p] [n_bel_p] = tyy ;
        tzz_bel [l_bel_p] [m_bel_p] [n_bel_p] = tzz ;

        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function void multiGetBelowTxxTyyTzz() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowTyz
  * STORAGE CLASS : static
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowTyz (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Tyz in overlap area
   */

  float ***tyz_bel,
        ***tyz_abv ;

  float    tyz1lm, tyz2lm, tyz3lm, tyz4lm, tyz5lm, tyz6lm,
           tyz1l , tyz2l , tyz3l , tyz4l , tyz5l , tyz6l ,
           tyz1lp, tyz2lp, tyz3lp, tyz4lp, tyz5lp, tyz6lp,
           tyz ;
           
  int      l_bel, l_bel_p,
           n_bel, n_bel_p,
           m_bel,
           l_abv, l_abv_m, l_abv_p,
           n_abv, n_abv_m, n_abv_p
            ;


  m_bel   = m_max_bel ;

  tyz_bel = vel_str_bel->Tyz ;
  tyz_abv = vel_str_abv->Tyz ;


  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;
   
  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     l_abv_m = l_abv - 1 ;
     l_abv_p = l_abv + 1 ;

     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_m = n_abv - 1 ;
        n_abv_p = n_abv + 1 ;

        tyz1lm = tyz_abv [l_abv_m] [ 0] [n_abv_m] ;
        tyz2lm = tyz_abv [l_abv_m] [ 0] [n_abv  ] ;
        tyz3lm = tyz_abv [l_abv_m] [ 0] [n_abv_p] ;
        tyz4lm = tyz_abv [l_abv_m] [-1] [n_abv_m] ;
        tyz5lm = tyz_abv [l_abv_m] [-1] [n_abv  ] ;
        tyz6lm = tyz_abv [l_abv_m] [-1] [n_abv_p] ;

        tyz1l  = tyz_abv [l_abv  ] [ 0] [n_abv_m] ;
        tyz2l  = tyz_abv [l_abv  ] [ 0] [n_abv  ] ;
        tyz3l  = tyz_abv [l_abv  ] [ 0] [n_abv_p] ;
        tyz4l  = tyz_abv [l_abv  ] [-1] [n_abv_m] ;
        tyz5l  = tyz_abv [l_abv  ] [-1] [n_abv  ] ;
        tyz6l  = tyz_abv [l_abv  ] [-1] [n_abv_p] ;

        tyz1lp = tyz_abv [l_abv_p] [ 0] [n_abv_m] ;
        tyz2lp = tyz_abv [l_abv_p] [ 0] [n_abv  ] ;
        tyz3lp = tyz_abv [l_abv_p] [ 0] [n_abv_p] ;
        tyz4lp = tyz_abv [l_abv_p] [-1] [n_abv_m] ;
        tyz5lp = tyz_abv [l_abv_p] [-1] [n_abv  ] ;
        tyz6lp = tyz_abv [l_abv_p] [-1] [n_abv_p] ;


        /*
         * Get Tyz [l_bel] [m_bel] [n_bel]
         */

        tyz = (
                 tyz1lm *  3.0
               + tyz2lm *  9.0
               + tyz5lm *  3.0
               + tyz4lm *  1.0
               + tyz1l  *  9.0
               + tyz2l  * 27.0
               + tyz5l  *  9.0
               + tyz4l  *  3.0

              ) * RC64 ;

        tyz_bel [l_bel] [m_bel] [n_bel] = tyz ;


        /*
         * Get Tyz [l_bel+1] [m_bel] [n_bel]
         */

        tyz = (
                 tyz1l  *  9.0
               + tyz2l  * 27.0
               + tyz5l  *  9.0
               + tyz4l  *  3.0
               + tyz1lp *  3.0
               + tyz2lp *  9.0
               + tyz5lp *  3.0
               + tyz4lp *  1.0

              ) * RC64 ;

        tyz_bel [l_bel_p] [m_bel] [n_bel] = tyz ;


        /*
         * Get Tyz [l_bel] [m_bel] [n_bel+1]
         */

        tyz = (
                 tyz2lm *  9.0
               + tyz3lm *  3.0
               + tyz6lm *  1.0
               + tyz5lm *  3.0
               + tyz2l  * 27.0
               + tyz3l  *  9.0
               + tyz6l  *  3.0
               + tyz5l  *  9.0

              ) * RC64 ;

        tyz_bel [l_bel] [m_bel] [n_bel_p] = tyz ;


        /*
         * Get Tyz [l_bel+1] [m_bel] [n_bel+1]
         */

        tyz = (
                 tyz2l  * 27.0 
               + tyz3l  *  9.0
               + tyz6l  *  3.0
               + tyz5l  *  9.0
               + tyz2lp *  9.0
               + tyz3lp *  3.0
               + tyz6lp *  1.0
               + tyz5lp *  3.0

              ) * RC64 ;

        tyz_bel [l_bel_p] [m_bel] [n_bel_p] = tyz ;


        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function void multiGetBelowTyz ( ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowTxy
  * STORAGE CLASS : static
  * RETURN VALUE  : void
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 static
  multiGetBelowTxy (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   )
  int             m_max_bel ;
  int             l_max_abv ;
  int             n_max_abv ;
  VelocityStress *vel_str_bel ;
  VelocityStress *vel_str_abv ;
 {
  /*
   * Tri-linear interpolation to get finer-grid Txy in overlap area
   */

  float ***txy_bel,
        ***txy_abv ;

  float    txy1lm, txy2lm, txy3lm, txy4lm,
           txy1l , txy2l , txy3l , txy4l ,
           txy1lp, txy2lp, txy3lp, txy4lp,
           txy ;

  int      l_bel, l_bel_p,
           n_bel, n_bel_p,
           m_bel, m_bel_p,
           l_abv, l_abv_m, l_abv_p,
           n_abv, n_abv_p
            ;


  m_bel   = m_max_bel ;
  m_bel_p = m_max_bel + 1 ;

  txy_bel = vel_str_bel->Txy ;
  txy_abv = vel_str_abv->Txy ;


  /*
   * Outer loop on l-index for coarse grid (y-direction)
   */

  l_bel   = 0 ;
  l_bel_p = 1 ;
   
  for ( l_abv=0; l_abv<l_max_abv; l_abv++ )
    {
     l_abv_p = l_abv + 1 ;
     l_abv_m = l_abv - 1 ;

     /*
      * Inner loop on n-index (x-direction)
      */

     n_bel   = 0 ;
     n_bel_p = 1 ;

     for ( n_abv=0; n_abv<n_max_abv; n_abv++ )
       {
        n_abv_p = n_abv + 1 ;

        txy1lm = txy_abv [l_abv_m] [1] [n_abv  ] ;
        txy2lm = txy_abv [l_abv_m] [1] [n_abv_p] ;
        txy3lm = txy_abv [l_abv_m] [0] [n_abv_p] ;
        txy4lm = txy_abv [l_abv_m] [0] [n_abv  ] ;

        txy1l  = txy_abv [l_abv  ] [1] [n_abv  ] ;
        txy2l  = txy_abv [l_abv  ] [1] [n_abv_p] ;
        txy3l  = txy_abv [l_abv  ] [0] [n_abv_p] ;
        txy4l  = txy_abv [l_abv  ] [0] [n_abv  ] ;

        txy1lp = txy_abv [l_abv_p] [1] [n_abv  ] ;
        txy2lp = txy_abv [l_abv_p] [1] [n_abv_p] ;
        txy3lp = txy_abv [l_abv_p] [0] [n_abv_p] ;
        txy4lp = txy_abv [l_abv_p] [0] [n_abv  ] ;


        /*
         * Get Txy [l_bel] [m_bel] [n_bel]     (1)
         */

        txy = (
                 txy4lm
               + txy4l  * 3.0

              ) * RC4 ;

        txy_bel [l_bel] [m_bel] [n_bel] = txy ;


        /*
         * Get Txy [l_bel+1] [m_bel] [n_bel]     (2)
         */

        txy = (
                 txy4l  * 3.0
               + txy4lp

              ) * RC4 ;

        txy_bel [l_bel_p] [m_bel] [n_bel] = txy ;


        /*
         * Get Txy [l_bel] [m_bel] [n_bel+1]     (3)
         */

        txy = (
                 txy3lm * 2.0
               + txy4lm * 2.0
               + txy3l  * 6.0
               + txy4l  * 6.0

              ) * RC16 ;

        txy_bel [l_bel] [m_bel] [n_bel_p] = txy ;


        /*
         * Get Txy [l_bel+1] [m_bel] [n_bel+1]     (4)
         */

        txy = (
                 txy3l  * 6.0
               + txy4l  * 6.0
               + txy3lp * 2.0
               + txy4lp * 2.0

              ) * RC16 ;

        txy_bel [l_bel_p] [m_bel] [n_bel_p] = txy ;


        /*
         * Get Txy [l_bel] [m_bel+1] [n_bel]     (5)
         */

        txy = (
                 txy1lm  * 2.0
               + txy4lm  * 2.0
               + txy1l   * 6.0
               + txy4l   * 6.0

              ) * RC16 ;

        txy_bel [l_bel] [m_bel_p] [n_bel] = txy ;


        /*
         * Get Txy [l_bel+1] [m_bel+1] [n_bel]     (6)
         */

        txy = (
                 txy1l  * 6.0
               + txy4l  * 6.0
               + txy1lp * 2.0
               + txy4lp * 2.0

              ) * RC16 ;

        txy_bel [l_bel_p] [m_bel_p] [n_bel] = txy ;


        /*
         * Get Txy [l_bel] [m_bel+1] [n_bel+1]     (7)
         */

       txy = (
                txy1lm *  4.0
              + txy2lm *  4.0
              + txy3lm *  4.0
              + txy4lm *  4.0
              + txy1l  * 12.0
              + txy2l  * 12.0
              + txy3l  * 12.0
              + txy4l  * 12.0

             ) * RC64 ;

        txy_bel [l_bel] [m_bel_p] [n_bel_p] = txy ;


        /*
         * Get Txy [l_bel+1] [m_bel+1] [n_bel_p]     (8)
         */

       txy = (
                txy1l  * 12.0
              + txy2l  * 12.0
              + txy3l  * 12.0
              + txy4l  * 12.0
              + txy1lp *  4.0
              + txy2lp *  4.0
              + txy3lp *  4.0
              + txy4lp *  4.0

             ) * RC64 ;

        txy_bel [l_bel_p] [m_bel_p] [n_bel_p] = txy ;


        n_bel   += 2 ;
        n_bel_p += 2 ;

       } /* end of for ( l_abv=0; ... */

      l_bel   += 2 ;
      l_bel_p += 2 ;

    } /* end of for ( l_abv=0; ... */

 } /* end of static function multiGetBelowTxy() ... */




 /*
  *****************************************************************
  * FUNCTION NAME : multiGetBelowT3D
  * STORAGE CLASS : extern
  * RETURN VALUE  : default
  * DATE          : March 1996
  * PROGRAMMED BY : Ted Bakowsky
  * MODIFIED      : Converted to pre-ANSI C, July 1996, by Ted 
  *                 Bakowsky
  *****************************************************************
  */

 multiGetBelowT3D (
                   par,
                   grid,
                   data
                  )
 Parameter      *par ;
 FdGrid         *grid ;
 VelocityStress *data ;
 {
  FdGrid         *grid_abv,         /* coarse grid                       */
                 *grid_bel ;        /* fine   grid                       */

  VelocityStress *vel_str_abv,      /* coarse grid velocity-stress array */
                 *vel_str_bel ;     /* fine   grid velocity-stress array */

  int             m_max_bel,
                  n_max_abv,
                  l_max_abv
                    ;

  grid_bel = &grid[ 0] ;
  grid_abv = &grid[-1] ;

  m_max_bel = grid_bel->m ;

  l_max_abv = grid_abv->l ;
  n_max_abv = grid_abv->n ;

  vel_str_bel = &data[ 0] ;
  vel_str_abv = &data[-1] ;

  multiGetBelowTxz (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   ) ;

  multiGetBelowTxxTyyTzz (
                          m_max_bel,
                          l_max_abv,
                          n_max_abv,
                          vel_str_bel,
                          vel_str_abv
                         ) ;

  multiGetBelowTyz (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   ) ;

  multiGetBelowTxy (
                    m_max_bel,
                    l_max_abv,
                    n_max_abv,
                    vel_str_bel,
                    vel_str_abv
                   ) ;

 } /* end of function: multiGetBelowT3D() ... */
