/*  3d updates (prettier version than in stencil3d)

   update2Vx2d   : 2th order (spatial) velocity update in x direction
   update2Vy2d   : 2th order (spatial) velocity update in y direction
   update2Vz2d   : 2th order (spatial) velocity update in z direction
   update2Txx2d  : 2th order (spatial) stress   update (normal x)
   update2Tyy2d  : 2th order (spatial) stress   update (normal y)

update2Vx(par, grid, Vx, Txx, Txy, Txz, l, m, n)
Parameter *par;
FdGrid    *grid;
float ***Vx, ***Txx, ***Txy, ***Txz;
int l, m, n;
{
   float irho;
   irho = grid->irho[l][m][n];
   Vx[l][m][n] = par->dt*((Txx[l  ][m  ][n  ] - Txx[l  ][m  ][n-1])/grid->dx  + 
                          (Txy[l  ][m  ][n  ] - Txy[l-1][m  ][n  ])/grid->dy  +
                          (Txz[l  ][m  ][n  ] - Txz[l  ][m-1][n  ])/grid->dz)
                           *irho + Vx[l][m][n];
}



update2Vy(par, grid, Vy, Txy, Tyy, Tyz, l, m, n)
Parameter *par;
FdGrid    *grid;
float ***Vy, ***Txy, ***Tyy, ***Tyz;
int l, m, n;
{
   float irho;
   irho = grid->irho[l][m][n];
   Vy[l][m][n] = par->dt*((Txy[l  ][m  ][n+1] - Txy[l  ][m  ][n  ])/grid->dx  + 
                          (Tyy[l+1][m  ][n  ] - Tyy[l  ][m  ][n  ])/grid->dy  +
                          (Tyz[l  ][m  ][n  ] - Tyz[l  ][m-1][n  ])/grid->dz)
                           *irho + Vy[l][m][n];
}


update2Vz(par, grid, Vz, Txz, Tyz, Tzz, l, m, n)
Parameter *par;
FdGrid    *grid;
float ***Vz, ***Txz, ***Tyz, ***Tzz;
int l, m, n;
{
   float irho;
   irho = grid->irho[l][m][n];
   Vz[l][m][n] = par->dt*((Txz[l  ][m  ][n+1] - Txz[l  ][m  ][n  ])/grid->dx  + 
                          (Tyz[l  ][m  ][n  ] - Tyz[l-1][m  ][n  ])/grid->dy  +
                          (Tzz[l  ][m+1][n  ] - Tzz[l  ][m  ][n  ])/grid->dz)
                           *irho + Vz[l][m][n];
}



update2Txx(par, grid, Txx, Vx, Vy, Vz, l, m, n)
Parameter *par;
FdGrid    *grid;
float ***Vx, ***Vy, ***Vz, ***Txx;
int l, m, n;
{
   float lamda, mu;
   lamda = grid->lamda[l][m][n];
   mu    = grid->mu[l][m][n];
   Txx[l][m][n] = par->dt*
          ((Vx[l  ][m  ][n+1] - Vx[l  ][m  ][n  ])*(lamda + 2.* mu)/grid->dx  +
           (Vy[l  ][m  ][n  ] - Vy[l-1][m  ][n  ])*(lamda         )/grid->dy  +
           (Vz[l  ][m  ][n  ] - Vz[l  ][m-1][n  ])*(lamda         )/grid->dz) +
           Txx[l][m][n];
}




update2Tyy(par, grid, Tyy, Vx, Vy, Vz, l, m, n)
Parameter *par;
FdGrid    *grid;
float ***Vx, ***Vy, ***Vz, ***Tyy;
int l, m, n;
{
   float lamda, mu;
   lamda = grid->lamda[l][m][n];
   mu    = grid->mu[l][m][n];
   Tyy[l][m][n] = par->dt*
          ((Vx[l  ][m  ][n+1] - Vx[l  ][m  ][n  ])*(lamda         )/grid->dx  +
           (Vy[l  ][m  ][n  ] - Vy[l-1][m  ][n  ])*(lamda + 2.* mu)/grid->dy  +
           (Vz[l  ][m  ][n  ] - Vz[l  ][m-1][n  ])*(lamda         )/grid->dz) +
           Tyy[l][m][n];
}




update2Tzz(par, grid, Tzz, Vx, Vy, Vz, l, m, n)
Parameter *par;
FdGrid    *grid;
float ***Vx, ***Vy, ***Vz, ***Tzz;
int l, m, n;
{
   float lamda, mu;
   lamda = grid->lamda[l][m][n];
   mu    = grid->mu[l][m][n];
   Tzz[l][m][n] = par->dt*
          ((Vx[l  ][m  ][n+1] - Vx[l  ][m  ][n  ])*(lamda         )/grid->dx  +
           (Vy[l  ][m  ][n  ] - Vy[l-1][m  ][n  ])*(lamda         )/grid->dy  +
           (Vz[l  ][m  ][n  ] - Vz[l  ][m-1][n  ])*(lamda + 2.* mu)/grid->dz) +
           Tzz[l][m][n];
}
*/

