/* file for old c subroutines */

#include "elas.h"

oldc() {;}

old_update4A2D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float ***Vx, ***Vy, ***Vz, ***Txx, ***Tyy, ***Tzz, ***Txy, ***Txz, ***Tyz;
   float ***g, ***u, c1, c2, **dX, **dZ;
   int l, lmin, d;
   int i, n, m, mmin;

   /* initialize */
   if (grid->dx != grid->dy) {fprintf(stderr, "ERROR: dx != dy\n"); exit(-1);}
   if (grid->dx != grid->dz) {fprintf(stderr, "ERROR: dx != dz\n"); exit(-1);}
   Txx = data->Txx; Tyy = data->Tyy; Tzz = data->Tzz;
   Txy = data->Txy; Txz = data->Txz; Tyz = data->Tyz;
   Vx = data->Vx; Vy = data->Vy; Vz = data->Vz;
   g = grid->lamda;
   u = grid->mu;
   d = data->nmax - data->nmin;
   n = data->nmin;
   lmin = data->lmin;
   mmin = data->mmin;
   c1 = C1*par->dt/grid->dx;
   c2 = C2*par->dt/grid->dx;

   /* update stresses along surface (now done in surface routine) */

   /* attenuation */
   attenuate2D(par, grid, data);

}

old_update4A3D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{

   /* initialize */

   /* update stresses along surface */
   /****** now in surface *******/

   /* attenuation */
   attenuate3D(par, grid, data);

}


/************************************************************************
   paraxial3d: paraxial boundary condition routines for 3D elastic code

   dabsorbLVR  :  Clayton-Engquist A1 absorbing boundary condition-(right side)
   dabsorbLVL  :  Clayton-Engquist A1 absorbing boundary condition-(left side)
   dabsorbLVLa :  Clayton-Engquist A1 absorbing boundary condition-(left side)
   dabsorbLVB  :  Clayton-Engquist A1 absorbing boundary condition-(bottom)
   dabsorbLVBa :  Clayton-Engquist A1 absorbing boundary condition-(bottom)
   dabsorbLVF  :  Clayton-Engquist A1 absorbing boundary condition -(front)
   dabsorbLVFa :  Clayton-Engquist A1 absorbing boundary condition -(front)
   dabsorbLVK  :  Clayton-Engquist A1 absorbing boundary condition - (back)
   dabsorbLVKa :  Clayton-Engquist A1 absorbing boundary condition - (back)
   dedgeVBLa   :   edge velocity boundary condition (bottom left)
   dedgeVBRa   :   edge velocity boundary condition (bottom right)
   dboundTR    :   stress boundary condition (right side)
   dboundTLa   :   stress boundary condition (left side) 
   dboundTL    :   stress boundary condition (left side) 
   dboundTBa   :   stress boundary condition (bottom)
   dboundTB    :   stress boundary condition (bottom)
   dboundTFa   :   stress boundary condition (front)
   dboundTF    :   stress boundary condition (front)
   dboundTKa   :   stress boundary condition (back)
   dboundTK    :   stress boundary condition (back)
   dedgeTBLa   :   edge stress boundary condition (bottom left)
   dedgeTBRa   :   edge stress boundary condition (bottom right)
   dcornerTBLF :   corner stress boundary condition (front bottom left)
   dcornerTBRF :   corner stress boundary condition (front bottom right)
   dcornerTTLF :   corner stress boundary condition (front top left)
   dcornerTTRF :   corner stress boundary condition (front top right)
   dcornerTBLK :   corner stress boundary condition (back bottom left)
   dcornerTBRK :   corner stress boundary condition (back bottom right)
   dcornerTTLK :   corner stress boundary condition (back top left)
   dcornerTTRK :   corner stress boundary condition (back top right)
   vboundTRvector:

   aboundTRvector: *

   * means not made acoustic
   
   compile: cc -g -c paraxial.c

   author:  denise maddix
   date  :  10 January 1995
   where :  llnl

   author:  shawn larsen  (vectorized routines and acoustic routines)
   date:    january 1997, august 2001                       
   where:   llnl                               
**************************************************************************/


/*************************************************************************
  VELOCITY BOUNDARY CONDITIONS
*************************************************************************/

dabsorbLVR(par,grid,data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 float VxLast,VyLast,VzLast,alpha,beta,xP,xS;
 int l,m,n;

 n = grid->n;    /* Points one out from right boundary */
 for(m=0; m < grid->m; m++) {
  for(l=0; l< grid->l; l++) {
   VxLast = data->Vx[l][m][n];
   VyLast = data->Vy[l][m][n];
   VzLast = data->Vz[l][m][n];

   /* Update velocities one out from boundary using 2nd order differencing*/
  update2Vx3d(par,grid,data->Vx,data->Txx,data->Txy,data->Txz,l,m,n);
  update2Vy3d(par,grid,data->Vy,data->Txy,data->Tyy,data->Tyz,l,m,n);
  update2Vz3d(par,grid,data->Vz,data->Txz,data->Tyz,data->Tzz,l,m,n);

   /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
   beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

   /* Determine the coefficients for paraxial extrapolation */
   xP = (grid->dx - alpha*par->dt)/(grid->dx + alpha*par->dt);
   xS = (grid->dx -  beta*par->dt)/(grid->dx +  beta*par->dt);

   /* Update velocities on right boundary using paraxial extrapolation */
   data->Vx[l][m][n+1] = VxLast + xP*(data->Vx[l][m][n+1] - data->Vx[l][m][n]);
   data->Vy[l][m][n+1] = VyLast + xS*(data->Vy[l][m][n+1] - data->Vy[l][m][n]);
   data->Vz[l][m][n+1] = VzLast + xS*(data->Vz[l][m][n+1] - data->Vz[l][m][n]);
 
   }
  }
} 
    
dabsorbLVL(par, grid,data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{

 float VxLast,VyLast,VzLast,alpha,beta,xP,xS;
 int l,m,n;

 n = -1;         /* Points one out from boundary on left side */
 for(m=0; m < grid->m; m++) {
  for(l=0; l< grid->l; l++) {

   VxLast = data->Vx[l][m][n];
   VyLast = data->Vy[l][m][n];
   VzLast = data->Vz[l][m][n];

/* Update velocities one out from left boundary using 2nd order scheme */
  update2Vx3d(par,grid,data->Vx,data->Txx,data->Txy,data->Txz,l,m,n);
  update2Vy3d(par,grid,data->Vy,data->Txy,data->Tyy,data->Tyz,l,m,n);
  update2Vz3d(par,grid,data->Vz,data->Txz,data->Tyz,data->Tzz,l,m,n);
  
   /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
   beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

   /* Determine the coefficients for paraxial extrapolation */
   xP = (grid->dx - alpha*par->dt)/(grid->dx + alpha*par->dt);
   xS = (grid->dx -  beta*par->dt)/(grid->dx +  beta*par->dt);

   /* Update velocities on left boundary using paraxial extrapolation */
   data->Vx[l][m][n-1] = VxLast + xP*(data->Vx[l][m][n-1] - data->Vx[l][m][n]);
   /* Update velocities using paraxial extrapolations in y, and z for 
    points two out from left boundary for stability */
   data->Vy[l][m][n-1] = VyLast + xS*(data->Vy[l][m][n-1] - data->Vy[l][m][n]);
   data->Vz[l][m][n-1] = VzLast + xS*(data->Vz[l][m][n-1] - data->Vz[l][m][n]);
  }
 }

}


dabsorbLVLa(par,grid,data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 float VxLast,VyLast,VzLast,alpha,beta,xP,xS;
 int l,m,n;

 n = -1;         /* Points one out from boundary on left side */
 for(m=0; m < grid->m; m++) {
  for(l=0; l< grid->l; l++) {

/* Update velocity in y and z direction one out from left boundary using
    4th order differencing */
  update4Vy3d(par,grid,data->Vy,data->Txy,data->Tyy,data->Tyz,l,m,n);
  update4Vz3d(par,grid,data->Vz,data->Txz,data->Tyz,data->Tzz,l,m,n);
  
   VxLast = data->Vx[l][m][n];
   VyLast = data->Vy[l][m][n-1];
   VzLast = data->Vz[l][m][n-1];

   /* Update velocities using 2nd order differencing*/
  update2Vx3d(par,grid,data->Vx,data->Txx,data->Txy,data->Txz,l,m,n);
  /* Update velocities in y and z two points out from left boundary */
  update2Vy3d(par,grid,data->Vy,data->Txy,data->Tyy,data->Tyz,l,m,n-1);
  update2Vz3d(par,grid,data->Vz,data->Txz,data->Tyz,data->Tzz,l,m,n-1);

   /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
   beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

   /* Determine the coefficients for paraxial extrapolation */
   xP = (grid->dx - alpha*par->dt)/(grid->dx + alpha*par->dt);
   xS = (grid->dx -  beta*par->dt)/(grid->dx +  beta*par->dt);

   /* Update velocities on left boundary using paraxial extrapolation */
   data->Vx[l][m][n-1] = VxLast + xP*(data->Vx[l][m][n-1] - data->Vx[l][m][n]);
   /* Update velocities using paraxial extrapolations in y, and z for 
    points three out from left boundary for stability */
   data->Vy[l][m][n-2] = VyLast + xS*(data->Vy[l][m][n-2] - data->Vy[l][m][n-1]);
   data->Vz[l][m][n-2] = VzLast + xS*(data->Vz[l][m][n-2] - data->Vz[l][m][n-1]);
   }
 }
}

dabsorbLVB(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
   float VxLast,VyLast,VzLast,alpha,beta,zP,zS;
   int l,m,n;

   m = grid->m;   /*  Points one down from bottom boundary */
   for(n=0; n < grid->n; n++) {
    for(l=0; l < grid->l; l++){
 
     VxLast = data->Vx[l][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l][m][n];

/* Update velocities one down from bottom using 2nd order differencing */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);

 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     zP = (grid->dz - alpha*par->dt)/(grid->dz + alpha*par->dt);
     zS = (grid->dz -  beta*par->dt)/(grid->dz +  beta*par->dt);

 /* Update velocities in x, y using paraxial extrapolations for points
    three out from bottom boundary for stability purposes */
  data->Vx[l][m+1][n] = VxLast + zS*(data->Vx[l][m+1][n] - data->Vx[l][m][n]);
  data->Vy[l][m+1][n] = VyLast + zS*(data->Vy[l][m+1][n] - data->Vy[l][m][n]);
  data->Vz[l][m+1][n] = VzLast + zP*(data->Vz[l][m+1][n] - data->Vz[l][m][n]);
   }
  }
 
}

dabsorbLVBa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
   float VxLast,VyLast,VzLast,alpha,beta,zP,zS;
   int l,m,n;

   m = grid->m;   /*  Points one down from bottom boundary */
   for(n=0; n < grid->n; n++) {
    for(l=0; l < grid->l; l++){

/* Update velocity in x and y directions one down from bottom using 
   4th order differencing */
     update4Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update4Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
 
     VxLast = data->Vx[l][m+1][n];
     VyLast = data->Vy[l][m+1][n];
     VzLast = data->Vz[l][m][n];

/*Update velocities in x, y two down from bottom boundary using 2nd order scheme
  Update velocity in z one down from bottom boundary */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m+1, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m+1, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);

 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     zP = (grid->dz - alpha*par->dt)/(grid->dz + alpha*par->dt);
     zS = (grid->dz -  beta*par->dt)/(grid->dz +  beta*par->dt);

 /* Update velocities in x, y using paraxial extrapolations for points
    three out from bottom boundary for stability purposes */
  data->Vx[l][m+2][n] = VxLast + zS*(data->Vx[l][m+2][n] - data->Vx[l][m+1][n]);
  data->Vy[l][m+2][n] = VyLast + zS*(data->Vy[l][m+2][n] - data->Vy[l][m+1][n]);
  data->Vz[l][m+1][n] = VzLast + zP*(data->Vz[l][m+1][n] - data->Vz[l][m][n]);
   }
  }
 
}

dabsorbLVF(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{

  float VxLast,VyLast,VzLast,alpha,beta,yP,yS;
  int l,m,n;

  l= -1;  /* Points one out from front face */
  for(n=0; n < grid->n; n++){
   for(m=0; m< grid->m; m++){

     VxLast = data->Vx[l][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l][m][n];

/* Update velocities one out from front using 2nd order differencing */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);
 
 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     yP = (grid->dy - alpha*par->dt)/(grid->dy + alpha*par->dt);
     yS = (grid->dy -  beta*par->dt)/(grid->dy +  beta*par->dt);
     
 /* Update velocities in x, z using paraxial extrapolations for points
    three out from bottom boundary for stability purposes */
  data->Vx[l-1][m][n] = VxLast + yS*(data->Vx[l-1][m][n] - data->Vx[l][m][n]);
  data->Vy[l-1][m][n] = VyLast + yP*(data->Vy[l-1][m][n] - data->Vy[l][m][n]);
  data->Vz[l-1][m][n] = VzLast + yS*(data->Vz[l-1][m][n] - data->Vz[l][m][n]);

  }
 }

}

dabsorbLVFa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{

  float VxLast,VyLast,VzLast,alpha,beta,yP,yS;
  int l,m,n;

  l= -1;  /* Points one out from front face */
  for(n=0; n < grid->n; n++){
   for(m=0; m< grid->m; m++){

/* Update velocity in x and z directions one out from front using 
   4th order differencing */
     update4Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update4Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);
 
     VxLast = data->Vx[l-1][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l-1][m][n];

/*Update velocities in x, z two out from front boundary using 2nd order scheme
  Update velocity in y one out from front face */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l-1, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l-1, m, n);

 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     yP = (grid->dy - alpha*par->dt)/(grid->dy + alpha*par->dt);
     yS = (grid->dy -  beta*par->dt)/(grid->dy +  beta*par->dt);
     
 /* Update velocities in x, z using paraxial extrapolations for points
    three out from bottom boundary for stability purposes */
  data->Vx[l-2][m][n] = VxLast + yS*(data->Vx[l-2][m][n] - data->Vx[l-1][m][n]);
  data->Vy[l-1][m][n] = VyLast + yP*(data->Vy[l-1][m][n] - data->Vy[l][m][n]);
  data->Vz[l-2][m][n] = VzLast + yS*(data->Vz[l-2][m][n] - data->Vz[l-1][m][n]);

  }
 }

}

dabsorbLVK(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;


{

  float VxLast,VyLast,VzLast,alpha,beta,yP,yS;
  int l,m,n;

  l= grid->l;  /* Points one out from back face */
  for(n=0; n < grid->n; n++){
   for(m=0; m< grid->m; m++){
 
     VxLast = data->Vx[l][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l][m][n];

/* Update velocities one out from back using 2nd order differencing */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);
 
 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     yP = (grid->dy - alpha*par->dt)/(grid->dy + alpha*par->dt);
     yS = (grid->dy -  beta*par->dt)/(grid->dy +  beta*par->dt);
     
 /* Update velocities using paraxial extrapolations for points
    two out from back face  */
  data->Vx[l+1][m][n] = VxLast + yS*(data->Vx[l+1][m][n] - data->Vx[l][m][n]);
  data->Vy[l+1][m][n] = VyLast + yP*(data->Vy[l+1][m][n] - data->Vy[l][m][n]);
  data->Vz[l+1][m][n] = VzLast + yS*(data->Vz[l+1][m][n] - data->Vz[l][m][n]);

  }
 }

}

dabsorbLVKa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;


{

  float VxLast,VyLast,VzLast,alpha,beta,yP,yS;
  int l,m,n;

  l= grid->l;  /* Points one out from back face */
  for(n=0; n < grid->n; n++){
   for(m=0; m< grid->m; m++){

/* Update velocity in x and z directions one out from back using 
   4th order differencing */
     update4Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update4Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);
 
     VxLast = data->Vx[l+1][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l+1][m][n];

/*Update velocities in x, z two out from back face using 2nd order scheme
  Update velocity in y one out from back face */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l+1, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l+1, m, n);

 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     yP = (grid->dy - alpha*par->dt)/(grid->dy + alpha*par->dt);
     yS = (grid->dy -  beta*par->dt)/(grid->dy +  beta*par->dt);
     
 /* Update velocities in x, z using paraxial extrapolations for points
    three out from back face for stability purposes */
  data->Vx[l+2][m][n] = VxLast + yS*(data->Vx[l+2][m][n] - data->Vx[l+1][m][n]);
  data->Vy[l+1][m][n] = VyLast + yP*(data->Vy[l+1][m][n] - data->Vy[l][m][n]);
  data->Vz[l+2][m][n] = VzLast + yS*(data->Vz[l+2][m][n] - data->Vz[l+1][m][n]);

  }
 }

}

 
dedgeVBLa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 float VxLast, VyLast, VzLast, alpha, beta, xP, xS, zP, zS;
 int l,m,n;
 m = grid-> m;    /* Points one down from bottom */
 n = -1;          /* Points one out from left side */
 for(l=0; l<grid->l; l++);  {   /* bottom left edge */
   
     VxLast = data->Vx[l][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l][m][n];

/* Update velocities using 2nd order differencing */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);
 
 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     xP = (grid->dx - alpha*par->dt)/(grid->dx + alpha*par->dt);
     xS = (grid->dx -  beta*par->dt)/(grid->dx +  beta*par->dt);
     
     zP = (grid->dz - alpha*par->dt)/(grid->dz + alpha*par->dt);
     zS = (grid->dz -  beta*par->dt)/(grid->dz +  beta*par->dt);

 /* Update velocities using paraxial extrapolations for points 
    two out and down along bottom left edge */
  data->Vx[l][m+1][n] = VxLast + zS*(data->Vx[l][m+1][n] - data->Vx[l][m][n]);
  data->Vy[l][m+1][n] = VyLast + zS*(data->Vy[l][m+1][n] - data->Vy[l][m][n]);
  data->Vz[l][m+1][n] = VzLast + zP*(data->Vz[l][m+1][n] - data->Vz[l][m][n]);

  data->Vx[l][m][n-1] = VxLast + xP*(data->Vx[l][m][n-1] - data->Vx[l][m][n]);
  data->Vy[l][m][n-1] = VyLast + xS*(data->Vy[l][m][n-1] - data->Vy[l][m][n]);
  data->Vz[l][m][n-1] = VzLast + xS*(data->Vz[l][m][n-1] - data->Vz[l][m][n]);
 
  }
 }


dedgeVBRa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 float VxLast, VyLast, VzLast, alpha, beta, xP, xS, zP, zS;
 int l,m,n;
 m = grid-> m;    /* Points one down from bottom */
 n = grid->n;          /* Points one out from right side */
 for(l=0; l<grid->l; l++);  {   /* bottom right edge */
   
     VxLast = data->Vx[l][m][n];
     VyLast = data->Vy[l][m][n];
     VzLast = data->Vz[l][m][n];

/* Update velocities using 2nd order differencing */
     update2Vx3d(par,grid, data->Vx, data->Txx, data->Txy, data->Txz, l, m, n);
     update2Vy3d(par,grid, data->Vy, data->Txy, data->Tyy, data->Tyz, l, m, n);
     update2Vz3d(par,grid, data->Vz, data->Txz, data->Tyz, data->Tzz, l, m, n);
 
 /* Calculate wave speed parameters */
   alpha=sqrt((grid->lamda[l][m][n]+2.*grid->mu[l][m][n])*grid->irho[l][m][n]);
    beta =sqrt((grid->mu[l][m][n])*grid->irho[l][m][n]);

 /* Determine the coefficients for paraxial extrapolation */
     xP = (grid->dx - alpha*par->dt)/(grid->dx + alpha*par->dt);
     xS = (grid->dx -  beta*par->dt)/(grid->dx +  beta*par->dt);
     
     zP = (grid->dz - alpha*par->dt)/(grid->dz + alpha*par->dt);
     zS = (grid->dz -  beta*par->dt)/(grid->dz +  beta*par->dt);

 /* Update velocities using paraxial extrapolations for points 
    two out and down along bottom right edge */
  data->Vx[l][m+1][n] = VxLast + zS*(data->Vx[l][m+1][n] - data->Vx[l][m][n]);
  data->Vy[l][m+1][n] = VyLast + zS*(data->Vy[l][m+1][n] - data->Vy[l][m][n]);
  data->Vz[l][m+1][n] = VzLast + zP*(data->Vz[l][m+1][n] - data->Vz[l][m][n]);

  data->Vx[l][m][n+1] = VxLast + xP*(data->Vx[l][m][n+1] - data->Vx[l][m][n]);
  data->Vy[l][m][n+1] = VyLast + xS*(data->Vy[l][m][n+1] - data->Vy[l][m][n]);
  data->Vz[l][m][n+1] = VzLast + xS*(data->Vz[l][m][n+1] - data->Vz[l][m][n]);
 
  }
 }


/****************************************************************************
  STRESS BOUNDARY CONDITIONS
*****************************************************************************/

dboundTR(par,grid,data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 int l,m,n;
 n = grid->n;   /* Points one in from boundary */
 for(m=0; m < grid->m; m++) {
  for(l=0; l< grid->l; l++) {
   update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
   update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
   update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
   update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
   update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
   update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
   update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n+1);
   update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n+1); 
   }
  }
}

dboundTLa(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int l,m, n;
   n = -1;
    for(l=0; l< grid->l; l++){
      /*
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, 0, n);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, 0, n);
      */
/* free surface boundary condition */
      if (grid->bz1 == SURFACE) {
        update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l,0,n);
        update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz,l,0,n);
        data->Tzz[l][0][n] = 0.;
        }
      else {
        update4Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l,0,n);
        update4Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz,l,0,n);
        update4Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz,l,0,n);
        }

/* Update stresses for points one out from left boundary with m = 0*/
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, 0, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, 0, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, 0, n);
/* Update stresses for points two out from left boundary with m = 0 */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l,0, n-1);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, 0, n);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l,0, n-1);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, 0, n - 1);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, 0, n - 1);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, 0, n - 1);
     }
   for(m = 1; m < grid->m; m++) {
    for(l = 0; l < grid->l; l++) {
/*  Update stresses for points one out from left boundary */
      update4Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
      update4Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
      update4Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two out from left boundary */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy,data->Vz, l, m, n - 1);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n - 1);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n - 1);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n - 1);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n - 1);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n - 1);
      }
     } 
}

dboundTL(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   int l,m, n;
   n = -1;
    for(l=0; l< grid->l; l++){
      /*
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, 0, n);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, 0, n);
      */
/* free surface boundary condition */
      if (grid->bz1 == SURFACE) {
        update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l,0,n);
        update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz,l,0,n);
        data->Tzz[l][0][n] = 0.;
        }
      else {
        update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l,0,n);
        update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz,l,0,n);
        update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz,l,0,n);
        }

/* Update stresses for points one out from left boundary with m = 0*/
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, 0, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, 0, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, 0, n);
/* Update stresses for points two out from left boundary with m = 0 */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l,0, n-1);
/*      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, 0, n);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l,0, n-1);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, 0, n - 1);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, 0, n - 1);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, 0, n - 1); */
     }
   for(m = 1; m < grid->m; m++) {
    for(l = 0; l < grid->l; l++) {
/*  Update stresses for points one out from left boundary */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two out from left boundary */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy,data->Vz, l, m, n - 1);
/*      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n - 1);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n - 1);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n - 1);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n - 1);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n - 1);  */
      }
     } 
}

dboundTBa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
  int l, m, n;
  m = grid->m;   /* Points one down from bottom boundary */
  for(n=0; n < grid->n; n++){
   for(l=0; l< grid->l; l++){
/* Update  stresses for points one down from bottom using 2nd order scheme */
      update4Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
      update4Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
      update4Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two down from bottom using 2nd order scheme */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l,m+1, n);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l,m+1,n);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l,m+1, n);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m+1, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m+1, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m+1, n); 
      }
    }
}

dboundTB(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
  int l, m, n;
  m = grid->m;   /* Points one down from bottom boundary */
  for(n=0; n < grid->n; n++){
   for(l=0; l< grid->l; l++){
/* Update  stresses for points one down from bottom using 2nd order scheme */
      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two down from bottom using 2nd order scheme */
/*      update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l,m+1, n);
      update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l,m+1,n); */
      update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l,m+1, n);
/*      update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m+1, n);
      update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m+1, n);
      update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m+1, n);  */ 
      }
    }
}

dboundTFa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
  int l, m, n;
  l = -1;   /* Points one out from front face */
  for(n=0; n < grid->n; n++){
   for(m=0; m < grid->m; m++){
/* Update stresses for points one out from front face */
    update4Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
    update4Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
    update4Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two out from front face using 2nd order scheme */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l-1,m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l-1,m,n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l-1,m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l-1, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l-1, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l-1, m, n); 
   }

  }

}

dboundTF(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
  int l, m, n;
  l = -1;   /* Points one out from front face */
  for(n=0; n < grid->n; n++){
   for(m=0; m < grid->m; m++){
/* Update stresses for points one out from front face */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two out from front face using 2nd order scheme */
/*    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l-1,m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l-1,m,n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l-1,m, n); */
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l-1, m, n);
/*    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l-1, m, n); */
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l-1, m, n);  
   }

  }
}

dboundTKa(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
  int l, m, n;
  l = grid->l;   /* Points one out from back face */
  for(n=0; n < grid->n; n++){
   for(m=0; m < grid->m; m++){
/* Update stresses for points one out from back face */
    update4Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
    update4Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
    update4Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two out from back face using 2nd order scheme */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l+1,m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l+1,m,n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l+1,m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l+1, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l+1, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l+1, m, n); 
   }

  }

}

dboundTK(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
  int l, m, n;
  l = grid->l;   /* Points one out from back face */
  for(n=0; n < grid->n; n++){
   for(m=0; m < grid->m; m++){
/* Update stresses for points one out from back face */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
/* Update stresses for points two out from back face using 2nd order scheme */
/*    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz,l+1,m, n); */
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l+1,m,n);
/*    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l+1,m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l+1, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l+1, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l+1, m, n);  */
   }

  }
}


dedgeTBLa(par,grid,data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 int l,m,n;
 m = grid->m;    /* Points one down from bottom */
 n = -1;         /* Points one out from left */
 for (l=0; l< grid->l; l++) {    /* Points along bottom edge */
   
 /* Update stresses for points one out from bottom left edge */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
  /* Update stresses for points two out from bottom left edge */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n-1);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m+1, n);
  }
}

dedgeTBRa(par,grid,data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;

{
 int l,m,n;
 m = grid->m;    /* Points one down from bottom */
 n = grid->n;         /* Points one out from right */
 for (l=0; l< grid->l; l++) {    /* Points along bottom edge */
   
 /* Update stresses for points one out from bottom left edge */
    update2Txx3d(par, grid, data->Txx, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tyy3d(par, grid, data->Tyy, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m, n);
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n);
    update2Tyz3d(par, grid, data->Tyz, data->Vy, data->Vz, l, m, n);
  /* Update stresses for points two out from bottom right edge */
    update2Txy3d(par, grid, data->Txy, data->Vx, data->Vy, l, m, n+1);
    update2Txz3d(par, grid, data->Txz, data->Vx, data->Vz, l, m, n+1);
    update2Tzz3d(par, grid, data->Tzz, data->Vx, data->Vy, data->Vz, l, m+1, n);
  }
}

/***************************************************************************
VECTORIZED ABSORBING BOUNDARY ROUTINES (sides only)
***************************************************************************/



/* not debugged or complete (surface) */
vboundTRvector(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;
{
   float ***Vx, ***Vy, ***Vz, ***Txx, ***Tyy, ***Tzz, ***Txy, ***Txz, ***Tyz;
   float ***g, ***u, *dX, *dY, *dZ, *U, *G, *X0, *Y0, *Z0, *G0, *U0;
   float **G2, **U2, **Txx2, **Tyy2, **Tzz2, **Txy2, **Txz2, **Tyz2;
   float c;
   int l, m, n, d, p;

   /* initialize */
   Txx = data->Txx; Tyy = data->Tyy; Tzz = data->Tzz;
   Txy = data->Txy; Txz = data->Txz; Tyz = data->Tyz;
   Vx = data->Vx; Vy = data->Vy; Vz = data->Vz;
   g = grid->lamda;
   u = grid->mu;

   /* get array differencing limits and coefficient */
   p = data->mmin;
   c = 1.0*par->dt/grid->dx;

   /* Points one out from right boundary */
   n = grid->n;   

   /* update stresses for grid planes */
   X0 = (float  *) malloc(grid->m*sizeof(float));
   Y0 = (float  *) malloc(grid->m*sizeof(float));
   Z0 = (float  *) malloc(grid->m*sizeof(float));
   G0 = (float  *) malloc(grid->m*sizeof(float));
   U0 = (float  *) malloc(grid->m*sizeof(float));
   for(l = data->lmin; l < data->lmax; l++) {
      G2 = g[l]; U2 = u[l]; 
      Txx2 = Txx[l]; Tyy2 = Tyy[l]; Tzz2 = Tzz[l];
      Txy2 = Txy[l]; Txz2 = Txz[l]; Tyz2 = Tyz[l];
      G = G0+p; U = U0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         *(G++) = c*G2[m][n];
         *(U++) = c*U2[m][n];
         }
      dX = X0+p; dY = Y0+p; dZ = Z0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         *(dX++) = (Vx[l][m][n+1] - Vx[l][m][n]);
         *(dY++) = (Vy[l][m][n] - Vy[l-1][m][n]);
         *(dZ++) = (Vz[l][m][n] - Vz[l][m-1][n]);
         }
      dX = X0+p; dY = Y0+p; dZ = Z0+p;
      G  = G0+p; U  = U0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         Txx2[m][n] += (*dX)*((*G) + 2.0*(*U)) + ((*dY) + (*dZ))*(*G);
         Tyy2[m][n] += (*dY)*((*G) + 2.0*(*U)) + ((*dX) + (*dZ))*(*G);
         Tzz2[m][n] += (*dZ)*((*G) + 2.0*(*U)) + ((*dX) + (*dY))*(*G);
         U++; G++; dX++; dY++; dZ++;
         }
      G = G0+p; U = U0+p; dX = X0+p; dY = Y0+p; dZ = Z0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         Txy2[m][n] += (*U)*((Vx[l+1][m][n] - Vx[l][m][n]) + 
                               (Vy[l][m][n] - Vy[l][m][n-1]));
         Txz2[m][n] += (*U)*((Vx[l][m+1][n] - Vx[l][m][n]) + 
                               (Vz[l][m][n] - Vz[l][m][n-1]));
         Tyz2[m][n] += (*U)*((Vy[l][m+1][n] - Vy[l][m][n]) + 
                               (Vz[l+1][m][n] - Vz[l][m][n]));
         U++; G++;
         }
      G = G0+p; U = U0+p;
      n++;
      for(m = data->mmin; m < data->mmax; m++) {
         *(G++) = c*G2[m][n];
         *(U++) = c*U2[m][n];
         }
      G = G0+p; U = U0+p; 
      for(m = data->mmin; m < data->mmax; m++) {
         Txy2[m][n] += (*U)*((Vx[l+1][m][n] - Vx[l][m][n]) + 
                               (Vy[l][m][n] - Vy[l][m][n-1]));
         Txz2[m][n] += (*U)*((Vx[l][m+1][n] - Vx[l][m][n]) + 
                               (Vz[l][m][n] - Vz[l][m][n-1]));
         U++; G++;
         }
      n--;
      }

   /* free memory */
   free(X0);
   free(Y0);
   free(Z0);
   free(G0);
   free(U0);

}




/* from test.c */
/* file for testing c subroutines */


/* not made acoustic */
/* not debugged or complete (surface) */
aboundTRvector(par, grid, data)
Parameter *par;
FdGrid *grid;
VelocityStress *data;
{
   float ***Vx, ***Vy, ***Vz, ***Txx, ***Tyy, ***Tzz, ***Txy, ***Txz, ***Tyz;
   float ***g, ***u, *dX, *dY, *dZ, *U, *G, *X0, *Y0, *Z0, *G0, *U0;
   float **G2, **U2, **Txx2, **Tyy2, **Tzz2, **Txy2, **Txz2, **Tyz2;
   float c;
   int l, m, n, d, p;

   /* initialize */
   Txx = data->Txx; Tyy = data->Tyy; Tzz = data->Tzz;
   Txy = data->Txy; Txz = data->Txz; Tyz = data->Tyz;
   Vx = data->Vx; Vy = data->Vy; Vz = data->Vz;
   g = grid->lamda;
   u = grid->mu;

   /* get array differencing limits and coefficient */
   p = data->mmin;
   c = 1.0*par->dt/grid->dx;

   /* Points one out from right boundary */
   n = grid->n;   

   /* update stresses for grid planes */
   X0 = (float  *) malloc(grid->m*sizeof(float));
   Y0 = (float  *) malloc(grid->m*sizeof(float));
   Z0 = (float  *) malloc(grid->m*sizeof(float));
   G0 = (float  *) malloc(grid->m*sizeof(float));
   U0 = (float  *) malloc(grid->m*sizeof(float));
   for(l = data->lmin; l < data->lmax; l++) {
      G2 = g[l]; U2 = u[l]; 
      Txx2 = Txx[l]; Tyy2 = Tyy[l]; Tzz2 = Tzz[l];
      Txy2 = Txy[l]; Txz2 = Txz[l]; Tyz2 = Tyz[l];
      G = G0+p; U = U0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         *(G++) = c*G2[m][n];
         *(U++) = c*U2[m][n];
         }
      dX = X0+p; dY = Y0+p; dZ = Z0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         *(dX++) = (Vx[l][m][n+1] - Vx[l][m][n]);
         *(dY++) = (Vy[l][m][n] - Vy[l-1][m][n]);
         *(dZ++) = (Vz[l][m][n] - Vz[l][m-1][n]);
         }
      dX = X0+p; dY = Y0+p; dZ = Z0+p;
      G  = G0+p; U  = U0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         Txx2[m][n] += (*dX)*((*G) + 2.0*(*U)) + ((*dY) + (*dZ))*(*G);
         Tyy2[m][n] += (*dY)*((*G) + 2.0*(*U)) + ((*dX) + (*dZ))*(*G);
         Tzz2[m][n] += (*dZ)*((*G) + 2.0*(*U)) + ((*dX) + (*dY))*(*G);
         U++; G++; dX++; dY++; dZ++;
         }
      G = G0+p; U = U0+p; dX = X0+p; dY = Y0+p; dZ = Z0+p;
      for(m = data->mmin; m < data->mmax; m++) {
         Txy2[m][n] += (*U)*((Vx[l+1][m][n] - Vx[l][m][n]) + 
                               (Vy[l][m][n] - Vy[l][m][n-1]));
         Txz2[m][n] += (*U)*((Vx[l][m+1][n] - Vx[l][m][n]) + 
                               (Vz[l][m][n] - Vz[l][m][n-1]));
         Tyz2[m][n] += (*U)*((Vy[l][m+1][n] - Vy[l][m][n]) + 
                               (Vz[l+1][m][n] - Vz[l][m][n]));
         U++; G++;
         }
      G = G0+p; U = U0+p;
      n++;
      for(m = data->mmin; m < data->mmax; m++) {
         *(G++) = c*G2[m][n];
         *(U++) = c*U2[m][n];
         }
      G = G0+p; U = U0+p; 
      for(m = data->mmin; m < data->mmax; m++) {
         Txy2[m][n] += (*U)*((Vx[l+1][m][n] - Vx[l][m][n]) + 
                               (Vy[l][m][n] - Vy[l][m][n-1]));
         Txz2[m][n] += (*U)*((Vx[l][m+1][n] - Vx[l][m][n]) + 
                               (Vz[l][m][n] - Vz[l][m][n-1]));
         U++; G++;
         }
      n--;
      }

   /* free memory */
   free(X0);
   free(Y0);
   free(Z0);
   free(G0);
   free(U0);

}


/****************************************************************************
        attenuate3D: three dimensional attenuation. 

   NOTE: The global memory variables (q1, ..., q6) have the following 
   mapping within this funciton:
   
            rxx = q1
            rzz = q2
            rxz = q3
            ryy = q4
            ryx = q5
            ryz = q6
   
   NOTE: the memory variable now have the following format:
   
            q1[L][l][m][n]

   where-
            L - solids dim
            l - y dim
            m - z dim
            n - x dim
    
****************************************************************************/
attenuate3D(par, grid, data)
Parameter      *par;
FdGrid         *grid;
VelocityStress *data;
{
   float dt, *dv1, *dv2, *dv3, *dv4, *dv5, *dv6;
   float c1, c2, ***Vx, ***Vy, ***Vz;
   float *Txx, *Tyy, *Tzz, *Txy, *Tyz, *Txz;
   float *lamda, *mu, *F0, *F1, *F2, *F3;
   /* right now these are 1D, but later they'll have an L dim. */
   float *rxx, *ryy, *rzz, *rxz, *rxy, *ryz;
   float s1, s2, s3, s4, s5, D1, D2, D3;
   int dn, l, m, n, mmin, L;

   /* initialize and allocate memory for derivative arrays */
   Vx = data->Vx;  Vy = data->Vy;  Vz = data->Vz;
   dt = par->dt;
   c1 = C1/grid->dx;  c2 = C2/grid->dx;            /* dx == dy == dz */
   dv1 = (float *) malloc(grid->n*sizeof(float));
   dv2 = (float *) malloc(grid->n*sizeof(float));
   dv3 = (float *) malloc(grid->n*sizeof(float));
   dv4 = (float *) malloc(grid->n*sizeof(float));
   dv5 = (float *) malloc(grid->n*sizeof(float));
   dv6 = (float *) malloc(grid->n*sizeof(float));
   dn = data->nmax - data->nmin;  
   mmin = data->mmin;
   L = 0;

   /* if (mmin < 1) mmin = 1; */
   if (grid->bz1 == SURFACE && mmin < 1) mmin = 1;
 
   /* l in the outer loop */
   for (l = data->lmin; l < data->lmax; l++) {

     /* calculate dVx/dx, dVy/dy, dVz/dz derivatives; update diag. memory & stress */
     for (m = mmin; m < data->mmax; m++) {
       n = data->nmin;

       /* get the data and pointers we need */
       Txx = data->Txx   [l][m]; Tyy = data->Tyy    [l][m]; 
       Tzz = data->Tzz   [l][m]; F0  = grid->F0     [l][m]; 
       F1  = grid->F1    [l][m]; F2  = grid->F2     [l][m]; 
       F3  = grid->F3    [l][m]; rxx = data->q1     [l][m];
       rzz = data->q2    [l][m]; ryy = data->q4     [l][m];
       mu  = grid->mu    [l][m]; lamda = grid->lamda[l][m];
       
       /* get the derivatives, fortran function to compute vector of derivatives */
      dvf_(dv1+n,Vx[l][m]+n-1,Vx[l][m]+n,Vx[l][m]+n+1,Vx[l][m]+n+2,&c1,&c2,&dn);
/*
      dvf_(dv2+n,Vy[l-1][m]+n,Vy[l][m]+n,Vy[l+1][m]+n,Vy[l+2][m]+n,&c1,&c2,&dn);
*/
      dvf_(dv2+n,Vy[l-2][m]+n,Vy[l-1][m]+n,Vy[l][m]+n,Vy[l+1][m]+n,&c1,&c2,&dn);
      dvf_(dv3+n,Vz[l][m-2]+n,Vz[l][m-1]+n,Vz[l][m]+n,Vz[l][m+1]+n,&c1,&c2,&dn);
     
       /* In C: This does the same thing as the fortran function defined below. 
     See the file vectorf.f for the actual function. 
       for(n = data->nmin; n < data->nmax; n++) {
    s1 = rxx[n]; s2 = ryy[n]; s3 = rzz[n];
    rxx[n] = F0[n]*s1 + F1[n]*(dv1[n] + dv2[n] + dv3[n]) + f2[n]*(dv2[n] + dv3[n]);
    ryy[n] = F0[n]*s2 + F1[n]*(dv1[n] + dv2[n] + dv3[n]) + f2[n]*(dv1[n] + dv3[n]);
    rzz[n] = F0[n]*s3 + F1[n]*(dv1[n] + dv2[n] + dv3[n]) + f2[n]*(dv1[n] + dv2[n]);
    s1 = (s1 + rxx[n])*0.5;
    s2 = (s2 + ryy[n])*0.5;
    s3 = (s3 + rzz[n])*0.5;
    D1 = (lamda[n] + 2.*mu[n]);
         D2 = (lamda[n]);
    Txx[n] += dt*(D1*dv1[n] + D2*(dv2[n] + dv3[n]) + s1);
    Tyy[n] += dt*(D1*dv2[n] + D2*(dv1[n] + dv3[n]) + s2);
    Tzz[n] += dt*(D1*dv3[n] + D2*(dv1[n] + dv2[n]) + s3);
    }
    */
       
       da3f_(rxx+n, ryy+n, rzz+n, Txx+n, Tyy+n, Tzz+n, lamda+n, mu+n, F0+n, F1+n, F2+n, dv1+n, dv2+n, dv3+n, &dt, &dn);
     }
     
     
     /* now do the same as above with the off-diagonal terms */
     for (m = mmin; m < data->mmax; m++) {
       n = data->nmin;
       F0  = grid->F0   [l][m]; F3  = grid->F3   [l][m];
       Txz = data->Txz  [l][m]; Txy = data->Txy  [l][m];
       Tyz = data->Tyz  [l][m]; rxz = data->q3   [l][m];
       rxy = data->q5   [l][m]; ryz = data->q6   [l][m];
       mu  = grid->mu   [l][m];
       
       /* fortran function to compute vector of derivatives */
       dvf_(dv1+n,Vx[l][m-1]+n,Vx[l][m]+n,Vx[l][m+1]+n,Vx[l][m+2]+n,&c1,&c2,&dn);  /* dVx/dz */
       dvf_(dv2+n,Vy[l][m-1]+n,Vy[l][m]+n,Vy[l][m+1]+n,Vy[l][m+2]+n,&c1,&c2,&dn);  /* dVy/dz */
       dvf_(dv3+n,Vx[l-1][m]+n,Vx[l][m]+n,Vx[l+1][m]+n,Vx[l+2][m]+n,&c1,&c2,&dn);  /* dVx/dy */
       dvf_(dv4+n,Vz[l][m]+n-2,Vz[l][m]+n-1,Vz[l][m]+n,Vz[l][m]+n+1,&c1,&c2,&dn);  /* dVz/dx */
       dvf_(dv5+n,Vz[l-1][m]+n,Vz[l][m]+n,Vz[l+1][m]+n,Vz[l+2][m]+n,&c1,&c2,&dn);  /* dVz/dy */
       dvf_(dv6+n,Vy[l][m]+n-2,Vy[l][m]+n-1,Vy[l][m]+n,Vy[l][m]+n+1,&c1,&c2,&dn);  /* dVy/dx */
       
       /* In C: Same as routines in fortran 
       for(n = data->nmin; n < data->nmax; n++) { 
    s3 = rxy[n]; s4 = rxz[n]; s5 = rxy[n];
    rxy[n] = F0[n]*s3 + F3[n]*(dv3[n] + dv6[n]);
    rxz[n] = F0[n]*s4 + F3[n]*(dv1[n] + dv4[n]);
    ryz[n] = F0[n]*s5 + F3[n]*(dv2[n] + dv5[n]);
    s3 = (s3 + rxy[n])*0.5;
    s4 = (s4 + rxz[n])*0.5;
    s5 = (s5 + ryz[n])*0.5;
    Txy[n] += dt*mu[n]*(dv4[n] + dv1[n]) +dt*s3;
    Txz[n] += dt*mu[n]*(dv5[n] + dv2[n]) +dt*s4;
    Tyz[n] += dt*mu[n]*(dv6[n] + dv3[n]) +dt*s5;
       }
       */

       da4f_(rxy+n, rxz+n, ryz+n, Txy+n, Txz+n, Tyz+n, mu+n, F0+n, F3+n, dv1+n, dv2+n, dv3+n, dv4+n, dv5+n, dv6+n, &dt, &dn);
     }
   }

   /* free memory */
   free(dv1);
   free(dv2);
   free(dv3);
   free(dv4);
   free(dv5);
   free(dv6);
}


