/***************************************************************************
CHANGE TO POLY3D.H AND COMBINE GETVEL.H AND VELMOD.H
***************************************************************************/


/*  getvel.h

   include file for program getvel

*/

/* by Anthony Lomax */

/*   history:
      20 DEC 1996      Extracted from wlssyn.h
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <limits.h>

#ifdef EXTERN_MODE
#define   EXTERN_TXT extern
#else
#define EXTERN_TXT
#endif

/*******************************************************************
INCLUDE VELMOD.H DIRCTLY IN FILE 

#include "velmod.h"
********************************************************************/

/* velmod.h include file */

/*    History:
   
   see wls.c

*/




EXTERN_TXT double min_x_cut;   /* minimum x distance cutoff */
EXTERN_TXT double max_x_cut;   /* maximum x distance cutoff */
EXTERN_TXT double min_y_cut;   /* minimum y distance cutoff */
EXTERN_TXT double max_y_cut;   /* maximum y distance cutoff */
EXTERN_TXT double min_z_cut;   /* minimum y distance cutoff */
EXTERN_TXT double max_z_cut;   /* maximum y distance cutoff */

EXTERN_TXT double min_lon_cut;  /*for lat-lon entry*/
EXTERN_TXT double max_lon_cut;
EXTERN_TXT double min_lat_cut;
EXTERN_TXT double max_lat_cut;

/**************************************************************************/
/* model definitions */
/**************************************************************************/

   /* layered model */

struct layer {
    double dtop;      /* depth to top of layer */
    double vptop;      /* P velocity at top of layer */
    double vpgrad;     /* P vel gradient in layer */
    double vstop;      /* S velocity at top of layer */
    double vsgrad;     /* S vel gradient in layer */
    double dentop;      /* density at top of layer */
    double dengrad;     /* density gradient in layer */
    double qpref;       /* p attenuation in layer (Q value) */
    double qsref;       /* s attenuation in layer (Q value) */
    double qwref;       /* attenuation central frequency in layer (Hz) */
    int plot;           /* line plot style */
};

#define MAX_LAYERS 100
EXTERN_TXT struct layer model_layer[MAX_LAYERS];
EXTERN_TXT int num_layers;     /* number of layers read in */
#define LAYEROFFSET 1000      /* base offset for layer model element id */

   /* rough model */

#define MAX_SINUSOIDS 20
#define MAX_ROUGH 20
struct rough_bndry {
    double ref_lev;     /* ref level */
    double vptop;      /* P velocity at ref lev */
    double vpgrad;     /* P vel gradient in layer */
    double vstop;      /* S velocity at ref level */
    double vsgrad;     /* S vel gradient in layer */
    double dentop;     /* density at ref level */
    double dengrad;    /* density gradient in layer */
    double zmax;    /* max z coord of bndry */
    double zmin;    /* min z coord of bndry */
   int num_sin;      /* number of sin functions to define boundary */
   double amp[MAX_SINUSOIDS];   /* amplitude of sinusoid (km) */
   double wavelen[MAX_SINUSOIDS];   /* wavelength of sinusoid (km) */
   double phase[MAX_SINUSOIDS];   /* phase offset of sinusoid (km) */
    int plot;           /* line plot style */
};

EXTERN_TXT struct rough_bndry model_rough[MAX_ROUGH];
EXTERN_TXT int num_rough;     /* number of layers read in */

   /* disk inclusions */

struct disk {
    double x;         /* x loc of disk center */
    double z;         /* z loc of disk center */
    double rad2;    /* square of radius of disk */
    double pvel;    /* P velocity in disk */
    double svel;    /* S velocity in disk */
    double den;     /* density in disk */
    int plot;        /* line plot style */
};

#define MAX_DISKS 100
EXTERN_TXT struct disk model_disk[MAX_DISKS];   /* array of disk structures */

EXTERN_TXT int num_disks;     /* number of disks read in */

   /* sphere inclusions (3-D only) */

struct sphere {
    double z;         /* z loc of sphere center */
    double x;         /* x loc of sphere center */
    double y;         /* y loc of sphere center */
    double rad2;     /* square of radius of sphere */
    double pvel;     /* P velocity in sphere */
    double svel;     /* S velocity in sphere */
    double den;     /* density in sphere */
};

   /* array of sphere structures */

#define MAX_SPHERES 100
EXTERN_TXT struct sphere model_sphere[MAX_SPHERES];

EXTERN_TXT int num_spheres;     /* number of spheres read in */

/** polygon model */

   /* vertex structure */

struct vertex {
    struct vertex *prev;   /* pointer to previous vertex */
    struct vertex *next;   /* pointer to next vertex */
   int id_vtx;      /* vertex identification */
    double x;         /* x loc of vertex */
    double y;         /* y loc of vertex */
    double z;         /* z loc of vertex */
};

EXTERN_TXT struct vertex *vtx_head;
EXTERN_TXT int num_vtx;     /* number of vertices read in */

   /* edge structure */

struct edge {
    struct edge *prev;   /* pointer to previous edge */
    struct edge *next;   /* pointer to next edge */
   int id_edge;      /* edge identification */
    struct vertex *v1;       /* first vertex */
    struct vertex *v2;       /* second vertex */
   int edge_plot;      /* plotting flag, 1 = draw edge, 0 = no draw */
   int refl_mode;      /* reflection mode  0 = no, 1 = yes (2D SH only) */
};

EXTERN_TXT struct edge *edge_head;
EXTERN_TXT int num_edge;     /* number of edges read in */

   /* 2-D or 3-D polygon structure */

struct polygon {
    struct polygon *prev;   /* pointer to previous polygon */
    struct polygon *next;   /* pointer to next polygon */
   int id_poly;      /* polygon identification */
   int n_edges;      /* number of edges */
    struct edge **edges;   /* edge list */
   double xmin;         /* min x val of polygon */
   double xmax;         /* max x val of polygon */
   double ymin;         /* min y val of polygon */
   double ymax;         /* max y val of polygon */
   double zmin;         /* min z val of polygon */
   double zmax;         /* max z val of polygon */
    struct normals *norm;   /* array of normal coefficients */
   double ref;         /* velocity reference level */
    double vpref;      /* P velocity at reference level */
    double vpgrad;      /* P vel gradient */
    double vsref;      /* S velocity at reference level */
    double vsgrad;      /* S vel gradient */
    double denref;      /* density at reference level */
    double dengrad;      /* density gradient */
};

EXTERN_TXT struct polygon *poly_head;
EXTERN_TXT int num_poly;     /* number of polygons read in */
#define POLYOFFSET 2000      /* base offset for polygon model element id */

   /* 3-D solid structure */

struct solid {
    struct solid *prev;   /* pointer to previous solid */
    struct solid *next;   /* pointer to next solid */
   int id_solid;      /* solid identification */
   int n_poly;         /* number of polygonss */
    struct polygon **poly;   /* polygon list */
   double xmin;         /* min x val of solid */
   double xmax;         /* max x val of solid */
   double ymin;         /* min x val of solid */
   double ymax;         /* max x val of solid */
   double zmin;         /* min z val of solid */
   double zmax;         /* max z val of solid */
    struct normals *norm;   /* array of normal coefficients */
   double ref;         /* velocity reference level */
    double vpref;      /* P velocity at reference level */
    double vpgrad;      /* P vel gradient */
    double vsref;      /* S velocity at reference level */
    double vsgrad;      /* S vel gradient */
    double denref;      /* density at reference level */
    double dengrad;      /* density gradient */
    double qpref;       /* p attenuation in layer (Q value) */
    double qsref;       /* s attenuation in layer (Q value) */
    double qwref;       /* attenuation central frequency in layer (Hz) */
};

EXTERN_TXT struct solid *solid_head;
EXTERN_TXT int num_solid;     /* number of solids read in */
#define SOLIDOFFSET 3000   /* base offset for solid model element id */

/* edge or polygon normal coeffs (ax + by + cz + d = 0) */
struct normals {
   double a;
   double b;
   double c;
   double d;
};



/**************************************************************************/
/* function prototypes */
/**************************************************************************/

EXTERN_TXT int get_model_layer();
EXTERN_TXT int get_model_disk();
EXTERN_TXT double get_vel();
EXTERN_TXT int get_vel_den();
EXTERN_TXT double get_layer_vel();
EXTERN_TXT double get_disk_vel();
EXTERN_TXT double get_sphere_vel();

EXTERN_TXT struct vertex *addvtx();
EXTERN_TXT struct edge *addedge();
EXTERN_TXT struct polygon *addpoly();
EXTERN_TXT struct solid *addsolid();
EXTERN_TXT void disp_model_poly();
EXTERN_TXT double get_poly_vel();
EXTERN_TXT double get_solid_vel();
EXTERN_TXT double get_rough_z();
EXTERN_TXT double get_rough_vel();

EXTERN_TXT int draw_model();
EXTERN_TXT int draw_dist_axis();
EXTERN_TXT int draw_time_axis();

EXTERN_TXT double get_fdiff_vel();


/*

EXTERN_TXT int init_graph();
EXTERN_TXT int init_disp();
EXTERN_TXT int close_graph();
EXTERN_TXT drawline();
EXTERN_TXT int draw_front();
EXTERN_TXT setline();
EXTERN_TXT setcolor();
EXTERN_TXT int set_colortable();
EXTERN_TXT gtextw();
EXTERN_TXT char keypress();
EXTERN_TXT message();
*/

/*************************************************************
END OF VELMOD.H
*************************************************************/



#define OUT_LEVEL_1 stdout

#define MAXLINE 200
#define MAXFILE 200

/*#define REAL_MIN min_normal()*/
#define REAL_MIN ((double) 1.0e-30)
#define REAL_MAX ((double) 1.0e+30)
#define REAL_EPSILON ((double) 1.0e-6)

/* the following defines needed for old cc version */
#define SEEK_SET 0
#define SEEK_CUR 1
/*#define REAL_MAX max_normal()*/



   /* function declarations */

EXTERN_TXT int puterr();

   /* variable declarations */

EXTERN_TXT double pi, rpd, c111;
EXTERN_TXT double cosang, sinang;
EXTERN_TXT double vmodel_vmean;

EXTERN_TXT int prog_mode_3d;
EXTERN_TXT int coord_sys;

EXTERN_TXT char errstr[MAXLINE];
EXTERN_TXT char line[MAXLINE];
EXTERN_TXT char title[MAXLINE];
EXTERN_TXT char process[MAXLINE];
EXTERN_TXT char comment[MAXLINE];   /* input file comment */
EXTERN_TXT char run_comm[MAXLINE];
EXTERN_TXT char sta_comm[MAXLINE];

EXTERN_TXT char progname[80];      /* program name = argv[0] */
EXTERN_TXT char progver[30];      /* program version = VER */
EXTERN_TXT char progdate[30];      /* program date = DATE */

EXTERN_TXT FILE *fp_input;
EXTERN_TXT char fn_input[100];
EXTERN_TXT char fn_out[100];

EXTERN_TXT double map_orig_lat, map_orig_long, map_rot;


   /* vel fd grid model */

EXTERN_TXT int fdgrid_flag;
EXTERN_TXT int fdgrid_numx, fdgrid_numz, fdgrid_numy;
EXTERN_TXT double fdgrid_xmin, fdgrid_xmax;
EXTERN_TXT double fdgrid_zmin, fdgrid_zmax;
EXTERN_TXT double fdgrid_ymin, fdgrid_ymax;
EXTERN_TXT double fdgrid_xstep, fdgrid_zstep, fdgrid_ystep;
EXTERN_TXT float *fdgrid_array;
EXTERN_TXT char vfile_name[MAXFILE];   /* vgrid file name */





