#include   <stdio.h>
#include   <sys/file.h>
#include   <math.h>

/*
   grademodel takes an input file (normally named "run".grad,
   where "run" is the code name for the FD run).  From this,
   it constructs three files, pvelocity, svelocity, and density.
   The first line of the input file is the background medium
   specified as pvelocity, svelocity, density.

   In the input file, you may specify homogeneous polygons:
      nvert pvel svel den ipat
         x0     z0
         x1     z1
          .      .
          .      .
         xnvert znvert
         x0     z0
   where xn,zn are the x,z coordinates of the vertices; ipat
   is a pattern designator for other programs which draw the
   model; and nvert is the number of vertices (2 < nvert < 50).

   You may also specify polygons with gradients:
      -nvert dum dum dum ipat
         x0     z0     p0     s0     d0
         x1     z1     p0     s0     d0
          .      .      .      .      .
          .      .      .      .      .
         xnvert znvert pnvert snvert dnvert
         x0     z0     p0     s0     d0
   xn, zn, ipat are as above, nvert must be preceded by a '-' (this
   is used as a flag in the program); dum indicates a dummy float
   which must be present; and pn, sn, dn are the pvelocity, svelocity
   and density at the vertex.  (3 <= nvert <= 4; if more vertices
   are required, break region into 3 and 4 vertex polys).

   Required parameters:
      model="run".grad
      pvelocity="run".pvel
      svelocity="run".svel
      density="run".den
      nx=nx
      nz=nz
   Optional parameter:
      shift=1 (default is 0):
         this parameter tells grademodel to shift
         coordinates by half a grid point before
         calculating interpolated media parameters,
         allowing exact coorespondence to the offset
         grid scheme used to fill the polygons.
*/

double ap, bp, cp, dp,
       as, bs, cs1, ds,
       ad, bd, cd, dd;

#define Pfunc(X,Z) ( dp * X * Z + cp * Z + bp * X + ap )
#define Sfunc(X,Z) ( ds * X * Z + cs1* Z + bs * X + as )
#define Dfunc(X,Z) ( dd * X * Z + cd * Z + bd * X + ad )

struct vertex
   {
   int xv, zv;
   struct vertex *f;
   struct vertex *b;
   }   base[200];

struct gradvertex
   {
   int xv, zv;
   float pv, sv, dv;
   struct gradvertex *f;
   struct gradvertex *b;
   }   gradbase[200];

int crosses[2*200];

int shift=0;

grademodel(v, nx, nz, npts, ix, iz, value, gradient)
float **v, value, gradient;
int nx, nz, npts, *ix, *iz;
   {
   struct vertex *head, *p;
   struct gradvertex *ghead, *gp;
   int x, z;
   int xstr, xend, i;
   int zvmin, zvmax, ncross;
   int n;

      if(npts < 3)
         {
         fprintf(stderr,"polynomial with < 3 vertices");
         exit(-1);
         }

      /* Set up a double linked ring of vertices. The pointer
       * to the ring (head) is set to the node with the maximum
       * x-value so that intersect() will not eliminate 'head'
       * while casting off vertices.
       */
      zvmin= 2000000; zvmax= -2000000;
      head= base;
      for(i=0; i<npts; i++)
         {
         p= &base[i];
         p->f= p+1; p->b= p-1;
         x = ix[i]; z = iz[i];
         p->xv= x; p->zv= z;
         if(p->zv > zvmax) { zvmax= p->zv; head= p; }
         if(p->zv < zvmin)   zvmin= p->zv;
         }
      p->f= base; base->b= p; /* close the ring */
      /* check for closure */
      if(base->xv != x || base->zv !=  z)
         {
         fprintf(stderr,"polygon does not close");
         fprintf(stderr,"first vertex: %d %d\n",
            base->xv,base->zv);
         fprintf(stderr,"last  vertex: %d %d\n", x,z);
         exit(-1);
         }
      if(zvmax > (nz-1)) zvmax= (nz-1);
      if(zvmin < 0 ) {
         value += ((float) -zvmin)*gradient;
         zvmin= 0;
         }
      p= head;
      do p->zv = 2* p->zv + 1; while( (p=p->f) != head);
      for(z= zvmin; z <= zvmax; z++) {
         ncross= intersect(2*z,crosses,head);
         /*ncross= intersect(2*z + 1,crosses,head); test case*/
         sort(crosses,ncross);
         for(i=0; i< ncross; i += 2) {
            xstr= crosses[i];
            if (xstr<0) xstr=0;
            xend= crosses[i+1];
            if (xend>=nx) xend = nx-1;
            for(x=xstr; x<=xend; x++) v[z][x] = value;
            }
         value += gradient;
         }

   }


intersect(z,crosses,head)
register int z; register int *crosses; struct vertex *head;
   {
   register int ncross, x; register struct vertex *p;

   ncross= 0;
   p= head;
   do
      {
      if(p->zv > z && p->b->zv > z) continue;
      if(p->zv < z && p->b->zv < z)
         {
         if(p->f->zv > z) continue;
         /* eliminate vertex */
         p->b->f= p->f;
         p->f->b= p->b;
         continue;
         }
      x= solve(z,p->zv,p->xv,p->b->zv,p->b->xv);
      crosses[ncross++]= x;
      }   while( (p=p->f) != head );
   return(ncross);
   }

gintersect(z,crosses,head)
register int z; register int *crosses; struct gradvertex *head;
   {
   register int ncross, x; register struct gradvertex *p;

   ncross= 0;
   p= head;
   do
      {
      if(p->zv > z && p->b->zv > z) continue;
      if(p->zv < z && p->b->zv < z)
         {
         if(p->f->zv > z) continue;
         /* eliminate vertex */
         p->b->f= p->f;
         p->f->b= p->b;
         continue;
         }
      x= solve(z,p->zv,p->xv,p->b->zv,p->b->xv);
      crosses[ncross++]= x;
      }   while( (p=p->f) != head );
   return(ncross);
   }

sort(vec,n)
register int *vec; register int n;
   {
   /* sort the elements of vec into ascending order. */
   register int *above, *below, *last;
   register int temp;
   last = vec + (n-1);
   for(above=vec; above<last; above++)
      {
      for(below=above+1; below<=last; below++)
         {
         if(*above > *below)
            {
            temp=   *above;
            *above= *below;
            *below= temp;
            }
         }
      }
   }
solve(pnot,p1,q1,p2,q2)
register int pnot,p1,q1,p2,q2;
   {
   /* floating point version */
   double invslope;
   register int qnot;
   if(pnot==p1) return(q1);
   if(pnot==p2) return(q2);
   if(q1==q2) return(q1);
   invslope= (q1-q2)/( (double) (p1-p2));
   qnot= (pnot-p1)*invslope + (double) q1 + 0.5;
   return(qnot);
   }

interpol(verts,nvert)
/*
   fits a polynomial surface to specified values of
   p-velocity, s-velocity and density (in turn) for
   three (plane) or four specified points.  Uses
   gaussian elimination - partial pivoting subroutines.
*/
struct gradvertex *verts;
int nvert;
{
   double **a, b[4];
   int *rows;
   double **init_pivot();
   int solve_pivot();
   void pivot();
   void backsub();
   void reaugment();

   switch (nvert) {

   case 3:
      /*
         Form of equation:
             | a00 a01 a02 |   | ap |   | a03 |
             | a10 a11 a12 | * | bp | = | a13 |
             | a20 a21 a22 |   | cp |   | a23 |
         Equation derived from
             ap + bp * xv + cp * zv + dp * xv * zv = pv,
         one equation for each vertex.  (For just three
         vertices, dp = 0.)  an0 = 1.  an3 are the media
         parameters specified at each grid point.
      */

      a = init_pivot(nvert,&rows);
      a[0][0] = 1; a[0][1] = verts[0].xv;
       a[0][2] = verts[0].zv + shift * 0.5;
      a[1][0] = 1; a[1][1] = verts[1].xv;
       a[1][2] = verts[1].zv + shift * 0.5;
      a[2][0] = 1; a[2][1] = verts[2].xv;
       a[2][2] = verts[2].zv + shift * 0.5;
      /* do p-velocity first */
      a[0][3] = verts[0].pv;
      a[1][3] = verts[1].pv;
      a[2][3] = verts[2].pv;
      if (solve_pivot(a, rows, nvert) < 0) {
         fprintf(stderr,"Singular matrix, check input vertices!\n");
         exit(-1);
      }
      backsub(a, nvert);
      ap = a[0][3];
      bp = a[1][3];
      cp = a[2][3];
      dp = 0.0;
      /* now do s-velocity using reaugmentation of a */
      b[0] = verts[0].sv;
      b[1] = verts[1].sv;
      b[2] = verts[2].sv;
      reaugment(a, b, rows, nvert);
      backsub(a, nvert);
      as = a[0][3];
      bs = a[1][3];
      cs1= a[2][3];
      ds = 0.0;
      /* finally, do density */
      b[0] = verts[0].dv;
      b[1] = verts[1].dv;
      b[2] = verts[2].dv;
      reaugment(a, b, rows, nvert);
      backsub(a, nvert);
      ad = a[0][3];
      bd = a[1][3];
      cd = a[2][3];
      dd = 0.0;
      break;
   
   case 4:
      /*
         Form of equation:
             | a00 a01 a02 a03 |   | ap |   | a04 |
             | a10 a11 a12 a13 | * | bp | = | a14 |
             | a20 a21 a22 a23 |   | cp |   | a24 |
             | a30 a31 a32 a33 |   | cp |   | a24 |
         Equation derived from
             ap + bp * xv + cp * zv + dp * xv * zv = pv,
         one equation for each vertex.  (For just three
         vertices, dp = 0.)  an0 = 1.  an4 are the media
         parameters specified at each grid point.
      */

      a = init_pivot(nvert,&rows);
      a[0][0] = 1; a[0][1] = verts[0].xv;
       a[0][2] = verts[0].zv + shift * 0.5;
       a[0][3] = verts[0].xv * a[0][2];
      a[1][0] = 1; a[1][1] = verts[1].xv;
       a[1][2] = verts[1].zv + shift * 0.5;
       a[1][3] = verts[1].xv * a[1][2];
      a[2][0] = 1; a[2][1] = verts[2].xv;
       a[2][2] = verts[2].zv + shift * 0.5;
       a[2][3] = verts[2].xv * a[2][2];
      a[3][0] = 1; a[3][1] = verts[3].xv;
       a[3][2] = verts[3].zv + shift * 0.5;
       a[3][3] = verts[3].xv * a[3][2];
      /* do p-velocity first */
      a[0][4] = verts[0].pv;
      a[1][4] = verts[1].pv;
      a[2][4] = verts[2].pv;
      a[3][4] = verts[3].pv;
      if (solve_pivot(a, rows, nvert) < 0) {
         fprintf(stderr,"Singular matrix, check input vertices!\n");
         exit(-1);
      }
      backsub(a, nvert);
      ap = a[0][4];
      bp = a[1][4];
      cp = a[2][4];
      dp = a[3][4];
      /* now do s-velocity using reaugmentation of a */
      b[0] = verts[0].sv;
      b[1] = verts[1].sv;
      b[2] = verts[2].sv;
      b[3] = verts[3].sv;
      reaugment(a, b, rows, nvert);
      backsub(a, nvert);
      as = a[0][4];
      bs = a[1][4];
      cs1= a[2][4];
      ds = a[3][4];
      /* finally, do density */
      b[0] = verts[0].dv;
      b[1] = verts[1].dv;
      b[2] = verts[2].dv;
      b[3] = verts[3].dv;
      reaugment(a, b, rows, nvert);
      backsub(a, nvert);
      ad = a[0][4];
      bd = a[1][4];
      cd = a[2][4];
      dd = a[3][4];
      break;
   
   default:
      fprintf(stderr,"Error in interpol:  %d vertices unsupported\n",
         nvert);
      exit(-1);
   }
}



/* FOLLOWING IS DSYSSOLVE.C */

/*
   Matrix A is specified as a[row][column]; that is
   a[row] is a pointer to an array that contains all
   the values for that row.
*/

#define SMALL 1.0e-100

double ** init_pivot(dimension,rows)
/*
   Initializes the system solver by providing an array to place
   the augmented matrix in.  Also holds the original row order
   for use by reaugment().
*/
int dimension;
int **rows;
{
   double **temp_ptr;
   int i;

   temp_ptr = (double **) malloc(sizeof(double *)*dimension);
   rows[0]  = (int    *) malloc(4*dimension);
   for (i=0; i<dimension; i++) {
      temp_ptr[i] = (double *) malloc(sizeof(double)*(dimension+1));
      rows[0][i] = i;
   }
   return(temp_ptr);
}

int solve_pivot(a, rows, n)
/*
   Solves a system of n linear equations with n unknowns by
   Gaussian elimination with partial pivoting.  Call init_pivot()
   first to provide appropriate storage of the augmented
   matrix.  Solve_pivot provides an upper triangular augmented
   matrix (overwriting a) to be used by the routine backsub()
   for back substitution.  The lower triangular matrix is also
   stored (in the zeroed portion of a) for use by reaugment().
*/
double **a;
int *rows, n;
{
   double len, max, *hold;
   int i, j, k, ihold, ret_val;
   void pivot();

   /* find "length" of matrix a */
   len = 0.;
   for (i=0; i<n; i++)
      for (j=0; j<n; j++)
         len += a[i][j] * a[i][j];
   len *= SMALL / (double) (n*n);
   ret_val = 0;

   for (i=0; i<n-1; i++) {

      /* check if all values in column are much smaller than length */
      max = fabs(a[i][i]);
      k = i;
      for (j=i+1; j<n; j++) {
         if (max < fabs(a[j][i])) {
            max = fabs(a[j][i]);
            k = j;
         }
      }
      if (max < len) {
         fprintf(stderr,"Solve_pivot: Matrix looks singular, column %d\n",i);
         ret_val = -1;
      }
      
      /* interchange rows */
      if (i != k) {
         hold = a[k];
         a[k] = a[i];
         a[i] = hold;
         ihold = rows[k];
         rows[k] = rows[i];
         rows[i] = ihold;
      }

      pivot(a, n, i);
   }
   return(ret_val);
}

void pivot(a, n, k)
/*
   Performs pivoting operation on lower right-hand submatrix.
   (Gaussian elimination... the row interchange is done before call.)
   Element a[k][k] is the pivot.  The lower triangular matrix
   (multipliers) is stored in the zeroed portion of the upper
   triangular matrix for future reference (the unit diagonal
   elements of the lower triangular matrix are implicit).
*/
double **a;
int n, k;
{
   double ratio;
   int i, j;

   for (i=k+1; i<n; i++) {
      ratio = -a[i][k] / a[k][k];
      a[i][k] = ratio;      /* store lower tri. value */
      for (j=k+1; j<=n; j++)
         a[i][j] += ratio * a[k][j];
   }
}

void backsub(a, n)
/*
   Performs backsubstitiution to get unknowns.
*/
double **a;
int n;
{
   double sum;
   int i, j;

   a[n-1][n] /= a[n-1][n-1];
   for (i=n-2; i>=0; i--) {
      sum = 0.0;
      for (j=i+1; j<n; j++)
         sum += a[i][j] * a[j][n];
      a[i][n] -= sum;
      a[i][n] /= a[i][i];
   }
}

void reaugment(a, b, rows, n)
/*
   Uses the lower triangular matrix saved from solve_pivot to
   reconstruct a new augmented matrix for any given vector b
   in A x = b.
*/
double **a, *b;
int *rows, n;
{
   int i, j;
   double sum;

   for (i=0; i<n; i++) {
      sum = 0.0;
      for (j=0; j<i; j++)
         sum += a[i][j] * a[j][n];
      sum += b[rows[i]];
      a[i][n] = sum;
   }
}
