/******************************************************************************
   sacutil.h: include file for SAC utility routines

   author: shawn larsen
   date:   june 1992
   where:  llnl
******************************************************************************/



/*****************
GENERAL PARAMETERS
*****************/

#define SACWORDS                   158  /* number of words in SAC header    */ 
#define WORDSIZE                     4  /* word size (bytes)                */
#define SACSIZE    (SACWORDS*WORDSIZE)  /* number of bytes in SAC header    */
#define HVERSION                     6  /* header version number            */
#define INTERNAL                     0  /* value of some internal variables */
#define OVERWRITE                    1  /* file overwrite permission        */
#define CALDA                        1  /* calculate from coordinates       */ 



/***********************
NUMBER OF EACH DATA TYPE
***********************/

#define FWORDS     70         /* number of floats   (words) */
#define IWORDS     35         /* number of ints     (words) */
#define LWORDS      5         /* number of logicals (words) */ 
#define KWORDS     48         /* number of character words  */



/*************************
DATA TYPE UNDEFINED VALUES 
*************************/

#define FVALUE   -12345.0      /* undefined floating point value */
#define IVALUE   -12345        /* undefined integer        value */
#define LVALUE        0        /* undefined logical        value */
#define KVALUE  "-12345  "     /* undefined character      value */



/********************************
WORD OFFSETS OF SPECIAL VARIABLES
********************************/

#define DNVHDR       76       /* offset (words) of header version number */
#define DINTERNAL4   77       /* offset (words) of 4th internal variable */
#define DINTERNAL5   78       /* offset (words) of 5th internal variable */
#define DLOVROK     107       /* offset (words) of file overwrite flag   */
#define DCALDA      108       /* offset (words) calculate flag           */



/********************************
WORD OFFSETS OF CERTAIN VARIABLES
********************************/

#define DDELTA      0         /* offset (words) of delta  */
#define DDEPMIN     1         /* offset (words) of depmin */ 
#define DDEPMAX     2         /* offset (words) of depmax */
#define DB          5         /* offset (words) of b      */
#define DE          6         /* offset (words) of e      */
#define DNPTS      79         /* offset (words) of npts   */
#define DIFTYPE    85         /* offset (words) of iftype */
#define DLEVEN    105         /* offset (words) of leven  */



/***********************************
WORD OFFSETS OF ADDITIONAL VARIABLES
***********************************/

#define DO          7         /* offset (words) of o      */
#define DSTLA      31         /* offset (words) of stla   */
#define DSTLO      32         /* offset (words) of stlo   */
#define DSTEL      33         /* offset (words) of stel   */
#define DSTDP      34         /* offset (words) of stdp   */
#define DEVLA      35         /* offset (words) of evla   */
#define DEVLO      36         /* offset (words) of evlo   */
#define DEVEL      37         /* offset (words) of evel   */
#define DEVDP      38         /* offset (words) of evdp   */
#define DCMPAZ     57         /* offset (words) of cmpaz  */
#define DCMPINC    58         /* offset (words) of cmpinc */
#define DNZYEAR    70         /* offset (words) of nzyear */
#define DNZJDAY    71         /* offset (words) of nzjday */
#define DNZHOUR    72         /* offset (words) of nzhour */
#define DNZMIN     73         /* offset (words) of nzmin  */
#define DNZSEC     74         /* offset (words) of nzsec  */
#define DNZMSEC    75         /* offset (words) of nzmsec */
#define DKSTNM    110         /* offset (words) of kstnm  */
#define DKCMPNM   150         /* offset (words) of kcmpnm */



/*********************
SHORT HEADER STRUCTURE
*********************/

struct sac_parameter
            {

            /* main parameters                          */
            float   delta;     /* type increment        */
            float  depmin;     /* minimum value         */
            float  depmax;     /* maximum value         */
            float       b;     /* starting time         */
            float       e;     /* ending   time         */
            int      npts;     /* number of points      */
            int    iftype;     /* type of file          */
            int     leven;     /* spacing of data       */

            /* additional parameters                    */
            float         o;     /* event origin time     */
            float      stla;     /* station latitude      */
            float      stlo;     /* station longitude     */
            float      stel;     /* station elevation     */
            float      stdp;     /* station depth         */
            float      evla;     /* event   latitude      */
            float      evlo;     /* event   longitude     */
            float      evel;     /* event   elevation     */
            float      evdp;     /* event   depth         */
            float    cmpaz ;     /* component azimuth     */
            float    cmpinc;     /* component inclination */
            int      nzyear;     /* reference year        */
            int      nzjday;     /* reference julian day  */
            int      nzhour;     /* reference hour        */
            int      nzmin ;     /* reference minute      */
            int      nzsec ;     /* reference second      */
            int      nzmsec;     /* reference millisecond */
            char  kstnm[10];     /* sta name  (x int dis) */
            char kcmpnm[10];     /* component (x, y, z)   */

            };



/*************
REDECLARATIONS
*************/

typedef struct sac_parameter SACParameter;
