/* file for testing c subroutines */

#include "elas.h"

nothing() {;}

/* topography subroutines to check if point is above surface */
getAirPoint(par, grid, l, m, n)
Parameter    *par;
FdGrid      *grid;
int       l, m, n;
{
   float z;
/*
return(0);
*/
   z = grid->z0 + ((float) (m-1))*grid->dz;
   /*n = grid->n0 + n*grid->s;*/
   n =            n*grid->s;
   if (par->d == 2) if (z >= grid->zi    [n]) return(0);
   if (par->d == 3) if (z >= grid->ziM[l][n]) return(0);
   return(1);
}


/* note: not set up for multigrid */
getSourceD(par, grid, source)
Parameter    *par;
FdGrid       *grid;
FdSource     *source;
{
   int i;
   for(i = 0; i < par->f; i++) if (source[i].type == CSOURCE)
      getDistributedSource(par, grid, source + i);
   for(i = 0; i < par->f; i++) if (source[i].type == ESOURCE)
      getDistributedSource(par, grid, source + i);
}


getDistributedSource(par, grid, source)
Parameter    *par;
FdGrid       *grid;
FdSource     *source;
{
   int i, l, m, n, t;
   float time, amplitude, rise, factor, rintegral;
   char line[82];
   FILE *file = NULL;

   /* open distributed slip input file */
   file = fopen(source->dfile, "r");
   if (file == NULL) {
      fprintf(stderr, "Error: Can't open distributed slip file \"%s\"\n",
                       source->dfile);
      exit(-1);
      }

   /* read file */
   i = 0;
   factor = (float) (1./sqrt((PI)));
   while(i < source->p) {
      if (fgets(line, 82, file) == NULL) {
         fprintf(stderr, "Error: premature end to distributed slip file\n"); 
         exit(-1);
         }
      sscanf(line, "%d %d %d %d %f", &l, &m, &n, &t, &amplitude);
      source->dfl[i] = l;
      source->dfm[i] = m;
      source->dfn[i] = n;
      source->dft[i] = t;
      source->dfa[i] = amplitude;
      if (source->type == ESOURCE) {
         sscanf(line, "%d %d %d %d %f %f", &l, &m, &n, &t, &amplitude, &rise);
/*
         slip /= 1.;
         mu = 3.e10;
         l = l - grid->l0;
         m = m - grid->m0;
         n = n - grid->n0;
         if (l > -1 && l <  grid->l &&
            m > -1 && m <  grid->m &&
            n > -1 && n <  grid->n   ) mu = grid->mu[l][m][n];
         source->dfa[i] = mu*10.*par->dx*par->dx*100.*100.*slip;
         rise *= 1.33333;
         if (rise < 0.67) rise = 0.67;
         if (rise < 1.00) rise = 1.00;
*/
         source->dfr[i] = rise;
         rintegral = factor*rise;
         source->dfa[i] /= rintegral;
         }
      i++;
      }
      if (source->type == ESOURCE && par->node == 0) {
         fprintf(stderr, "***rise time conversions in source\n");
         }

   /* close file */
   fclose(file);

}


addSourceT(par, grid, data, source, t)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int                t;
{
   int i, g;
   if (grid->d == 2) for(g = 0; g < par->g; g++) for(i = 0; i < par->f; i++) {
      if (source[i].type == BSOURCE) {
         addFaultSource(par, grid + g, data + g, source + i, t);
         }
      else if (source[i].type == CSOURCE) {
         addDistributedSource(par, grid + g, data + g, source + i, t);
         } 
      else if (source[i].type == ESOURCE) {
         addDistributedSource2(par, grid + g, data + g, source + i, t);
         } 
      else {
         addSourceT2D(par, grid + g, data + g, source + i, t);
         }
      }
   if (grid->d == 3) for(g = 0; g < par->g; g++) for(i = 0; i < par->f; i++) {
      if (source[i].type == BSOURCE) {
         addFaultSource(par, grid + g, data + g, source + i, t);
         }
      else if (source[i].type == CSOURCE) {
         addDistributedSource(par, grid + g, data + g, source + i, t);
         } 
      else if (source[i].type == ESOURCE) {
         addDistributedSource2(par, grid + g, data + g, source + i, t);
         } 
      else {
         addSourceT3D(par, grid + g, data + g, source + i, t);
         }
      }
}


addSourceT2D(par, grid, data, source, t)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int                t;
{
   float M, F, f, ft, dx, dy, dz, ds, dv, dt, irho, dy2d;
   int l, m, n;

   /* determine source location */
   dx = grid->dx;
   dy = grid->dy;
   dz = grid->dz;
   l  = 0;
   n  = (int) floor((source->x - grid->x0)/dx);
   m  = (int) floor((source->z - grid->z0)/dz);
   if (n < -1 || n > grid->n) return(0);
   if (m < -1 || m > grid->m) return(0);

   if ((f = source->f[t]) == 0.0) return(0);
   M  = f;
   F  = f;
   dt = par->dt;
   ds = dx   *dz;
   dv = dx*dy*dz;
   dy2d = dy/(KMTOM); /* force-density in y direction because of line source */
   irho = grid->irho[l][m][n];
   switch(source->type) {
      case PSOURCE:
         data->Txx[l  ][m  ][n  ] += -M*dt            /dv*dy2d;
         data->Tzz[l  ][m  ][n  ] += -M*dt            /dv*dy2d;
         break;
      case MSOURCE:
         data->Txx[l  ][m  ][n  ] += -M*dt*source->Mxx/dv*dy2d     ;
         data->Tzz[l  ][m  ][n  ] += -M*dt*source->Mzz/dv*dy2d     ;
         data->Txz[l  ][m  ][n  ] += -M*dt*source->Mxz/dv*dy2d*0.25;
         data->Txz[l  ][m  ][n+1] += -M*dt*source->Mxz/dv*dy2d*0.25;
         data->Txz[l  ][m-1][n  ] += -M*dt*source->Mxz/dv*dy2d*0.25;
         data->Txz[l  ][m-1][n+1] += -M*dt*source->Mxz/dv*dy2d*0.25;
         break;
      case FSOURCE:
         data->Vx[l  ][m  ][n  ] += dt*((( F*source->Fx    )/dv)*dy2d)*irho/2.;
         data->Vx[l  ][m  ][n+1] += dt*((( F*source->Fx    )/dv)*dy2d)*irho/2.;
         data->Vz[l  ][m-1][n  ] += dt*((( F*source->Fz    )/dv)*dy2d)*irho/2.;
         data->Vz[l  ][m  ][n  ] += dt*((( F*source->Fz    )/dv)*dy2d)*irho/2.;
         break;
      case RSOURCE:
         data->Vx [l  ][m  ][n  ] +=    ((( F*source->Fx    )   )     )        ;
         data->Vz [l  ][m  ][n  ] +=    ((( F*source->Fz    )   )     )        ;
         data->Tzz[l  ][m  ][n  ] += -M   *source->Mzz             ;
         /*
         data->Vx[l  ][m  ][n  ] +=    ((( F*source->Fx    )   )     )     /2.;
         data->Vx[l  ][m  ][n+1] +=    ((( F*source->Fx    )   )     )     /2.;
         data->Vz[l  ][m-1][n  ] +=    ((( F*source->Fz    )   )     )     /2.;
         data->Vz[l  ][m  ][n  ] +=    ((( F*source->Fz    )   )     )     /2.;
         */
         break;
      case SSOURCE:
           data->Vx[l  ][m  ][n  ] += dt*(((-M            /dx)/dv)*dy2d)*irho;
         data->Vx[l  ][m+1][n  ] += dt*((( M            /dx)/dv)*dy2d)*irho;
           data->Vz[l  ][m  ][n  ] += dt*(((-M            /dz)/dv)*dy2d)*irho;
         data->Vz[l  ][m  ][n-1] += dt*((( M            /dz)/dv)*dy2d)*irho;
         break;
      case TSOURCE:
         data->Txx[l ][m  ][n  ] += f;
         data->Tzz[l ][m  ][n  ] += f;
         fprintf(stderr, "Error: TSOURCE non-functional\n");
         exit(-1);
         break;
      default:
         fprintf(stderr, "ERROR: unknown source type %d\n", source->type);
         exit(-1);
         break;
      }

}



addSourceT3D(par, grid, data, source, t)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int                t;
{
   float M, F, f, ft, dx, dy, dz, dv, dt, irho;
   int l, m, n;

   /* determine source location */
   dx = grid->dx;
   dy = grid->dy;
   dz = grid->dz;
   n  = (int) floor((source->x - grid->x0)/dx);
   m  = (int) floor((source->z - grid->z0)/dz);
   l  = (int) floor((source->y - grid->y0)/dy);
   if (n < -1 || n > grid->n) return(0);
   if (m < -1 || m > grid->m) return(0);
   if (l < -1 || l > grid->l) return(0);

   if ((f = source->f[t]) == 0.0) return(0);
   M  = f;
   F  = f;
   dt = par->dt;
   dv = dx*dy*dz;
   irho = grid->irho[l][m][n];
   switch(source->type) {
      case PSOURCE:
         if (par->model != 2) {
         data->Txx[l  ][m  ][n  ] += -M*dt            /dv;
         data->Tyy[l  ][m  ][n  ] += -M*dt            /dv;
         data->Tzz[l  ][m  ][n  ] += -M*dt            /dv;
         }
         if (par->model == 2)
         data->T  [l  ][m  ][n  ] += -M*dt            /dv;
         break;
      case MSOURCE:
         data->Txx[l  ][m  ][n  ] += -M*dt*source->Mxx/dv     ;
         data->Tyy[l  ][m  ][n  ] += -M*dt*source->Myy/dv     ;
         data->Tzz[l  ][m  ][n  ] += -M*dt*source->Mzz/dv     ;
         data->Txz[l  ][m  ][n  ] += -M*dt*source->Mxz/dv*0.25;
         data->Txz[l  ][m  ][n+1] += -M*dt*source->Mxz/dv*0.25;
         data->Txz[l  ][m-1][n  ] += -M*dt*source->Mxz/dv*0.25;
         data->Txz[l  ][m-1][n+1] += -M*dt*source->Mxz/dv*0.25;
         data->Txy[l  ][m  ][n  ] += -M*dt*source->Mxy/dv*0.25;
         data->Txy[l  ][m  ][n+1] += -M*dt*source->Mxy/dv*0.25;
         data->Txy[l-1][m  ][n  ] += -M*dt*source->Mxy/dv*0.25;
         data->Txy[l-1][m  ][n+1] += -M*dt*source->Mxy/dv*0.25;
         data->Tyz[l  ][m  ][n  ] += -M*dt*source->Myz/dv*0.25;
         data->Tyz[l  ][m-1][n  ] += -M*dt*source->Myz/dv*0.25;
         data->Tyz[l-1][m  ][n  ] += -M*dt*source->Myz/dv*0.25;
         data->Tyz[l-1][m-1][n  ] += -M*dt*source->Myz/dv*0.25;
         break;
      case FSOURCE:
         data->Vx[l  ][m  ][n  ] += dt*(( F*source->Fx    )/dv)*irho/2.;
         data->Vx[l  ][m  ][n+1] += dt*(( F*source->Fx    )/dv)*irho/2.;
         data->Vy[l-1][m  ][n  ] += dt*(( F*source->Fy    )/dv)*irho/2.;
         data->Vy[l  ][m  ][n  ] += dt*(( F*source->Fy    )/dv)*irho/2.;
         data->Vz[l  ][m-1][n  ] += dt*(( F*source->Fz    )/dv)*irho/2.;
         data->Vz[l  ][m  ][n  ] += dt*(( F*source->Fz    )/dv)*irho/2.;
         break;
      case RSOURCE:
         data->Vx [l  ][m  ][n  ] +=    (( F*source->Fx    )   )        ;
         data->Vy [l  ][m  ][n  ] +=    (( F*source->Fy    )   )        ;
         data->Vz [l  ][m  ][n  ] +=    (( F*source->Fz    )   )        ;
         data->Txx[l  ][m  ][n  ] += -M   *source->Mxx        ;
         data->Tyy[l  ][m  ][n  ] += -M   *source->Myy        ;
         data->Tzz[l  ][m  ][n  ] += -M   *source->Mzz        ;
         /*
         data->Vx[l  ][m  ][n  ] +=    (( F*source->Fx    )   )     /2.;
         data->Vx[l  ][m  ][n+1] +=    (( F*source->Fx    )   )     /2.;
         data->Vy[l-1][m  ][n  ] +=    (( F*source->Fy    )   )     /2.;
         data->Vy[l  ][m  ][n  ] +=    (( F*source->Fy    )   )     /2.;
         data->Vz[l  ][m-1][n  ] +=    (( F*source->Fz    )   )     /2.;
         data->Vz[l  ][m  ][n  ] +=    (( F*source->Fz    )   )     /2.;
         */
         break;
      case SSOURCE:
           data->Vx[l  ][m  ][n  ] += dt*((-M            /dx)/dv)*irho;
         data->Vx[l  ][m+1][n  ] += dt*(( M            /dx)/dv)*irho;
           data->Vz[l  ][m  ][n  ] += dt*((-M            /dz)/dv)*irho;
         data->Vz[l  ][m  ][n-1] += dt*(( M            /dz)/dv)*irho;
         fprintf(stderr, "Error: SSOURCE non-functional in 3D\n");
         exit(-1);
         break;
      case TSOURCE:
         data->Txx[l ][m  ][n  ] += f;
         data->Tyy[l ][m  ][n  ] += f;
         data->Tzz[l ][m  ][n  ] += f;
         fprintf(stderr, "Error: TSOURCE non-functional\n");
         exit(-1);
         break;
      default:
         fprintf(stderr, "ERROR: unknown source type %d\n", source->type);
         exit(-1);
         break;
      }

}



addFaultSource(par, grid, data, source, t)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int                t;
{
   double sqrt(), fabs();
   float x, y, z, d, w, strike, dip, rake; 
   float angle, x0, x1, y0, y1, z0, z1, dx, dy, dz, dw, df; 
   float dlength, dwidth, dh, dx0, dy0, s;
   float v, distance, time, fx0, fy0, fz0;
   float M, f1, f4, dt, dv, dy2d;
   float Mxx, Myy, Mzz, Mxy, Mxz, Myz;
   int l, m, n, l0, m0, n0, tindex, fpoint;

/*
addFaultSource2(par, grid, data, source, t);
return(0);
*/

   /* initialize */
   d = source->length; w = source->width;
   strike = source->strike; dip = source->dip;
   v = source->v;
   if (v <= 0.) v = 0.8*par->minS;
   if (v <= 0.) v = 0.5*par->minP;

   /* recalculate parameters for 2D problems (apparent dip) */
   if (par->d == 2 && fabs(dip) != 90.) {
      if ((s = sin(strike + 90.)/(RAD)) == 0.) {
         fprintf(stderr, "Error: invalid 2D strike\n");
         exit(-1);
         }
      if (sin(strike/(RAD)) != 0.) {
         fprintf(stderr, "Error: can't have non 0 (180) strike in 2D\n");
         exit(-1);
         }
      }

   /* determine global coordinates */
   angle = strike/(RAD);
   x0 = source->x - (d/2.)*sin(strike/(RAD));
   y0 = source->y - (d/2.)*cos(strike/(RAD));
   x1 = source->x + (d/2.)*sin(strike/(RAD));
   y1 = source->y + (d/2.)*cos(strike/(RAD));
   /*
   z0 = source->z;
   */
   z0 = source->depth;

   /* determine coordinates at top of fault */
   /*
   dh = 0.;
   if (dip != 0.) dh = (source->depth - z0)*cos(dip/(RAD))/sin(dip/(RAD));
   x0 += dh*cos(strike/(RAD));
   y0 -= dh*sin(strike/(RAD));
   z0  = source->depth;
   */

   /* determine source coordinates */
   fx0  = source->x + source->s0*sin(strike/(RAD));
   fy0  = source->y + source->s0*cos(strike/(RAD));
   dh   = source->d0*cos(dip/(RAD));
   if (dip < 0.) dh = -dh;
   fx0 += dh*cos(strike/(RAD));
   fy0 -= dh*sin(strike/(RAD));
   fz0  = z0 + fabs(source->d0*sin(dip/(RAD)));
   
   /* determine increments */ /**** VERTICAL ONLY *****/
   dx = par->dx*sin(strike/(RAD));
   dy = par->dy*cos(strike/(RAD));
   if (fabs(dx) >= fabs(dy)) {
      if (dx >= 0.) dx =  par->dx;
      if (dx <  0.) dx = -par->dx;
/***add ***/
      dy = dx*tan((90. - strike)/(RAD));
/***/
      dlength = sqrt(dx*dx + dy*dy);
      }
   else {
      if (dy >= 0.) dy =  par->dy;
      if (dy <  0.) dy = -par->dy;
/***add ***/
      dx = dy*tan(strike/(RAD));
/***/
      dlength = sqrt(dx*dx + dy*dy);
      }
   if (fabs(dip) >= 45.) {
      dz = par->dz;
      dw = fabs(dz/sin(dip/(RAD)));
      dh = fabs(dw*cos(dip/(RAD)));
      if (dip > 90. || dip < 0.) dh = -dh;
      }
   else {
      dw = par->dx;
      dz = fabs(dw*sin(dip/(RAD)));
      dh = fabs(dw*cos(dip/(RAD)));
      if (dip > 90. || dip < 0.) dh = -dh;
      }
   dwidth = dw;
   dx0 =  dh*cos(strike/(RAD));
   dy0 = -dh*sin(strike/(RAD));

   /* get amplitudes */
   /*if ((M = source->f[t]) == 0.0) return(0);*/
   M = 1.0;
   dt = par->dt;
   dv = grid->dx*grid->dy*grid->dz;
   f1 = -M*dt/dv;
   f4 = -M*dt/dv*0.25; 
   Mxx = f1*source->Mxx;
   Myy = f1*source->Myy;
   Mzz = f1*source->Mzz;
   Mxy = f4*source->Mxy;
   Mxz = f4*source->Mxz;
   Myz = f4*source->Myz;
   if (par->d == 2) {
      dy2d = grid->dy/(KMTOM); /* force-density in y direction (line source) */
      Mxx *= dy2d;
      Mzz *= dy2d;
      Mxz *= dy2d;
      }

   /* loop over points */ 
   z = z0; dw = 0.;
   while(dw <= w) {
      x = x0; y = y0; df = 0.;
      while(df <= d) {
         n = (int) rint((x - grid->x0)/grid->dx);
         m = (int) rint((z - grid->z0)/grid->dz);
         l = (int) rint((y - grid->y0)/grid->dy);
         fpoint = 1;
         if (n < -1 || n > grid->n) fpoint = 0;
         if (m < -1 || m > grid->m) fpoint = 0;
         if (l < -1 || l > grid->l) fpoint = 0;
         if (fpoint) {
            distance = (x-fx0)*(x-fx0) + (y-fy0)*(y-fy0) + (z-fz0)*(z-fz0);
            distance = sqrt(distance);
            time = distance/v;
            tindex = t - (int) (rint(time/par->dt));
            if (tindex <  0) M = 0.;
            if (tindex >= 0) M = source->f[tindex];
            }
         if (par->d == 2 && fpoint) {
            data->Txx[0  ][m  ][n  ] += M*Mxx;
            data->Tzz[0  ][m  ][n  ] += M*Mzz;
            data->Txz[0  ][m  ][n  ] += M*Mxz;
            data->Txz[0  ][m  ][n+1] += M*Mxz;
            data->Txz[0  ][m-1][n  ] += M*Mxz;
            data->Txz[0  ][m-1][n+1] += M*Mxz;
            }
         if (par->d == 3 && fpoint) {
            data->Txx[l  ][m  ][n  ] += M*Mxx;
            data->Tyy[l  ][m  ][n  ] += M*Myy;
            data->Tzz[l  ][m  ][n  ] += M*Mzz;
            data->Txz[l  ][m  ][n  ] += M*Mxz;
            data->Txz[l  ][m  ][n+1] += M*Mxz;
            data->Txz[l  ][m-1][n  ] += M*Mxz;
            data->Txz[l  ][m-1][n+1] += M*Mxz;
            data->Txy[l  ][m  ][n  ] += M*Mxy;
            data->Txy[l  ][m  ][n+1] += M*Mxy;
            data->Txy[l-1][m  ][n  ] += M*Mxy;
            data->Txy[l-1][m  ][n+1] += M*Mxy;
            data->Tyz[l  ][m  ][n  ] += M*Myz;
            data->Tyz[l  ][m-1][n  ] += M*Myz;
            data->Tyz[l-1][m  ][n  ] += M*Myz;
            data->Tyz[l-1][m-1][n  ] += M*Myz;
            }
         x  += dx;
         y  += dy;
         df += dlength;
         } 
      x0 += dx0;
      y0 += dy0;
      z  += dz;
      dw += dwidth;
      }

}




addDistributedSource(par, grid, data, source, time)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int             time;
{
   double sqrt(), fabs();
   float A, M, f1, f4, dt, dv, dy2d;
   float Mxx, Myy, Mzz, Mxy, Mxz, Myz;
   int p, l, m, n, t;

   /* initialize */
   /*if ((M = source->f[time]) == 0.0) return(0);*/
   M = 1.0;
   dt = par->dt;
   dv = grid->dx*grid->dy*grid->dz;
   f1 = -M*dt/dv;
   f4 = -M*dt/dv*0.25; 
   Mxx = f1*source->Mxx;
   Myy = f1*source->Myy;
   Mzz = f1*source->Mzz;
   Mxy = f4*source->Mxy;
   Mxz = f4*source->Mxz;
   Myz = f4*source->Myz;
   if (par->d == 2) {
      dy2d = grid->dy/(KMTOM); /* force-density in y direction (line source) */
      Mxx *= dy2d;
      Mzz *= dy2d;
      Mxz *= dy2d;
      }

   /* loop over points */
   for(p = 0; p < source->p; p++) {
      t = time - source->dft[p];
      l = source->dfl[p] - grid->l0;
      m = source->dfm[p] - grid->m0;
      n = source->dfn[p] - grid->n0;
      A = source->dfa[p];
      if (l < -1 || l >  grid->l) continue;
      if (m < -1 || m >  grid->m) continue;
      if (n < -1 || n >  grid->n) continue;
      if (t <  0 || t >=  par->t) continue;
      M = A*source->f[t];
      if (par->d == 2) {
         data->Txx[0  ][m  ][n  ] += M*Mxx;
         data->Tzz[0  ][m  ][n  ] += M*Mzz;
         data->Txz[0  ][m  ][n  ] += M*Mxz;
         data->Txz[0  ][m  ][n+1] += M*Mxz;
         data->Txz[0  ][m-1][n  ] += M*Mxz;
         data->Txz[0  ][m-1][n+1] += M*Mxz;
         }
      if (par->d == 3) {
         data->Txx[l  ][m  ][n  ] += M*Mxx;
         data->Tyy[l  ][m  ][n  ] += M*Myy;
         data->Tzz[l  ][m  ][n  ] += M*Mzz;
         data->Txz[l  ][m  ][n  ] += M*Mxz;
         data->Txz[l  ][m  ][n+1] += M*Mxz;
         data->Txz[l  ][m-1][n  ] += M*Mxz;
         data->Txz[l  ][m-1][n+1] += M*Mxz;
         data->Txy[l  ][m  ][n  ] += M*Mxy;
         data->Txy[l  ][m  ][n+1] += M*Mxy;
         data->Txy[l-1][m  ][n  ] += M*Mxy;
         data->Txy[l-1][m  ][n+1] += M*Mxy;
         data->Tyz[l  ][m  ][n  ] += M*Myz;
         data->Tyz[l  ][m-1][n  ] += M*Myz;
         data->Tyz[l-1][m  ][n  ] += M*Myz;
         data->Tyz[l-1][m-1][n  ] += M*Myz;
         }
      }

}


addDistributedSource2(par, grid, data, source, time)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int             time;
{
   double sqrt(), fabs();
   float A, M, f1, f4, dt, dv, dy2d, gtime, factor, amplitude;
   float Mxx, Myy, Mzz, Mxy, Mxz, Myz;
   int p, l, m, n, t;

   /* initialize */
   /*if ((M = source->f[time]) == 0.0) return(0);*/
   M = 1.0;
   dt = par->dt;
   dv = grid->dx*grid->dy*grid->dz;
   f1 = -M*dt/dv;
   f4 = -M*dt/dv*0.25; 
   Mxx = f1*source->Mxx;
   Myy = f1*source->Myy;
   Mzz = f1*source->Mzz;
   Mxy = f4*source->Mxy;
   Mxz = f4*source->Mxz;
   Myz = f4*source->Myz;
   if (par->d == 2) {
      dy2d = grid->dy/(KMTOM); /* force-density in y direction (line source) */
      Mxx *= dy2d;
      Mzz *= dy2d;
      Mxz *= dy2d;
      }

   /* loop over points */
   for(p = 0; p < source->p; p++) {
      t = time - source->dft[p];
      l = source->dfl[p] - grid->l0;
      m = source->dfm[p] - grid->m0;
      n = source->dfn[p] - grid->n0;
      A = source->dfa[p];
      if (l < -1 || l >  grid->l) continue;
      if (m < -1 || m >  grid->m) continue;
      if (n < -1 || n >  grid->n) continue;
      /*if (t <  0 || t >=  par->t) continue;*/
      gtime = ((float) (t))*dt - 0.5*source->dfr[p] - source->t0;
      factor  = (PI/source->dfr[p]*gtime);
      factor *= factor;
      amplitude = exp(-factor)*source->amp;
      /*M = A*source->f[t];*/
      M = A*amplitude;
      if (par->d == 2) {
         data->Txx[0  ][m  ][n  ] += M*Mxx;
         data->Tzz[0  ][m  ][n  ] += M*Mzz;
         data->Txz[0  ][m  ][n  ] += M*Mxz;
         data->Txz[0  ][m  ][n+1] += M*Mxz;
         data->Txz[0  ][m-1][n  ] += M*Mxz;
         data->Txz[0  ][m-1][n+1] += M*Mxz;
         }
      if (par->d == 3) {
         data->Txx[l  ][m  ][n  ] += M*Mxx;
         data->Tyy[l  ][m  ][n  ] += M*Myy;
         data->Tzz[l  ][m  ][n  ] += M*Mzz;
         data->Txz[l  ][m  ][n  ] += M*Mxz;
         data->Txz[l  ][m  ][n+1] += M*Mxz;
         data->Txz[l  ][m-1][n  ] += M*Mxz;
         data->Txz[l  ][m-1][n+1] += M*Mxz;
         data->Txy[l  ][m  ][n  ] += M*Mxy;
         data->Txy[l  ][m  ][n+1] += M*Mxy;
         data->Txy[l-1][m  ][n  ] += M*Mxy;
         data->Txy[l-1][m  ][n+1] += M*Mxy;
         data->Tyz[l  ][m  ][n  ] += M*Myz;
         data->Tyz[l  ][m-1][n  ] += M*Myz;
         data->Tyz[l-1][m  ][n  ] += M*Myz;
         data->Tyz[l-1][m-1][n  ] += M*Myz;
         }
      }

}



/* output routines for time series output */
writeFieldT(par, grid, data, image, t)
Parameter        *par;
FdGrid          *grid;
VelocityStress  *data;
Image          *image;
int                 t;
{
   float dt;
   int d, g, x, y, sample, ftype, time;
   FILE *file, *getFFileT();

   /* initialize and return if no output */
   file   = NULL;
   d      = image->d;
   sample = image->sample;
   ftype  = image->mode;
   if (image->sample < 1) return(0);
   if (image->movie  < 1) return(0);

   /* begin parallel synchronization and open file (write/append) */
   mpNodeOrder(0);
   file = getFFileT(par, grid, image, ftype, t);

   /* write initial global header including time step information */
   if (par->node == 0 && (t + 1) == image->movie) {
      x = (d == XYPLANE || d == XZPLANE) ? (par->n) : (par->l);
      y = (d == YZPLANE || d == XZPLANE) ? (par->m) : (par->l);
      time = par->t/image->movie;
      dt   = par->dt*((float) image->movie);
      fwrite(&x     , 4, 1, file);
      fwrite(&y     , 4, 1, file);
      fwrite(&time  , 4, 1, file);
      fwrite(&dt    , 4, 1, file);
      }

   /* for each multigrid write out field data (including subgrid header) */
   for(g = 0; g < par->g; g++) {
      writeSubFieldT(par, grid + g, data + g, image, file);
      }

   /* close output image file and end parallel synchronization */
   fclose(file);
   mpNodeOrder(1);

}


writeSubFieldT(par, grid, data, image, file)
Parameter        *par;
FdGrid          *grid;
VelocityStress  *data;
Image          *image;
FILE            *file;
{
   int x, y, x0, y0, l, m, n, l1, m1, n1, l2, m2, n2, f, sample, d;
   int offset, type;

   /* initialize and get sample */
   type   = image->mode;
   sample = image->sample;
   if (sample < 1) return(0);

   /* get grid limits */
   if (image->d == XYPLANE) {
      if ((m1 = image->plane - grid->m0) < 0) return(0);
      if ((m1 = m1/grid->s) >= grid->m) return(0);
      m2 = m1 + 1;
      l1 = (offset = (grid->l0%sample)) ? (sample - offset) : (0); 
      n1 = (offset = (grid->n0%sample)) ? (sample - offset) : (0); 
      l1 = 0;
      n1 = 0; 
      l2 = grid->l;
      n2 = grid->n;
      x  = grid->n*grid->s;
      y  = grid->l*grid->s;
      x0 = grid->n0;
      y0 = grid->l0;
      }
   if (image->d == XZPLANE) {
      if ((l1 = image->plane - grid->l0) < 0) return(0);
      if ((l1 = l1/grid->s) >= grid->l) return(0);
      l2 = l1 + 1;
      m1 = (offset = (grid->m0%sample)) ? (sample - offset) : (0); 
      n1 = (offset = (grid->n0%sample)) ? (sample - offset) : (0); 
      m1 = 0; 
      n1 = 0; 
      m2 = grid->m;
      n2 = grid->n;
      x  = grid->n*grid->s;
      y  = grid->m*grid->s;
      x0 = grid->n0;
      y0 = grid->m0;
      }
   if (image->d == YZPLANE) {
      if ((n1 = image->plane - grid->n0) < 0) return(0);
      if ((n1 = n1/grid->s) >= grid->n) return(0);
      n2 = n1 + 1;
      l1 = (offset = (grid->l0%sample)) ? (sample - offset) : (0); 
      m1 = (offset = (grid->m0%sample)) ? (sample - offset) : (0); 
      l1 = 0; 
      m1 = 0; 
      l2 = grid->l;
      m2 = grid->m;
      x  = grid->l*grid->s;
      y  = grid->m*grid->s;
      x0 = grid->l0;
      y0 = grid->m0;
      }

   /* get apparent grid sampling factor */
   f = sample/grid->s; if (f == 0) f = 1;
   f = f*grid->s;

   /* write header */
   fwrite(&x,         4, 1, file);
   fwrite(&y,         4, 1, file);
   fwrite(&x0,        4, 1, file);
   fwrite(&y0,        4, 1, file);
   fwrite(&f,         4, 1, file);

   /* determine actual grid sampling factor */
   f = sample/grid->s; if (f == 0) f = 1;

   /* write image file (all rows not including boundary) */
   d = image->d;

   if (type == WAVEVX ) writeV(data->Vx , l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVEVY ) writeV(data->Vy , l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVEVZ ) writeV(data->Vz , l1, l2, m1, m2, n1, n2, f, d, file);
   if (par->model != 2) { 
   if (type == WAVETXX) writeV(data->Txx, l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETYY) writeV(data->Tyy, l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETZZ) writeV(data->Tzz, l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETXY) writeV(data->Txy, l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETXZ) writeV(data->Txz, l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETYZ) writeV(data->Tyz, l1, l2, m1, m2, n1, n2, f, d, file);
   }
   if (par->model == 2) {
   if (type == WAVETXX) writeV(data->T  , l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETYY) writeV(data->T  , l1, l2, m1, m2, n1, n2, f, d, file);
   if (type == WAVETZZ) writeV(data->T  , l1, l2, m1, m2, n1, n2, f, d, file);
   }
   if (type == WAVEFP ) writeP(par, grid, data, l1,l2,m1,m2,n1,n2, f, d, file);
   if (type == WAVEFS ) writeS(par, grid, data, l1,l2,m1,m2,n1,n2, f, d, file);
   
}


FILE *
getFFileT(par, grid, image, type, t)
Parameter     *par;
FdGrid       *grid;
Image       *image;
int        type, t;
{
   int g;
   char fileName[82], ext[10], mode[5];
   FILE *file;

   /* get output file name extension */
   if (type == WAVEVX ) strcpy(ext,  "Vx");
   if (type == WAVEVY ) strcpy(ext,  "Vy");
   if (type == WAVEVZ ) strcpy(ext,  "Vz");
   if (type == WAVETXX) strcpy(ext, "Txx");
   if (type == WAVETYY) strcpy(ext, "Tyy");
   if (type == WAVETZZ) strcpy(ext, "Tzz");
   if (type == WAVETXY) strcpy(ext, "Txy");
   if (type == WAVETXZ) strcpy(ext, "Txz");
   if (type == WAVETYZ) strcpy(ext, "Tyz");
   if (type == WAVEFP ) strcpy(ext,  "FP");
   if (type == WAVEFS ) strcpy(ext,  "FS");

   /* determine open file mode */
   strcpy(mode, "a"); 
   if (par->node == 0 && (t + 1) == image->movie) strcpy(mode, "w");

   /* get file name */
   strcpy(fileName, image->file);
   strcat(fileName,  ".");
   strcat(fileName,  ext);
   
   /* open and return file */
   if ((file = fopen(fileName, mode)) == NULL) {
      fprintf(stderr, "ERROR: can't open image file \"%s\"\n", fileName);
      exit(-1);
      }

   return(file);

}


getTraces(par, grid, traces)
Parameter   *par;
FdGrid      *grid;
Traces      *traces;
{
   int i;
   for(i = 0; i < par->a; i++) getTracesF(par, grid, traces + i);
}

getTracesF(par, grid, traces)
Parameter   *par;
FdGrid      *grid;
Traces      *traces;
{
   float x, y, z, dx, dy, dz;
   int i, j, l, m, n, p, nglobal, nlocal, insideSubGrid, insideGrid;
   char line[82];
   FILE *file;

   /* initialize */
   nglobal = 0;
   nlocal  = 0;
   p = traces->p;

   /* get traces from coordinates */
   if (p > 1) {
      dx = (traces->x2 - traces->x1)/((float) (p - 1));
      dy = (traces->y2 - traces->y1)/((float) (p - 1));
      dz = (traces->z2 - traces->z1)/((float) (p - 1));
      for(j = 0; j < p; j++) {
         x = traces->x1 + dx*((float) j);
         y = traces->y1 + dy*((float) j);
         z = traces->z1 + dz*((float) j);
         insideGrid = 0;
         if (par->d == 2) y = 0.;
         for(i = 0; i < par->g; i++) {
            insideSubGrid = 1;
            l  = (int) floor((y - grid[i].y0)/grid[i].dy);
            m  = (int) floor((z - grid[i].z0)/grid[i].dz);
            n  = (int) floor((x - grid[i].x0)/grid[i].dx);
            if (l < 0 || l >= grid[i].l) insideSubGrid = 0;
            if (m < 0 || m >= grid[i].m) insideSubGrid = 0;
            if (n < 0 || n >= grid[i].n) insideSubGrid = 0;
            if (insideSubGrid) insideGrid = 1;
            }
         if (insideGrid) {
            traces->order[nlocal] = nglobal;
            traces->l    [nlocal] = l; 
            traces->m    [nlocal] = m; 
            traces->n    [nlocal] = n; 
            traces->x    [nlocal] = x; 
            traces->y    [nlocal] = y; 
            traces->z    [nlocal] = z; 
            nlocal++;
            }
         nglobal++;
         }
      }

   /* open input file */
   if (traces->tfile[0]) {
      file = fopen(traces->tfile, "r");
      if (file == NULL) {
         fprintf(stderr,"Error: Can't open trace file \"%s\"\n",traces->tfile);
         exit(1);
         }
      }

   /* read input file and data lines */
   if (traces->tfile[0]) {
      while(fgets(line, 256, file) != NULL) {
         if (getLine(line) != LINE_VALUE) continue;
         sscanf(line, "%f %f %f", &x, &y, &z);
         x *= KMTOM;
         y *= KMTOM;
         z *= KMTOM;
         insideGrid = 0;
         for(i = 0; i < par->g; i++) {
            insideSubGrid = 1;
            l  = (int) floor((y - grid[i].y0)/grid[i].dy);
            m  = (int) floor((z - grid[i].z0)/grid[i].dz);
            n  = (int) floor((x - grid[i].x0)/grid[i].dx);
            if (l < 0 || l >= grid[i].l) insideSubGrid = 0;
            if (m < 0 || m >= grid[i].m) insideSubGrid = 0;
            if (n < 0 || n >= grid[i].n) insideSubGrid = 0;
            if (insideSubGrid) insideGrid = 1;
            }
         if (insideGrid) {
            traces->order[nlocal] = nglobal;
            traces->l    [nlocal] = l; 
            traces->m    [nlocal] = m; 
            traces->n    [nlocal] = n; 
            traces->x    [nlocal] = x; 
            traces->y    [nlocal] = y; 
            traces->z    [nlocal] = z; 
            nlocal++;
            }
         nglobal++;
         }
      }

   /* close input file */
   if (traces->tfile[0]) fclose(file);

}

/* doesn't work with multigrid */
writeTraces(par, grid, data, traces)
Parameter       *par;
FdGrid          *grid;
VelocityStress  *data;
Traces          *traces;
{
   float intVx2D(), intVz2D(), intVx3D(), intVy3D(), intVz3D();
   float intP_2D(), intS_2D(), intP_3D(), intS_3D();
   float *Vx, *Vy, *Vz, *P, *S, fx, fy, fz, dx, dy, dz;
   int i, g, l, m, n, nlocal;
   unsigned int mode;

   /* begin parallel synchronization and initialize */
   mpNodeOrder(0);
   nlocal = traces->nlocal;
   mode   = (unsigned int) traces->mode;

   /* return if no data to output */
   if (nlocal == 0) {
      mpNodeOrder(1);
      return(0);
      }

   /* allocate temporary memory */
   if (mode & TRACEVX) Vx = (float *) malloc(nlocal*sizeof(float));
   if (mode & TRACEVY) Vy = (float *) malloc(nlocal*sizeof(float));
   if (mode & TRACEVZ) Vz = (float *) malloc(nlocal*sizeof(float));
   if (mode & TRACEP ) P  = (float *) malloc(nlocal*sizeof(float));
   if (mode & TRACES ) S  = (float *) malloc(nlocal*sizeof(float));

   /* determine values */
   for(g = 0; g < par->g; g++) {
      dx = grid[g].dx; dy = grid[g].dy; dz = grid[g].dz;
      for(i = 0; i < nlocal; i++) {
         l  = (int) floor((traces->y[i] - grid[g].y0)/dy);
         m  = (int) floor((traces->z[i] - grid[g].z0)/dz);
         n  = (int) floor((traces->x[i] - grid[g].x0)/dx);
         fy = ((traces->y[i] - grid[g].y0)/dy - (float) l);
         fx = ((traces->x[i] - grid[g].x0)/dx - (float) n);
         fz = ((traces->z[i] - grid[g].z0)/dz - (float) m);
         if (par->d == 2) {l = 0; fy = 0.;}
         if (l < 0 || l >= grid[g].l) {fprintf(stderr,"Bad traces\n");exit(0);}
         if (m < 0 || m >= grid[g].m) {fprintf(stderr,"Bad traces\n");exit(0);}
         if (n < 0 || n >= grid[g].n) {fprintf(stderr,"Bad traces\n");exit(0);}
         if (par->d == 2) {
            if (mode & TRACEVX) Vx[i] = intVx2D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACEVZ) Vz[i] = intVz2D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACEP ) P [i] = intP_2D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACES ) S [i] = intS_2D(par,grid,data,l,m,n,fx,fy,fz);
            }
         if (par->d == 3) {
            if (mode & TRACEVX) Vx[i] = intVx3D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACEVY) Vy[i] = intVy3D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACEVZ) Vz[i] = intVz3D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACEP ) P [i] = intP_3D(par,grid,data,l,m,n,fx,fy,fz);
            if (mode & TRACES ) S [i] = intS_3D(par,grid,data,l,m,n,fx,fy,fz);
            }
         }
      }
   
   /* write data to trace file */
   if (mode & TRACEVX) writeTracesFile(Vx, nlocal, traces->file, "TVx");
   if (mode & TRACEVY) writeTracesFile(Vy, nlocal, traces->file, "TVy");
   if (mode & TRACEVZ) writeTracesFile(Vz, nlocal, traces->file, "TVz");
   if (mode & TRACEP ) writeTracesFile(P , nlocal, traces->file, "TP" );
   if (mode & TRACES ) writeTracesFile(S , nlocal, traces->file, "TS" );

   /* free temporary memory */
   if (mode & TRACEVX) free(Vx);
   if (mode & TRACEVY) free(Vy);
   if (mode & TRACEVZ) free(Vz);
   if (mode & TRACEP ) free(P );
   if (mode & TRACES ) free(S );

   /* end parallel synchronization */
   mpNodeOrder(1);

}

writeTracesFile(value, n, name, ext)
float *value;
int    n;
char  *name;
char  *ext;
{
   char filename[82];
   FILE *file;
   strcpy(filename, name);
   strcat(filename, ext );
   file = fopen(filename, "a");
   if (file == NULL) {
      fprintf(stderr, "Error: Can't open traces file \"%s\"\n", filename);
      exit(1);
      }
   fwrite(value, sizeof(float), n, file);
   fclose(file);
}


/* write header information */
/* doesn't work with multigrid */
writeTracesH(par, traces)
Parameter       *par;
Traces          *traces;
{
   float dt;
   int *order, nlocal, nglobal, t;
   unsigned int mode;

   /* begin parallel synchronization and initialize */
   mpNodeOrder(0);

   /* initialize */
   nglobal = traces->nglobal;
   nlocal  = traces->nlocal;
   mode    = (unsigned int) traces->mode;
   order   = traces->order;
   dt      = par->dt*((float) traces->sample);
   t       = par->t/traces->sample;

   /* write header */
   if (par->node == 0) {
      if (mode & TRACEVX)writeTracesFileH1(nglobal, t, dt, traces->file, "TVx");
      if (mode & TRACEVY)writeTracesFileH1(nglobal, t, dt, traces->file, "TVy");
      if (mode & TRACEVZ)writeTracesFileH1(nglobal, t, dt, traces->file, "TVz");
      if (mode & TRACEP )writeTracesFileH1(nglobal, t, dt, traces->file, "TP" );
      if (mode & TRACES )writeTracesFileH1(nglobal, t, dt, traces->file, "TS" );
      }

   /* write ordering data to trace file */
   if (nlocal > 0) {
      if (mode & TRACEVX) writeTracesFileH2(order, nlocal, traces->file, "TVx");
      if (mode & TRACEVY) writeTracesFileH2(order, nlocal, traces->file, "TVy");
      if (mode & TRACEVZ) writeTracesFileH2(order, nlocal, traces->file, "TVz");
      if (mode & TRACEP ) writeTracesFileH2(order, nlocal, traces->file, "TP" );
      if (mode & TRACES ) writeTracesFileH2(order, nlocal, traces->file, "TS" );
      }

   /* write end of ordering message to trace file */
   if (par->node == par->nnode - 1) {
      if (mode & TRACEVX) writeTracesFileH3(traces->file, "TVx");
      if (mode & TRACEVY) writeTracesFileH3(traces->file, "TVy");
      if (mode & TRACEVZ) writeTracesFileH3(traces->file, "TVz");
      if (mode & TRACEP ) writeTracesFileH3(traces->file, "TP" );
      if (mode & TRACES ) writeTracesFileH3(traces->file, "TS" );
      }

   /* end parallel synchronization */
   mpNodeOrder(1);

}

writeTracesFileH1(nglobal, t, dt, name, ext)
float dt;
int   nglobal, t;
char  *name;
char  *ext;
{
   char filename[82];
   FILE *file;
   strcpy(filename, name);
   strcat(filename, ext );
   file = fopen(filename, "w");
   if (file == NULL) {
      fprintf(stderr, "Error: Can't open traces file \"%s\"\n", filename);
      exit(1);
      }
   fwrite(&nglobal, sizeof(int  ), 1, file);
   fwrite(&t      , sizeof(int  ), 1, file);
   fwrite(&dt     , sizeof(float), 1, file);
   fclose(file);
}

writeTracesFileH2(value, n, name, ext)
int   *value;
int    n;
char  *name;
char  *ext;
{
   char filename[82];
   FILE *file;
   strcpy(filename, name);
   strcat(filename, ext );
   file = fopen(filename, "a");
   if (file == NULL) {
      fprintf(stderr, "Error: Can't open traces file \"%s\"\n", filename);
      exit(1);
      }
   fwrite(value, sizeof(int), n, file);
   fclose(file);
}

writeTracesFileH3(name, ext)
char  *name;
char  *ext;
{
   int value;
   char filename[82];
   FILE *file;
   value = -1;
   strcpy(filename, name);
   strcat(filename, ext );
   file = fopen(filename, "a");
   if (file == NULL) {
      fprintf(stderr, "Error: Can't open traces file \"%s\"\n", filename);
      exit(1);
      }
   fwrite(&value, sizeof(int), 1, file);
   fclose(file);
}


/* special seg fill option */
segfill(par, grid, l1, l2, m1, m2, n1, n2)
Parameter    *par;
FdGrid      *grid;
int l1, l2, m1, m2, n1, n2;
{
   float depth;
   float calculate_vs();
   float calculate_dens();
   int l, m, n;

   /* loop over points and fill */
   for(m = m1; m <= m2; m++) {
      depth = (grid->z0 + ((float) m)*grid->dz)/(KMTOM);
      for(l = l1; l <= l2; l++) {
         for(n = n1; n <= n2; n++) {
            if (par->model != 2)
            grid->beta[l][m][n] = calculate_vs  (depth, grid->alpha[l][m][n]);
            grid->rho [l][m][n] = calculate_dens(depth, grid->alpha[l][m][n]);
            }
         }
      }
         
}


/* e3d_vs_dens.c - read the vp information from the SEG/EAGE salt model
                    (interpolated to 10 m grid spacing), and calculate the
          corresponding vs and density values.

   Uses Vp/Vs vs depth, and the density vs depth relations
     that Shawn Larsen used (Sep '98) to generate synthetic elastic traces, 
     and write out the derived Vs values.

   This program expects the input file is arranged the way program 'e3d'
     expects - with x (n/s) varying fastest, z (depth) next fastest, and
     y (e/w) varying slowest. Thus, the input values are arranged as a
     series of vertical N-S sections that step to the East

   Input units are expected to be km/sec (as e3d expects)



   Sep 22, 1999
*/


/* This calculation uses the Vp/Vs ratio with depth relation suggested by
   Leon Thomsen, July, 1998
   */
#define NUM_VPVS 6
float calculate_vs(depth, vp)
float depth, vp;
{
  int i;
  float vp_m; 
  float ratio;
  float vs;
  float dep[NUM_VPVS]={0.0,0.6,1.5,3.0,4.5,6.0}; /* depth values (in km)*/
  float vp_vs[NUM_VPVS]={5.0,3.5,3.0,2.5,2.3,2.0}; /* Leon's Vp/Vs values */
  float vp_vs_interp;
  float vp_vs_salt = 1.9; /* Vp/Vs for salt */

  /* convert vp from km/sec to m/sec */
/*
  vp_m = vp * 1000.0;
*/
  vp_m = vp;

  /* check for the special case of Vp of salt (4482 m/sec) */
  if((floor(vp_m) == 4482) || (ceil(vp_m) == 4482)) {
    vs = vp / vp_vs_salt;
    return(vs);
  }

  /* check for the special case of Vp of sea water (1500 m/sec) */
  if((floor(vp_m) == 1500) || (ceil(vp_m) == 1500)) {
    vs = 0.0;
    return(vs);
  }

  /* do a depth-interpolation for Vp/Vs */
  for(i=0;i<NUM_VPVS-1;i++) {
    if((depth >= dep[i]) && (depth <= dep[i+1])) {
      ratio = (depth - dep[i]) / (dep[i+1] - dep[i]);
      /* calculate the vp/vs value */
      vp_vs_interp = ratio * (vp_vs[i+1] - vp_vs[i]) + vp_vs[i];
      break;
    }
  }

  vs = vp / vp_vs_interp;
  return(vs);
}

/* This calculation uses the Vp to density relation suggested by Walter
   Kessinger, July, 1998
   */

float calculate_dens(depth, vp)
float depth, vp;
{
  float dens;
  float dens_salt = 2.2; /* g/cc */
  float dens_sea_water = 1.0; /* fresh water, actually */
  float vp_m, vp_fps, fpm = 3.28084;

  /* convert vp from km/sec to m/sec */
/*
  vp_m = vp * 1000.0;
*/
  vp_m = vp;

  /* check for the special case of Vp of salt (4482 m/sec) */
  if((floor(vp_m) == 4482) || (ceil(vp_m) == 4482)) {
    dens = dens_salt*(GM3TOKGM3);
    return(dens);
  }

  /* check for the special case of Vp of sea water (1500 m/sec) */
  if((floor(vp_m) == 1500) || (ceil(vp_m) == 1500)) {
    dens = dens_sea_water*(GM3TOKGM3);
    return(dens);
  }

  /* calculate the density - Gardner relation (Gardner, G.H.F., L.W. Gardner,
     and A.R. Gregory, "Formation velocity and density-The dianostic basics 
     for stratigraphic traps" Geophysics, Vol 39, #6, p 770-780, 1974.
     density = 0.23 * (Vp (ft/sec))^0.25, where ^0.25 is the fourth root
   */
  vp_fps = vp_m * fpm;
  dens = 0.23 * (float) pow((double) vp_fps, (double) 0.25);
  dens *= (GM3TOKGM3);
  return(dens);

}


/* write image volumes (does not work for multigrid) */
writeVolume(par, grid, data, image, t)
Parameter        *par;
FdGrid          *grid;
VelocityStress  *data;
Image          *image;
int                 t;
{
   float ***Vx, ***Vy, ***Vz, dx, dy, dz, P, S, getP3D(), getS3D();
   int mode, sample;
   int l, m, n;
   int nx, ny, nz, l0, m0, n0, x, y, z, x0, y0, z0, x1, y1, z1;
   FILE *file, *getVolumeFile();

   /* initialize */
   Vx = data->Vx; Vy = data->Vy; Vz = data->Vz;
   dx = grid->dx; dy = grid->dy; dz = grid->dz;
   sample = image->sample;
   mode   = image->mode;
   if (sample <= 0) return(0);

   /* determine dimensions and coordinates (complicated) */ 
   /* x,y,z are grid node dimensions for sampled global grid */
   x  = (par->n - 1)/sample + 1;
   y  = (par->l - 1)/sample + 1;
   z  = (par->m - 1)/sample + 1;
   /* n0,l0,m0 are global grid node positions of 1st sampled grid in subgrid */
   l0 = (grid->l0%sample) ? (grid->l0+(sample-grid->l0%sample)) : (grid->l0);
   m0 = (grid->m0%sample) ? (grid->m0+(sample-grid->m0%sample)) : (grid->m0);
   n0 = (grid->n0%sample) ? (grid->n0+(sample-grid->n0%sample)) : (grid->n0);
   /* x0,y0,z0 are full dimensions of subgrid from first sampled point */
   x0 = n0 - grid->n0;
   y0 = l0 - grid->l0;
   z0 = m0 - grid->m0;
   /* l0,m0,n0 are now global positions of first sampled grid in sampled grid */
   l0 = l0/sample;
   m0 = m0/sample;
   n0 = n0/sample;
   /* nx,yy,nz are sampled dimensions of subgrid */
   nx  = (grid->n - x0 - 1)/sample + 1;
   ny  = (grid->l - y0 - 1)/sample + 1;
   nz  = (grid->m - z0 - 1)/sample + 1;

   /* begin parallel synchronization */ 
   mpNodeOrder(0);

   /* open file */
   file = getVolumeFile(par, grid, image, mode, t);

   /* write header information */
   fwrite(&x , sizeof(int), 1, file); 
   fwrite(&y , sizeof(int), 1, file); 
   fwrite(&z , sizeof(int), 1, file); 
   fwrite(&nx, sizeof(int), 1, file);
   fwrite(&ny, sizeof(int), 1, file);
   fwrite(&nz, sizeof(int), 1, file);
   fwrite(&n0, sizeof(int), 1, file);
   fwrite(&l0, sizeof(int), 1, file);
   fwrite(&m0, sizeof(int), 1, file);

   /* write grid */
   if (mode == VOLUMEP) {
      for(l = y0; l < grid->l; l = l + sample) {
         for(m = z0; m < grid->m; m = m + sample) { 
            for(n = x0; n < grid->n; n = n + sample) {
               P = getP3D(Vx, Vy, Vz, dx, dy, dz, l, m, n);
               fwrite(&P, sizeof(float), 1, file);
               }
            }
         }
      }
   if (mode == VOLUMES) {
      for(l = y0; l < grid->l; l = l + sample) {
         for(m = z0; m < grid->m; m = m + sample) { 
            for(n = x0; n < grid->n; n = n + sample) {
               S = getS3D(Vx, Vy, Vz, dx, dy, dz, l, m, n);
               fwrite(&S, sizeof(float), 1, file);
               }
            }
         }
      }

   /* close file */
   fclose(file);

   /* end parallel synchronization */
   mpNodeOrder(1);

}

FILE *
getVolumeFile(par, grid, image, mode, t)
Parameter     *par;
FdGrid       *grid;
Image       *image;
int        mode, t;
{
   char fileName[82], ext[10], rwa[5];
   FILE *file;

   /* get volume file name */
   if (mode == VOLUMEP) strcpy(ext, "3DP");
   if (mode == VOLUMES) strcpy(ext, "3DS");
   getFileName(fileName, image->file, ext, t + 1, par->t);

   /* open output volume file */
   strcpy(rwa, "a");
   if (par->node == 0) strcpy(rwa, "w");
   if ((file = fopen(fileName, rwa)) == NULL) {
      fprintf(stderr, "ERROR: can't open volume file \"%s\"\n", fileName);
      exit(-1);
      }
   
   /* return file pointer */
   return(file);

}


/* this loops over fault coordinates */
addFaultSource2(par, grid, data, source, t)
Parameter       *par;
FdGrid         *grid;
VelocityStress *data;
FdSource     *source;
int                t;
{
   double sqrt();
   float x, y, z, length, width, strike, dip, rake; 
   float angle, x0, x1, y0, y1, z0, dx, dy, dz; 
   float dlength, dwidth, dh, dx0, dy0, s;
   float v, distance, time, fx0, fy0, fz0;
   float M, factor1, factor4, dt, dv;
   float Mxx, Myy, Mzz, Mxy, Mxz, Myz;
   float fx, fy, fz, f1, f2, f3, f4, f5, f6, f7, f8;
   int l, m, n, l0, m0, n0, tindex, fpoint, gpoint;
   int i, j, nlength, nwidth;

   /* initialize */
   length = source->length; width = source->width;
   strike = source->strike; dip = source->dip;
   v = source->v;
   if (v <= 0.) v = 0.8*par->minS;
   if (v <= 0.) v = 0.5*par->minP;

   /* recalculate parameters for 2D problems (apparent dip) (addFaultSource) */
   if (par->d == 2) {
      fprintf(stderr, "Error: can't use addFaultSource2 for 2-D problems\n");
      exit(1);
      }

   /* determine global coordinates at fault ends (along top) */
   angle = strike/(RAD);
   x0 = source->x - (length/2.)*sin(strike/(RAD));
   y0 = source->y - (length/2.)*cos(strike/(RAD));
   x1 = source->x + (length/2.)*sin(strike/(RAD));
   y1 = source->y + (length/2.)*cos(strike/(RAD));
   z0 = source->depth;

   /* determine subfault increment in length and width */
   nlength = (int) rint(length/grid->dx);
   if (nlength <= 0) nlength = 1;
   dlength = length/((float) nlength);
   nwidth = (int) rint(width/grid->dz);
   if (nwidth <= 0) nwidth  = 1;
   dwidth = width/((float) nwidth);

   /* determine increments */
   dx  =  dlength*sin(strike/(RAD));
   dy  =  dlength*cos(strike/(RAD));
   dz  =   dwidth*sin(dip   /(RAD));
   dh  =   dwidth*cos(dip   /(RAD));
   dx0 =       dh*cos(strike/(RAD));
   dy0 =      -dh*sin(strike/(RAD));

   /* determine starting coordinates within first subfault */
   x0 += dx/2. + dx0/2.;
   y0 += dy/2. + dy0/2.;
   z0 += dz/2.         ;

   /* determine source coordinates */
   fx0  = source->x + source->s0*sin(strike/(RAD));
   fy0  = source->y + source->s0*cos(strike/(RAD));
   dh   = source->d0*cos(dip/(RAD));
   fx0 += dh*cos(strike/(RAD));
   fy0 -= dh*sin(strike/(RAD));
   fz0  = source->depth + source->d0*sin(dip/(RAD));
   
   /* get amplitudes (need force-density in y direction for 2D (line source)) */
   M        = 1.0;
   dt       = par->dt;
   dv       = grid->dx*grid->dy*grid->dz;
   factor1  = -M*dt/dv;
   factor4  = -M*dt/dv*0.25; 
   factor4 *= 4.;
   Mxx      = factor1*source->Mxx;
   Myy      = factor1*source->Myy;
   Mzz      = factor1*source->Mzz;
   Mxy      = factor4*source->Mxy;
   Mxz      = factor4*source->Mxz;
   Myz      = factor4*source->Myz;

   /* loop over points */
   z = z0;
   for(j = 0; j < nwidth; j++) {
      x = x0; 
      y = y0;
      for(i = 0; i < nlength; i++) {
         n = (int) rint((x - grid->x0)/grid->dx);
         m = (int) rint((z - grid->z0)/grid->dz);
         l = (int) rint((y - grid->y0)/grid->dy);
         fpoint = 1;
         if (n < -2 || n > grid->n+1) fpoint = 0;
         if (m < -2 || m > grid->m+1) fpoint = 0;
         if (l < -2 || l > grid->l+1) fpoint = 0;
         if (fpoint) {
            distance = (x-fx0)*(x-fx0) + (y-fy0)*(y-fy0) + (z-fz0)*(z-fz0);
            distance = sqrt(distance);
            time = distance/v;
            tindex = t - (int) (rint(time/par->dt));
            if (tindex <  0) M = 0.;
            if (tindex >= 0) M = source->f[tindex];
            }
         if (fpoint) {
            /* normal components */
            n  = (int) floor((x - grid->x0)/grid->dx - 0.5);
            m  = (int) floor((z - grid->z0)/grid->dz      );
            l  = (int) floor((y - grid->y0)/grid->dy      );
            gpoint = 1;
            if (n < -2 || n > grid->n) gpoint = 0;
            if (m < -2 || m > grid->m) gpoint = 0;
            if (l < -2 || l > grid->l) gpoint = 0;
            if (gpoint) {
               fx = ((x - grid->x0)/grid->dx - 0.5 - (float) n);
               fy = ((y - grid->y0)/grid->dy       - (float) l);
               fz = ((z - grid->z0)/grid->dz       - (float) m);
               f1 = (1. - fx)*(1. - fy)*(1. - fz);
               f2 =       fx *(1. - fy)*(1. - fz);
               f3 = (1. - fx)*(1. - fy)*      fz ;
               f4 =       fx *(1. - fy)*      fz ;
               f5 = (1. - fx)*      fy *(1. - fz);
               f6 =       fx *      fy *(1. - fz);
               f7 = (1. - fx)*      fy *      fz ;
               f8 =       fx *      fy *      fz ;
               data->Txx[l  ][m  ][n  ] += M*Mxx*f1;
               data->Txx[l  ][m  ][n+1] += M*Mxx*f2;
               data->Txx[l  ][m+1][n  ] += M*Mxx*f3;
               data->Txx[l  ][m+1][n+1] += M*Mxx*f4;
               data->Txx[l+1][m  ][n  ] += M*Mxx*f5;
               data->Txx[l+1][m  ][n+1] += M*Mxx*f6;
               data->Txx[l+1][m+1][n  ] += M*Mxx*f7;
               data->Txx[l+1][m+1][n+1] += M*Mxx*f8;
               data->Tyy[l  ][m  ][n  ] += M*Myy*f1;
               data->Tyy[l  ][m  ][n+1] += M*Myy*f2;
               data->Tyy[l  ][m+1][n  ] += M*Myy*f3;
               data->Tyy[l  ][m+1][n+1] += M*Myy*f4;
               data->Tyy[l+1][m  ][n  ] += M*Myy*f5;
               data->Tyy[l+1][m  ][n+1] += M*Myy*f6;
               data->Tyy[l+1][m+1][n  ] += M*Myy*f7;
               data->Tyy[l+1][m+1][n+1] += M*Myy*f8;
               data->Tzz[l  ][m  ][n  ] += M*Mzz*f1;
               data->Tzz[l  ][m  ][n+1] += M*Mzz*f2;
               data->Tzz[l  ][m+1][n  ] += M*Mzz*f3;
               data->Tzz[l  ][m+1][n+1] += M*Mzz*f4;
               data->Tzz[l+1][m  ][n  ] += M*Mzz*f5;
               data->Tzz[l+1][m  ][n+1] += M*Mzz*f6;
               data->Tzz[l+1][m+1][n  ] += M*Mzz*f7;
               data->Tzz[l+1][m+1][n+1] += M*Mzz*f8;
               }
            /* xy component */
            n  = (int) floor((x - grid->x0)/grid->dx      );
            m  = (int) floor((z - grid->z0)/grid->dz      );
            l  = (int) floor((y - grid->y0)/grid->dy - 0.5);
            gpoint = 1;
            if (n < -2 || n > grid->n) gpoint = 0;
            if (m < -2 || m > grid->m) gpoint = 0;
            if (l < -2 || l > grid->l) gpoint = 0;
            if (gpoint) {
               fx = ((x - grid->x0)/grid->dx       - (float) n);
               fy = ((y - grid->y0)/grid->dy - 0.5 - (float) l);
               fz = ((z - grid->z0)/grid->dz       - (float) m);
               f1 = (1. - fx)*(1. - fy)*(1. - fz);
               f2 =       fx *(1. - fy)*(1. - fz);
               f3 = (1. - fx)*(1. - fy)*      fz ;
               f4 =       fx *(1. - fy)*      fz ;
               f5 = (1. - fx)*      fy *(1. - fz);
               f6 =       fx *      fy *(1. - fz);
               f7 = (1. - fx)*      fy *      fz ;
               f8 =       fx *      fy *      fz ;
               data->Txy[l  ][m  ][n  ] += M*Mxy*f1;
               data->Txy[l  ][m  ][n+1] += M*Mxy*f2;
               data->Txy[l  ][m+1][n  ] += M*Mxy*f3;
               data->Txy[l  ][m+1][n+1] += M*Mxy*f4;
               data->Txy[l+1][m  ][n  ] += M*Mxy*f5;
               data->Txy[l+1][m  ][n+1] += M*Mxy*f6;
               data->Txy[l+1][m+1][n  ] += M*Mxy*f7;
               data->Txy[l+1][m+1][n+1] += M*Mxy*f8;
               }
            /* xz component */
            n  = (int) floor((x - grid->x0)/grid->dx      );
            m  = (int) floor((z - grid->z0)/grid->dz - 0.5);
            l  = (int) floor((y - grid->y0)/grid->dy      );
            gpoint = 1;
            if (n < -2 || n > grid->n) gpoint = 0;
            if (m < -2 || m > grid->m) gpoint = 0;
            if (l < -2 || l > grid->l) gpoint = 0;
            if (gpoint) {
               fx = ((x - grid->x0)/grid->dx       - (float) n);
               fy = ((y - grid->y0)/grid->dy       - (float) l);
               fz = ((z - grid->z0)/grid->dz - 0.5 - (float) m);
               f1 = (1. - fx)*(1. - fy)*(1. - fz);
               f2 =       fx *(1. - fy)*(1. - fz);
               f3 = (1. - fx)*(1. - fy)*      fz ;
               f4 =       fx *(1. - fy)*      fz ;
               f5 = (1. - fx)*      fy *(1. - fz);
               f6 =       fx *      fy *(1. - fz);
               f7 = (1. - fx)*      fy *      fz ;
               f8 =       fx *      fy *      fz ;
               data->Txz[l  ][m  ][n  ] += M*Mxz*f1;
               data->Txz[l  ][m  ][n+1] += M*Mxz*f2;
               data->Txz[l  ][m+1][n  ] += M*Mxz*f3;
               data->Txz[l  ][m+1][n+1] += M*Mxz*f4;
               data->Txz[l+1][m  ][n  ] += M*Mxz*f5;
               data->Txz[l+1][m  ][n+1] += M*Mxz*f6;
               data->Txz[l+1][m+1][n  ] += M*Mxz*f7;
               data->Txz[l+1][m+1][n+1] += M*Mxz*f8;
               }
            /* yz component */
            n  = (int) floor((x - grid->x0)/grid->dx - 0.5);
            m  = (int) floor((z - grid->z0)/grid->dz - 0.5);
            l  = (int) floor((y - grid->y0)/grid->dy - 0.5);
            gpoint = 1;
            if (n < -2 || n > grid->n) gpoint = 0;
            if (m < -2 || m > grid->m) gpoint = 0;
            if (l < -2 || l > grid->l) gpoint = 0;
            if (gpoint) {
               fx = ((x - grid->x0)/grid->dx - 0.5 - (float) n);
               fy = ((y - grid->y0)/grid->dy - 0.5 - (float) l);
               fz = ((z - grid->z0)/grid->dz - 0.5 - (float) m);
               f1 = (1. - fx)*(1. - fy)*(1. - fz);
               f2 =       fx *(1. - fy)*(1. - fz);
               f3 = (1. - fx)*(1. - fy)*      fz ;
               f4 =       fx *(1. - fy)*      fz ;
               f5 = (1. - fx)*      fy *(1. - fz);
               f6 =       fx *      fy *(1. - fz);
               f7 = (1. - fx)*      fy *      fz ;
               f8 =       fx *      fy *      fz ;
               data->Tyz[l  ][m  ][n  ] += M*Myz*f1;
               data->Tyz[l  ][m  ][n+1] += M*Myz*f2;
               data->Tyz[l  ][m+1][n  ] += M*Myz*f3;
               data->Tyz[l  ][m+1][n+1] += M*Myz*f4;
               data->Tyz[l+1][m  ][n  ] += M*Myz*f5;
               data->Tyz[l+1][m  ][n+1] += M*Myz*f6;
               data->Tyz[l+1][m+1][n  ] += M*Myz*f7;
               data->Tyz[l+1][m+1][n+1] += M*Myz*f8;
               }
            }
         x += dx;
         y += dy;
         }
      x0 += dx0;
      y0 += dy0;
      z  += dz;
      }

}

