/******************************************************************************
   timestep: time-step through the equations

    ================================================= 
   |     Copyright (c) Shawn C. Larsen               |
   |     Seismological Laboratory                    |
   |     California Institute of Technology          |
   |     Pasadena, CA 91125                          |
   |     All Rights Reserved.                        |
    ================================================= 

   compile: cc -g -c timestep.c

   note: input units are km, s, km/s; g/cm3; dyne-cm;
   note: used  units are mks (m, s, m/s, kg/m3; N-m)

   author: shawn larsen
   date  : december 1990
   where : caltech
*****************************************************************************/

#include "elas.h"

timeStep(par, source, grid, data, t)
Parameter      *par;
FdSource       *source;
FdGrid         *grid;
VelocityStress *data;
int t;
{

   if (grid->d == 2) timeStep2D(par, source, grid, data, t);
   if (grid->d == 3) timeStep3D(par, source, grid, data, t);
}



timeStep2D(par, source, grid, data, t)
Parameter      *par;
FdSource       *source;
FdGrid         *grid;
VelocityStress *data;
int t;
{
   int g;

   /* get limits of active grid */
   if (par->active) getActiveGrid(par, source, grid, data, t);

   /* 4th order velocity updates */
   if (data->t > 0) getClock(par, grid, 2, 0);
   for(g = 0; g < par->g; g++) update4V (par, grid + g, data + g);
   if (data->t > 0) getClock(par, grid, 2, 1);
   if(par->join) if(t < par->tjoin) addJoinV(par, grid, data, t);

   /* boundary conditions for velocities */
   for(g = 0; g < par->g; g++) boundaryV2D(par, grid + g, data + g);

   /* damping for bottom boundary */
/* now in boundary routine but not vdensity grid
   if (par->damp) damp(par, grid, data);
*/

   /* 4th order stress updates */
   if (data->t > 0) getClock(par, grid, 2, 0);
   for(g = 0; g < par->g; g++) update4T (par, grid + g, data + g);
   if (data->t > 0) getClock(par, grid, 2, 1);
   if(par->join) if(t < par->tjoin) addJoinT(par, grid, data, t);

   /* boundary conditions for stresses   */
   for(g = 0; g < par->g; g++) boundaryT2D(par, grid + g, data + g);

   /* attenuation */
   /*
   if (par->q) for(g = 0; g < par->g; g++) attenuate2D(par, grid + g, data + g);
   */

}


timeStep3D(par, source, grid, data, t)
Parameter      *par;
FdSource       *source;
FdGrid         *grid;
VelocityStress *data;
int t;
{
   int g;

   /* get limits of active grid */
   if (par->active) getActiveGrid(par, source, grid, data, t);

   /* 4th order velocity updates */
   if (data->t > 0) getClock(par, grid, 2, 0);
   for(g = 0; g < par->g; g++) update4V (par, grid + g, data + g);
   if (data->t > 0) getClock(par, grid, 2, 1);

   /* boundary conditions for velocities */
   for(g = 0; g < par->g; g++) boundaryV3D(par, grid + g, data + g);

   /* amplitude reduction boundary condition */
   /**if (par->damp) damp(par, grid, data);**/

   /* get 4th order stress updates on grid (with 2-node boundary) */
   if (data->t > 0) getClock(par, grid, 2, 0);
   for(g = 0; g < par->g; g++) update4T(par, grid + g, data + g);
   if (data->t > 0) getClock(par, grid, 2, 1);

   /* boundary conditions for stress */
   for(g = 0; g < par->g; g++) boundaryT3D(par, grid + g, data + g);

}
