/******************************************************************************
   getTopog: get finite difference topography by assigning 
                  appropriate density values above the free surface.

   usage: getGrid(par, input, grid)
      par    - parameters
      grid   - grid parameters
      medium - grid medium parameters

   note: input units are km, s, km/s; g/cm3
   note: used  units are mks (m, s, m/s, kg/m3)

   Author: Craig A. Schultz
           LLNL, University of California
      Livermore, CA 94551
      L-205, (510)422-5798
   Last Modified: 1/9/97

******************************************************************************/

#include "elas.h"
#include "sacutil.h"
#include <memory.h>
#include <fcntl.h>

#define  DBF  15              /* #define  DBF  35   */
#define  ntap 60              /* #define  ntap 80   */


#define  mul0 1.0
#define  mulF 0.25            /* #define  mulF 0.35 */
#define  mulT 0.25
#define  mul  mulF*mul0
#define  KFACTOR  (0.90)      /* new k factor from paper (was 0.68) */
/*
#define  KFACTOR  (0.68)
*/


getTopography(par, grid)
Parameter      *par;
FdGrid         *grid;
{

}


getTopogX2d(par, grid)
Parameter   *par;
FdGrid     *grid;
{
  FdTopo topo[1];
  float dist,gpts,dgpts;
  float alpha,beta,mu,lamda,rho;
  float lalast,mulast,rhlast;
  float alphalast, betalast, rholast;
  float zic,zim,basvl; 
  int n1,n2,m1,m2,n,l,m;
  float slp1,slp2;
  int nf,nt,ntnew,i,j;
  int high,low,tmp;

  int nave = 1;
  float zdeep,zshal;

  int bpt;
  float dif;

  float **nwrho;
  int **nwm;
  int **imatrix1();
  float **matrix1();
  float *vector();
  int ii;

  int p;

  /* get memory */
  nwrho=matrix1(0,grid->n,0,nave);
  nwm=imatrix1(0,grid->n,0,nave);
  topo->zi = vector(0,par->n);

  /* initialize */
  for(j = 0; j <= nave; j++) for(i = 0; i <= grid->n; i++) nwrho[i][j] = 0.;

  /* get topography */
  topo->depth = par->depth;
  topo->ext   = par->ext;
  strcpy(topo->file, par->tfile);

  l = 0;

  if (topo->file[0])readTopoFile(par,grid,topo);
  if (!topo->file[0])for(n=0;n<par->n;n++)topo->zi[n]=topo->depth*KMTOM;

  /* CAS - adjustement of free surface for taper and averaging */
/* UNADJUSTMENT DONE AT END for grid topo */
  for(n=0;n<par->n;n++)
    topo->zi[n] += (log(KFACTOR)/log(mulF))*grid->dx; 
  topo->zi[par->n]=topo->zi[par->n-1];

  p = (grid->n+1)*sizeof(float);
  memcpy((char *) (grid->zi), (char *) (topo->zi+grid->n0), p);
/*
  topo->zi += grid->n0;
*/

   m1 = 0;
   n1 = 0;
   m2 = grid->m - 1;
   n2 = grid->n - 1;

  for(n = n1; n <= n2; n++) {
    /* continuous case */
    zic = grid->zi[n]; 

    for(m = m2; m >= m1; m--){
   alpha = grid->alpha[l][m][n];
   beta  = grid->beta [l][m][n];
   rho   = grid->rho  [l][m][n];
   if (alpha > 0.0 && rho > 0.0) {
      alphalast = alpha;
       betalast = beta ;
        rholast = rho  ;
      }
   if (alpha <= 0. && rho <= 0.) {
fprintf(stderr, "Error: zero alpha:rho in topography\n");
exit(-1);
      alpha = alphalast;
      beta  =  betalast;
      rho   =   rholast;
      }
if (alpha <= 0. || rho <=0.) {
fprintf(stderr, "Error: zero alpha or rho in medium (topography)\n");
exit(-1);
}

      zim   = ((float) (m + grid->m0))*grid->dx;
      
      if(zim < zic){
   rho  *= mul0; 
   rhoDecay2d(par,grid,topo,&dist,n+grid->n0,m+grid->m0); 
   gpts = dist/grid->dx;
   
   /* CAS - adjusted density taper - optional */
   if(gpts >= (ntap-DBF) ){ 
     basvl = (DBF - 2.0)/DBF;
     dgpts = (gpts - (ntap-DBF) );
     gpts = (ntap-DBF) + pow(basvl,dgpts) * dgpts;
   }
/*
   rho  *= pow(mulF,gpts);  
*/
rho  *= pow(mulT, gpts);  
      }

      grid->rho  [0][m][n] = rho;

    }
  }

   /* update edges for parallel (could do this directly in routine
      above by changing n1,n2,m1,m2 to -1,grid->n,-1,grid->m for
      mpp boundaries. would have to change back to original n1,n2,m1,m2
      at this point in routine) */
   mpUpdate2D(par, grid, NULL, 4);


  /* pretty much everything below this point should be added */
  /* CAS - post-smoothing to reduce noise along interface 1_8_96*/
  /* please do not release from lab */
  for(n = n1; n <= n2; n++) {
    zic = grid->zi[n]; 
    /*determine the begining grid point*/
    bpt = ((int) (zic / grid->dx));
    if(bpt <= 2){
      /*
      fprintf(stderr,"Warning: topography too close to upper boundary\n ...\n");
      */
    }
    dif = zic - ((float) bpt) * grid->dx;
    if(dif > 0.5*grid->dx )bpt = ((int) (zic / grid->dx)) + 1;

    for(ii = 0;ii<=nave;ii++){
      m = bpt - ii;
      m = m - grid->m0;           /* m is value in local mpp grid */
      nwm[n][ii] = -999;
      if(m >= 0 && m <= grid->m - 1){
/*   zim   = ((float) m)*grid->dx; m would be local */
   
   if(n>0 && n<(grid->n-1) )
     nwrho[n][ii] = 5.0/(1.0/grid->rho[0][m][n-1]+1.0/grid->rho[0][m+1][n] +
               1.0/grid->rho[0][m][n+1]+1.0/grid->rho[0][m-1][n] +
               1.0/grid->rho[0][m][n]) ;
   if(n == 0 && grid->bx1 == MESSAGEPASS)
     nwrho[n][ii] = 5.0/(1.0/grid->rho[0][m][n-1]+1.0/grid->rho[0][m+1][n] +
               1.0/grid->rho[0][m][n+1]+1.0/grid->rho[0][m-1][n] +
               1.0/grid->rho[0][m][n]) ;
   if(n == grid->n-1 && grid->bx2 == MESSAGEPASS)
     nwrho[n][ii] = 5.0/(1.0/grid->rho[0][m][n-1]+1.0/grid->rho[0][m+1][n] +
               1.0/grid->rho[0][m][n+1]+1.0/grid->rho[0][m-1][n] +
               1.0/grid->rho[0][m][n]) ;

   if(nwrho[n][ii] == 0.) {
     nwrho[n][ii] = grid->rho[0][m][n];
     /*fprintf(stderr, "Zero rho value at m: %d n: %d, exiting ...\n",m,n);*/
   }
     
   nwm[n][ii] = m;
      }
    }
  }

  /* CAS - adjusting value if it is along a boundary 1_8_96 */
  /* please do not release from lab */
  for(ii = 0;ii<=nave;ii++){
    if (grid->bx1 != MESSAGEPASS) {
      nwm[0][ii] = nwm[1][ii]; 
      nwrho[0][ii] = nwrho[1][ii];
      }
    if (grid->bx2 != MESSAGEPASS) {
      nwm[grid->n-1][ii] = nwm[grid->n-2][ii]; 
      nwrho[grid->n-1][ii] = nwrho[grid->n-2][ii];
      }
  }

  /* CAS - recalculating density, lambda, mu with new density values 1/8/96 */
  /* please do not release from lab */
  for(ii = 0;ii<=nave;ii++){
    for(n = n1; n <= n2; n++) {
      m = nwm[n][ii];
      if(m != -999){
   rho = nwrho[n][ii];
   grid->rho  [0][m][n] = rho;
      }
    }

  }

/*
  topo->zi -= grid->n0;
*/

  free_matrix1(nwrho, 0,grid->n,0,nave);
  free_matrix1(nwm,   0,grid->n,0,nave);
  free_vector(topo->zi, 0, par->n);

/* UNADJUST of free surface for taper and averaging */
  for(n = 0; n < grid->n + 1; n++)
    grid->zi[n] -= (log(KFACTOR)/log(mulF))*grid->dx;

}

/***************************************************************************
3D Topography
coded by shawn larsen (11/01) llnl
(from 2D version by craig schultz)
***************************************************************************/
getTopogX3d(par, grid)
Parameter   *par;
FdGrid     *grid;
{
   FdTopo topo[1];
   float dist, gpts, dgpts;
   float alpha, beta, rho;
   float zic, zim, basvl, dif;
   float ***nwrho;
   int ***nwm;
   int nave, bpt;
   int n1, n2, l1, l2, m1, m2, n, l, m;
   int i, j, k, ii, gflag, p;
   int    ***itensor1();
   float   ***tensor1();
   float    **matrix1();

   /* get memory */
   nave      = 1;
   nwrho     =  tensor1(0, grid->l, 0, grid->n, 0, nave);
   nwm       = itensor1(0, grid->l, 0, grid->n, 0, nave);
   topo->ziM =  matrix1(0,  par->l, 0,  par->n);

   /* initialize */
   for(j = 0; j <= nave; j++) 
      for(i = 0; i <= grid->n; i++) 
         for(k = 0; k <= grid->l; k++) 
            nwrho[k][i][j] = 0.;

   /* get topography */
   topo->depth = par->depth;
   strcpy(topo->file, par->tfile);

   if ( topo->file[0]) readTopoFile(par, grid, topo);
   if (!topo->file[0]) 
      for(l = 0; l < par->l; l++) 
         for(n = 0; n < par->n; n++) 
            topo->ziM[l][n] = topo->depth*KMTOM;

   /* CAS - adjustement of free surface for taper and averaging */
   /* UNADJUSTMENT DONE AT END for grid topo */
   for(l = 0; l < par->l; l++) for(n = 0; n < par->n; n++)
      topo->ziM[l][n] += (log(KFACTOR)/log(mulF))*grid->dx; 
   for(l = 0; l < par->l; l++) topo->ziM[l][par->n] = topo->ziM[l][par->n-1];
   for(n = 0; n < par->n; n++) topo->ziM[par->l][n] = topo->ziM[par->l-1][n];
   topo->ziM[par->l][par->n] = topo->ziM[par->l-1][par->n-1];

   /* copy topography values from global matrix into subgrid matrix */
   p = (grid->n+1)*sizeof(float);
   for(l = 0; l < grid->l+1; l++)
      memcpy((char *) (grid->ziM[l]), 
             (char *) (topo->ziM[l+grid->l0]+grid->n0), p);

   l1 = 0;
   m1 = 0;
   n1 = 0;
   l2 = grid->l - 1;
   m2 = grid->m - 1;
   n2 = grid->n - 1;

   for(l = l1; l <= l2; l++) for(n = n1; n <= n2; n++) {

      /* continuous case */
      zic = grid->ziM[l][n]; 

      for(m = m2; m >= m1; m--) {
         alpha = grid->alpha[l][m][n];
         beta  = grid->beta [l][m][n];
         rho   = grid->rho  [l][m][n];
         if (alpha <= 0. && rho <= 0.) {
            fprintf(stderr, "Error: zero alpha:rho in topography\n");
            exit(-1);
            }
         if (alpha <= 0. || rho <= 0.) {
            fprintf(stderr,"Error: zero alpha or rho in medium(topography)\n");
            exit(-1);
            }

         zim   = ((float) (m + grid->m0))*grid->dx;

         if (zim < zic) {
            rho  *= mul0; 
            /*
            rhoDecay3d(par,grid,topo,&dist,l+grid->l0,n+grid->n0,m+grid->m0); 
            */
rhoDecay3dQuick(par,grid,topo,&dist,l+grid->l0,n+grid->n0,m+grid->m0); 
            gpts = dist/grid->dx;
   
            /* CAS - adjusted density taper - optional */
            if(gpts >= (ntap - DBF) ) { 
               basvl = (DBF - 2.0)/DBF;
               dgpts = (gpts - (ntap - DBF) );
               gpts = (ntap - DBF) + pow(basvl, dgpts)*dgpts;
               }
/*
            rho  *= pow(mulF, gpts);  
*/
            rho  *= pow(mulT, gpts);  
            }
         grid->rho[l][m][n] = rho;

         /* end loop over m (depth) (z) */
         } 
      /* end loops over l, n (horz) (y, x) */
      }    

   /* update edges for parallel (could do this directly in routine 
      above by changing (l1, l2, n1, n2, m1, m2) to (-1, grid->l, 
      -1, grid->n, -1, grid->m) for mpp boundaries. would have to 
      change back to original (l1, l2, n1, n2, m1, m2) at this point 
      in routine) */
   mpUpdate3D(par, grid, NULL, 4);


   /* pretty much everything below this point should be added */
   /* CAS - post-smoothing to reduce noise along interface 1_8_96 */
   /* please do not release from lab */
   for(l = l1; l <= l2; l++) for(n = n1; n <= n2; n++) {
      zic = grid->ziM[l][n]; 
      /*determine the begining grid point*/
      bpt = ((int) (zic/grid->dx));
      dif = zic - ((float) bpt)*grid->dx;
      if (dif > 0.5*grid->dx) bpt = ((int) (zic/grid->dx)) + 1;

      /* determine if point is along physical boundary of grid */
      gflag = 1;
      if (n ==           0 && grid->bx1 != MESSAGEPASS) gflag = 0;
      if (n == grid->n - 1 && grid->bx2 != MESSAGEPASS) gflag = 0;
      if (l ==           0 && grid->by1 != MESSAGEPASS) gflag = 0;
      if (l == grid->l - 1 && grid->by2 != MESSAGEPASS) gflag = 0;

      for(ii = 0; ii <= nave; ii++) {
         m = bpt - ii;
         m = m - grid->m0;           /* m is value in local mpp grid */
         nwm[l][n][ii] = -999;
         if (m >= 0 && m <= grid->m - 1) {
            if (gflag) {
               nwrho[l][n][ii] = 7.0/(
               1.0/grid->rho[l+1][m  ][n  ] +
               1.0/grid->rho[l-1][m  ][n  ] +
               1.0/grid->rho[l  ][m  ][n-1] +
               1.0/grid->rho[l  ][m+1][n  ] +
               1.0/grid->rho[l  ][m  ][n+1] +
               1.0/grid->rho[l  ][m-1][n  ] +
               1.0/grid->rho[l  ][m  ][n  ]);
/*
               nwrho[l][n][ii] = 5.0/(
               1.0/grid->rho[l  ][m  ][n-1] +
               1.0/grid->rho[l  ][m+1][n  ] +
               1.0/grid->rho[l  ][m  ][n+1] +
               1.0/grid->rho[l  ][m-1][n  ] +
               1.0/grid->rho[l  ][m  ][n  ]);
*/
               }
            if(nwrho[l][n][ii] == 0.) {
               nwrho[l][n][ii] = grid->rho[l][m][n];
               /*fprintf(stderr, "Zero rho at m: %d n: %d, ...\n", m, n);*/
               }
            nwm[l][n][ii] = m;
            }
         }
      }

   /* CAS - adjusting value if it is along a boundary 1_8_96 */
   /* please do not release from lab */
   for(ii = 0; ii <= nave; ii++) {
      if (grid->bx1 != MESSAGEPASS) {
         for(l = 0; l < grid->l; l++) {
            nwm  [l][0][ii] = nwm  [l][1][ii]; 
            nwrho[l][0][ii] = nwrho[l][1][ii];
            }
         }
      if (grid->bx2 != MESSAGEPASS) {
         for(l = 0; l < grid->l; l++) {
            nwm  [l][grid->n-1][ii] = nwm  [l][grid->n-2][ii]; 
            nwrho[l][grid->n-1][ii] = nwrho[l][grid->n-2][ii];
            }
         }
      if (grid->by1 != MESSAGEPASS) {
         for(n = 0; n < grid->n; n++) {
            nwm  [0][n][ii] = nwm  [1][n][ii]; 
            nwrho[0][n][ii] = nwrho[1][n][ii];
            }
         }
      if (grid->by2 != MESSAGEPASS) {
         for(n = 0; n < grid->n; n++) {
            nwm  [grid->l-1][n][ii] = nwm  [grid->l-2][n][ii]; 
            nwrho[grid->l-1][n][ii] = nwrho[grid->l-2][n][ii];
            }
         }
      }

   /* CAS - recalculating density with new density values 1/8/96 */
   /* please do not release from lab */
   for(ii = 0; ii <= nave; ii++) {
      for(l = l1; l <= l2; l++) for(n = n1; n <= n2; n++) {
         m = nwm[l][n][ii];
         if (m != -999) {
            rho = nwrho[l][n][ii];
            grid->rho  [l][m][n] = rho;
            }
         }
      }


   free_tensor1(nwrho,     0, grid->l, 0, grid->n, 0, nave);
   free_tensor1(nwm,       0, grid->l, 0, grid->n, 0, nave);
   free_matrix1(topo->ziM, 0,  par->l, 0,  par->n);

   /* UNADJUST of free surface for taper and averaging */
   for(l = 0; l < grid->l + 1; l++)
      for(n = 0; n < grid->n + 1; n++)
         grid->ziM[l][n] -= (log(KFACTOR)/log(mulF))*grid->dx;

}


int getNextStep(topo,n1,n2,dx)
FdTopo *topo;
int n1,n2;
float dx;
{
 int n,np;
 np = n1;
 for(n = n1+1; n < n2; n++)
    if(fabs(topo->zi[n1+1]-topo->zi[n+1]) >= dx){
      np=n; break;
    }
 np = (np == n1) ? n2 : np; 
 return np;
}

readTopoFile(par,grid,topo)
Parameter *par;
FdGrid    *grid;
FdTopo    *topo;
{
  char header[SACSIZE];
  float dx;
  int n,l,npts;
  int file;
  FILE *ifid;

  if(grid->d == 2){
    if ((file = open(topo->file, O_RDONLY)) == -1) {
      fprintf(stderr, "ERROR: Can't open digitized topography file \"%s\"\n",topo->file);
      exit(-1);
    };
    read(file, header, SACSIZE);
    dx    = *((float *) &header[WORDSIZE*DDELTA]);
    npts  = *((int   *) &header[WORDSIZE*DNPTS ]);
if (npts > par->n) npts = par->n;
/***
    if(dx != grid->dx && dx*KMTOM != grid->dx){
      fprintf(stderr, "ERROR: topo file dx not compatible\n");
      exit(-1);
    }
***/
    for(n = 0; n < par->n; n++) topo->zi[n] = 0.;
    read(file, topo->zi, npts*sizeof(float));
    for(n=0; n < par->n; n++) topo->zi[n] *= KMTOM;
    close(file);
  }
   if (grid->d == 3){
      if ((ifid = fopen(topo->file, "r")) == NULL) {
         fprintf(stderr,"ERROR: digitized topography file \"%s\"\n",topo->file);
         exit(-1);
          };
      for(l = 0; l < par->l; l++) {
         for(n = 0; n < par->n; n++) {
            /*
            fscanf(ifid, "%f", &topo->ziM[l][n]);
            */
            fread(&topo->ziM[l][n], sizeof(float), 1, ifid);
            topo->ziM[l][n] *= KMTOM;
            }
         }
      fclose(ifid);
      }
}

rhoDecay2d(par,grid,topo,dist,npos,mpos)
Parameter *par;
FdGrid    *grid;
FdTopo    *topo;
float     *dist;
int   npos,mpos;
{
  float px,pz,cx,cz;
  float x1,x2,z1,z2;
  float   m0,b0,tmp;
  float   bval,eval;
  int          nval;

  *dist = 1.e28;

  px = ( ((float) npos) + 0.5 ) * grid->dx;
  pz =   ((float) mpos)         * grid->dx;
  
  bval = ntap; 
  eval = ntap;
  if( (npos-bval) <      0) bval = npos;
  if( (npos+eval) > par->n) eval = par->n-npos;

  for(nval=npos-bval; nval<npos+eval; nval++){
    x1 = ( ((float) nval) + 0.5 ) * grid->dx;
    x2 = ( ((float) nval) + 1.5 ) * grid->dx;
    z1 = topo->zi[nval];
    z2 = topo->zi[nval+1];

    m0 = (z2 - z1)/grid->dx;
    b0 = z1 - m0*x1;;

    cx = ( px - m0*(b0-pz) ) / (1. + m0*m0);
    cz = m0*cx + b0;

    if(cx >= x1 && cx <= x2){
      tmp = (cx - px)*(cx - px) + (cz - pz)*(cz - pz) ;
      if( tmp < (*dist)) *dist = tmp;
    } else {
      tmp = (x1 - px)*(x1 - px) + (z1 - pz)*(z1 - pz) ;
      if( tmp < (*dist) ) *dist = tmp;
      tmp = (x2 - px)*(x2 - px) + (z2 - pz)*(z2 - pz) ;
      if( tmp < (*dist) ) *dist = tmp;
    }
  }
  *dist = sqrt( *dist );
}


rhoDecay3d(par, grid, topo, dist, lpos, npos, mpos)
Parameter        *par;
FdGrid          *grid;
FdTopo          *topo;
float           *dist;
int  lpos, npos, mpos;
{
   float px, pz, cx, cz;
   float x1, x2, z1, z2;
   float   m0, b0, tmp;
   float   bval, eval;
   int          nval;

   *dist = 1.e28;

   px = (((float) npos) + 0.5)*grid->dx;
   pz =  ((float) mpos)       *grid->dx;
  
   bval = ntap; 
   eval = ntap;
   if ((npos-bval) <      0) bval = npos;
   if ((npos+eval) > par->n) eval = par->n-npos;

   for(nval = npos - bval; nval < npos + eval; nval++) {
      x1 = (((float) nval) + 0.5 )*grid->dx;
      x2 = (((float) nval) + 1.5 )*grid->dx;
      z1 = topo->ziM[lpos][nval];
      z2 = topo->ziM[lpos][nval+1];

      m0 = (z2 - z1)/grid->dx;
      b0 = z1 - m0*x1;;

      cx = (px - m0*(b0 - pz))/(1. + m0*m0);
      cz = m0*cx + b0;

      if (cx >= x1 && cx <= x2) {
         tmp = (cx - px)*(cx - px) + (cz - pz)*(cz - pz);
         if (tmp < (*dist)) *dist = tmp;
         } 
      else {
         tmp = (x1 - px)*(x1 - px) + (z1 - pz)*(z1 - pz);
         if (tmp < (*dist)) *dist = tmp;
         tmp = (x2 - px)*(x2 - px) + (z2 - pz)*(z2 - pz);
         if (tmp < (*dist)) *dist = tmp;
         }
      }
   *dist = sqrt(*dist);

}

rhoDecay3dQuick(par, grid, topo, dist, lpos, npos, mpos)
Parameter        *par;
FdGrid          *grid;
FdTopo          *topo;
float           *dist;
int  lpos, npos, mpos;
{
   float px, py, pz, cx, cy, cz;
   float x1, x2, y1, y2, z1, z2, dx, dy, dz;
   float   m0, b0, tmp;
   float   lbval, leval, nbval, neval;
   int     lval , nval , mval;
   int     ntap2;

   *dist = 1.e28;

/*
   px = (((float) npos) + 0.5)*grid->dx;
   pz =  ((float) mpos)       *grid->dx;
*/
   px = (((float) npos)      )*grid->dx;
   py = (((float) lpos)      )*grid->dx;
   pz =  ((float) mpos)       *grid->dx;
  
   ntap2 = ntap/2;
   mval  = ((int) rint(topo->ziM[lpos][npos]/grid->dz)) - mpos + 2;
   ntap2 = (mval < ntap2) ? (mval) : (ntap2);
   nbval = ntap2; 
   neval = ntap2;
   lbval = ntap2; 
   leval = ntap2;

   if ((npos-nbval) <      0) nbval = npos;
   if ((npos+neval) > par->n) neval = par->n-npos;
   if ((lpos-lbval) <      0) lbval = lpos;
   if ((lpos+leval) > par->l) leval = par->l-lpos;

   for(lval = lpos - lbval; lval < lpos + leval; lval++)
   for(nval = npos - nbval; nval < npos + neval; nval++) {

      x1 = ((float) nval)*grid->dx;
      y1 = ((float) lval)*grid->dy;
      z1 = topo->ziM[lval][nval];

      dx = (x1 - px);
      dy = (y1 - py);
      dz = (z1 - pz);
      tmp = dx*dx + dy*dy + dz*dz;
      if (tmp < (*dist)) *dist = tmp;

      }

   *dist = sqrt(*dist);

}


float *vector(nl,nh)
int nl,nh;
{
   float *v;

   v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
   if (!v) printf("allocation failure in vector()\n");
   return v-nl;
}

free_vector(v,nl,nh)
float *v;
int nl,nh;
{
   free((char*) (v+nl));
}

float **matrix1(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
   int i;
   float **m;

   m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
   if (!m) printf("allocation failure 1 in matrix()\n");
   m -= nrl;

   for(i=nrl;i<=nrh;i++) {
      m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
      if (!m[i]) printf("allocation failure 2 in matrix()\n");
      m[i] -= ncl;
   }
   return m;
}

float ***tensor1(nrl,nrh,ncl,nch,ntl,nth)
int nrl,nrh,ncl,nch,ntl,nth;
{
   int i,j;
   float ***m;

   m=(float ***) malloc((unsigned) (nrh-nrl+1)*sizeof(float**));
   if (!m) printf("allocation failure 1 in tensor()\n");
   m -= nrl;

   for(i=nrl;i<=nrh;i++) {
     m[i]=(float **) malloc((unsigned) (nch-ncl+1)*sizeof(float*));
     if (!m[i]) printf("allocation failure 2 in tensor()\n");
     m[i] -= ncl;
   }

   for(i=nrl;i<=nrh;i++) {
     for(j=ncl;j<=nch;j++) {
       m[i][j]=(float *) malloc((unsigned) (nth-ntl+1)*sizeof(float));
       if (!m[i][j]) printf("allocation failure 3 in tensor()\n");
       m[i][j] -= ntl;
     }
   }

   return m;
}

int *ivector(nl,nh)
int nl,nh;
{
   int *v;

   v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
   if (!v) printf("allocation failure in ivector()\n");
   return v-nl;
}


int **imatrix1(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
   int i,**m;

   m=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
   if (!m) printf("allocation failure 1 in imatrix()\n");
   m -= nrl;

   for(i=nrl;i<=nrh;i++) {
      m[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int));
      if (!m[i]) printf("allocation failure 2 in imatrix()\n");
      m[i] -= ncl;
   }
   return m;
}

int   ***itensor1(nrl,nrh,ncl,nch,ntl,nth)
int nrl,nrh,ncl,nch,ntl,nth;
{
   int i,j;
   int   ***m;

   m=(int   ***) malloc((unsigned) (nrh-nrl+1)*sizeof(  int**));
   if (!m) printf("allocation failure 1 in itensor()\n");
   m -= nrl;

   for(i=nrl;i<=nrh;i++) {
     m[i]=(int   **) malloc((unsigned) (nch-ncl+1)*sizeof(  int*));
     if (!m[i]) printf("allocation failure 2 in itensor()\n");
     m[i] -= ncl;
   }

   for(i=nrl;i<=nrh;i++) {
     for(j=ncl;j<=nch;j++) {
       m[i][j]=(int   *) malloc((unsigned) (nth-ntl+1)*sizeof(  int));
       if (!m[i][j]) printf("allocation failure 3 in itensor()\n");
       m[i][j] -= ntl;
     }
   }

   return m;
}

free_tensor1(m,nrl,nrh,ncl,nch,ntl,nth)
float ***m;
int nrl,nrh,ncl,nch,ntl,nth;
{
   int i,j;

        for(i=nrh;i>=nrl;i--){
          for(j=nch;j>=ncl;j--){
            free((char**) (m[i][j]+ntl));
           }
        }

   for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
   free((char*) (m+nrl));
}

free_itensor1(m,nrl,nrh,ncl,nch,ntl,nth)
int   ***m;
int nrl,nrh,ncl,nch,ntl,nth;
{
   int i,j;

        for(i=nrh;i>=nrl;i--){
          for(j=nch;j>=ncl;j--){
            free((char**) (m[i][j]+ntl));
           }
        }

   for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
   free((char*) (m+nrl));
}


/*
void 
*/
free_matrix1(m,nrl,nrh,ncl,nch)
float **m;
int nrl,nrh,ncl,nch;
{
   int i;

   for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
   free((char*) (m+nrl));
}

