c     vectorf: fortran vector derivative routines for elastic finite-difference

      subroutine dtf(d, r0, r1, r2, r3, c1, c2, l, p)
      real*4 d(1), r0(1), r1(1), r2(1), r3(1), p(1) 
      real*4 c1, c2
      integer l, n

      do n = 1, l
        d(n) = d(n) + (c1*(r2(n) - r1(n)) + c2*(r3(n) - r0(n)))*p(n)
      enddo

      return
      end


      subroutine dvf(d, r0, r1, r2, r3, c1, c2, l)
      real*4 d(1), r0(1), r1(1), r2(1), r3(1)
      real*4 c1, c2
      integer l, n

      do n = 1, l
        d(n) = (c1*(r2(n) - r1(n)) + c2*(r3(n) - r0(n)))
      enddo

      return
      end


      subroutine dv0f(d, r0, r1, r2, r3, c1, c2, l, g, u)
      real*4 d(1), r0(1), r1(1), r2(1), r3(1), g(1), u(1) 
      real*4 c1, c2
      integer l, n

      do n = 1, l
        d(n) = (c1*(r2(n) - r1(n)) + c2*(r3(n) - r0(n)))
      enddo

      return
      end


      subroutine dv3f(d, r0, r1, r2, r3, c1, c2, l, g, u)
      real*4 d(1), r0(1), r1(1), r2(1), r3(1), g(1), u(1) 
      real*4 c1, c2
      integer l, n

      do n = 1, l
        d(n) = d(n) + (c1*(r2(n) - r1(n)) + c2*(r3(n) - r0(n)))*u(n)
      enddo

      return
      end


      subroutine addv2f(d, v1, v2, l, g, u)
      real*4 d(1), v1(1), v2(1), g(1), u(1)
      integer l, n

c     do n = 1, l
c       d(n) = d(n) + (g(n) + u(n)*2.)*v1(n) + g(n)*v2(n)
c     enddo

      do n = 1, l
        d(n) = d(n) + g(n)*v2(n)
      enddo
      do n = 1, l
        d(n) = d(n) + (g(n) + u(n)*2.)*v1(n)
      enddo

      return
      end

      subroutine addv3f(d, v1, v2, v3, l, g, u)
      real*4 d(1), v1(1), v2(1), v3(1), g(1), u(1)
      integer l, n

c     do n = 1, l
c       d(n) = d(n) + (g(n) + u(n)*2.)*v1(n) + g(n)*(v2(n) + v3(n))
c     enddo

      do n = 1, l
        d(n) = d(n) + g(n)*(v2(n) + v3(n))
      enddo
      do n = 1, l
        d(n) = d(n) + (g(n) + u(n)*2.)*v1(n)
      enddo

      return
      end

      subroutine addvaf(d, v1, v2, v3, l, g   )
      real*4 d(1), v1(1), v2(1), v3(1), g(1)      
      integer l, n

      do n = 1, l
        d(n) = d(n) + g(n)*(v1(n) + v2(n) + v3(n))
      enddo

      return
      end

c     acoustic subroutines
      subroutine dt3da(d, r01, r02, r03, r04, 
     x                    r05, r06, r07, r08, 
     x                    r09, r10, r11, r12, 
     x                    c1, c2, l, g)
      real*4 d(1), g(1) 
      real*4 r01(1), r02(1), r03(1), r04(1) 
      real*4 r05(1), r06(1), r07(1), r08(1) 
      real*4 r09(1), r10(1), r11(1), r12(1) 
      real*4 c1, c2
      integer l, n
c     real*4 v1, v2, v3, v4, v5, v6, s1, s2

      do n = 1, l
        d(n) = d(n) + (c1*(r03(n)-r02(n)+r07(n)-r06(n)+r11(n)-r10(n)) +
     x                 c2*(r04(n)-r01(n)+r08(n)-r05(n)+r12(n)-r09(n)))*
     x                 g(n)
c       d(n) = d(n) + (c1*(r03(n)+r11(n)+r07(n)-r06(n)-r02(n)-r10(n)) +
c    x                 c2*(r04(n)+r12(n)+r08(n)-r05(n)-r01(n)-r09(n)))*
c    x                 g(n)
c       d(n) = d(n) + (c1*(r03(n)+r07(n)+r11(n)-r02(n)-r06(n)-r10(n)) +
c    x                 c2*(r04(n)+r08(n)+r12(n)-r01(n)-r05(n)-r09(n)))*
c    x                 g(n)
c       v1 = r03(n) - r02(n)
c       v2 = r07(n) - r06(n)
c       v3 = r11(n) - r10(n)
c       v4 = r04(n) - r01(n)
c       v5 = r08(n) - r05(n)
c       v6 = r12(n) - r09(n)
c       s1 = v1 + v2 + v3
c       s2 = v4 + v5 + v6
c       d(n) = d(n) + (c1*s1 + c2*s2)*g(n)
      enddo

      return
      end


c     medium averaging subroutines

      subroutine rf(d, p0, p1, l)
      real*4 d(1), p0(1), p1(1)
      integer l, n

      do n = 1, l
        d(n) = 2.0/(p0(n) + p1(n))
      enddo

      return
      end

      subroutine uf(d, u1, u2, u3, u4, l)
      real*4 d(1), u1(1), u2(1), u3(1), u4(1)
      integer l, n

      do n = 1, l
c       inverse averaging the precision unacceptable way
c       d(n) = 4.0*(u1(n)*u2(n)*u3(n)*u4(n))/
c    .             (u3(n)*u4(n)*(u1(n) + u2(n)) +
c    .              u1(n)*u2(n)*(u3(n) + u4(n)))
c       inverse averaging the precision acceptable way
        d(n) = 4.0/(1./u1(n) + 1./u2(n) + 1./u3(n) + 1./u4(n))
c       straight averaging
c       d(n) = 0.25*(u1(n) + u2(n) + u3(n) + u4(n))
      enddo

      return
      end


c     attenuation subroutines

      subroutine da1f(q1,q2,Txx,Tzz,lamda,mu,F0,F1,F2,dv1,dv2,dt,l)
      real*4 q1(1), q2(1), Txx(1), Tzz(1), lamda(1), mu(1)
      real*4 F0(1), F1(1), F2(1), dv1(1), dv2(1), dt 
      real*4 s1, s2, fdv, D1, D2
      integer l, n
      do n = 1, l
        s1 = q1(n)
        s2 = q2(n)
        fdv = F1(n)*(dv1(n) + dv2(n))
        q1(n) = F0(n)*s1 + fdv + F2(n)*dv2(n)
        q2(n) = F0(n)*s2 + fdv + F2(n)*dv1(n)
        s1 = (s1 + q1(n))*0.5
        s2 = (s2 + q2(n))*0.5
        D1 = (lamda(n) + 2.*mu(n))
        D2 = (lamda(n))
        Txx(n) = Txx(n) + dt*(D1*dv1(n) + D2*dv2(n) + s1)
        Tzz(n) = Tzz(n) + dt*(D2*dv1(n) + D1*dv2(n) + s2)
      enddo
      return
      end

      subroutine da2f(q3, Txz, mu, F0, F3, dv1, dv2, dt, l)
      real*4 q3(1), Txz(1), mu(1), F0(1), F3(1), dv1(1), dv2(1), dt 
      real*4 s3, dv
      integer l, n
      do n = 1, l
        s3 = q3(n)
        dv = dv1(n) + dv2(n)
        q3(n) = F0(n)*s3 + F3(n)*dv
        s3 = (s3 + q3(n))*0.5
        Txz(n) = Txz(n) + dt*(mu(n)*dv + s3)
      enddo
      return
      end



c the following are for vectorized paraxial updates


      subroutine dv02f(d, r1, r2, c, l, g, u)
      real*4 d(1), r1(1), r2(1), g(1), u(1)
      real*4 c
      integer l, n

      do n = 1, l
        d(n) = (c*(r2(n) - r1(n)))
      enddo

      return
      end


      subroutine da02f(d, r1, r2, c, l      )
      real*4 d(1), r1(1), r2(1)            
      real*4 c
      integer l, n

      do n = 1, l
        d(n) = (c*(r2(n) - r1(n)))
      enddo

      return
      end


      subroutine dv32f(d, r1, r2, c, l, g, u)
      real*4 d(1), r1(1), r2(1), g(1), u(1)
      real*4 c
      integer l, n

      do n = 1, l
        d(n) = d(n) + (c*(r2(n) - r1(n)))*u(n)
      enddo

      return
      end

      subroutine dv7ft(d, v1, v2, v3, l, g, u)
      real*4 d(1), v1(1), v2(1), v3(1), g(1), u(1)
      integer l, m

      do m = 1, l
        d(m) = v1(m)*(g(m) + 2.0*u(m)) + (v2(m) + v3(m))*g(m)
      enddo

      return
      end


      subroutine dt12f(d, r1, r2, s1, s2, t1, t2, c, l, p)
      real d(1), r1(1), r2(1), s1(1), s2(1), t1(1), t2(1), p(1), c
      integer l, n

      do n = 1, l
        d(n) = d(n) + c*((r2(n) - r1(n)) + 
     .                   (s2(n) - s1(n)) + 
     .                   (t2(n) - t1(n)))*p(n) 
      enddo

      return
      end


      subroutine da12f(d, r1, r2,                 c, l, p)
      real d(1), r1(1), r2(1),                             p(1), c
      integer l, n

      do n = 1, l
        d(n) = d(n) + c*((r2(n) - r1(n)))*p(n) 
      enddo

      return
      end


      subroutine da1bf(d, l, g, u, p)
      real d(1), g(1), u(1), p(1)
      integer l, n

      do n = 1, l
        d(n) = (g(n) + 2.*u(n))*p(n)
      enddo

      return
      end


      subroutine da1af(d, l, g,    p)
      real d(1), g(1),       p(1)
      integer l, n

      do n = 1, l
        d(n) = (g(n)          )*p(n)
      enddo

      return
      end


      subroutine db1bf(d, l, g, u, p)
      real d(1), g(1), u(1), p(1)
      integer l, n

      do n = 1, l
        d(n) = u(n)*p(n)
      enddo

      return
      end


      subroutine ds1bf(d, l)
      real d(1)
      integer l, n

      do n = 1, l
        d(n) = sqrt(d(n))
      enddo

      return
      end


      subroutine dz1bf(d, v, dh, dt, l)
      real d(1), v(1), dh, dt
      integer l, n

      do n = 1, l
        d(n) = (dh - v(n)*dt)/(dh + v(n)*dt)
      enddo

      return
      end


      subroutine dp1bf(d, v, r, t, l)
      real d(1), v(1), r(1), t(1)
      integer l, n

      do n = 1, l
        d(n) = r(n) + t(n)*(d(n) - v(n))
      enddo

      return
      end


      subroutine dp1af(d, v, r,    l)
      real d(1), v(1), r(1)      
      integer l, n

      do n = 1, l
        d(n) = r(n) +      (d(n) - v(n))
      enddo

      return
      end


      subroutine sv02f(d, v, r1, r2, s1, s2, l)
      real d(1), v(1), r1(1), r2(1), s1(1), s2(1)
      integer l, n

      do n = 1, l
        d(n) = v(n) + ((r2(n) - r1(n)) + (s2(n) - s1(n)))
      enddo

      return
      end


      subroutine sv12f(d, v, r1, r2, s1, s2, l, g, u)
      real d(1), v(1), r1(1), r2(1), s1(1), s2(1), g(1), u(1)
      integer l, n

      do n = 1, l
        d(n) = v(n) + ((r2(n) - r1(n)) + (s2(n) - s1(n)))*
     .                g(n)/(g(n) + 2.*u(n))
      enddo

      return
      end

