/******************************************************************************
   visual: convert wavefield to double byte color array - send to window

   usage: writeVisual(grid, data, visual);
      FdGrid         - finite difference grid
      VelocityStress - displacement and stress data
      visual         - runtime visualization parameters

   return: none

   compile: cc -g -c visual.c

   author: shawn larsen
   date:   january 1991
   where:  caltech
******************************************************************************/

#include "elas.h"

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

#define CLEANUP     0
#define PSCOLOR     1
#define PSGRAY      2
#define BORDER      4
#define MINWIDTH   25
#define MINHEIGHT  25
#define XINIT       0
#define YINIT       0
#define GETCOLORVALUE(i) ((unsigned short) (65535/58)*abs((int) (((i)+1)/2)-64))

writeVisual(par, grid, data, visual)
Parameter          *par;
FdGrid            *grid;
VelocityStress    *data;
Image           *visual;
{
   int i;
   if (data->t >= 280)
      i = 0;

   if (grid->d == 2) writeVisual2D(par, grid, data, visual);
   if (grid->d == 3) writeVisual3D(par, grid, data, visual);
/*
   if (grid->d == 3 && par->g == 2) writeVisual3Dzzz(par, grid, data, visual);
   if (grid->d == 3 && par->g == 1) writeVisual3Dxxx(par, grid, data, visual);
*/
}



writeVisual2D(par, grid, data, visual)
Parameter          *par;
FdGrid            *grid;
VelocityStress    *data;
Image           *visual;
{
   float ***X, ***Y, ***Z;
   static int first = 1;
   int i, g, plane, mode, n, m, p, sample;
   int nx, ny, nz, m0, n0, mdim, ndim;
   unsigned char *bytePS;

   /* get grid size */
   m = 0;
   for(g = 0; g < par->g; g++) m += (grid[g].m - 1)/(visual->sample << g) + 1;
   n = (grid[0].n - 1)/visual->sample + 1;
   p = m*n;

   /* get memory for byte array */
   bytePS = (unsigned char *) malloc(p*sizeof(unsigned char));

   /* get PS byte array */
   for(g = 0; g < par->g; g++) {
      p -= ((grid[g].m - 1)/(visual->sample << g) + 1)*n;
      X = data[g].Vx;
      Y = data[g].Vy;
      Z = data[g].Vz;
      nx = grid[g].n; ny = grid[g].l; nz = grid[g].m;
      mode  = D2PLANE;
      plane = 0;
      getField(par, grid+g, X, Y, Z, nx, ny, nz, plane, mode, bytePS+p, visual);
      }

   /* open window on first pass (CAN'T INTEGRATE VDENSITY and PARALLEL) */
   mdim = (par->nnode > 1) ? ((par->m - 1)/visual->sample + 1) : (m); 
   ndim = (par->n - 1)/visual->sample + 1;
   if (par->node == 0 && first) {
      openWindow(ndim, mdim);
      first = 0;
      }

   /* determine plotting coordinates (for multiple grids/nodes) */
   m    = m;         /* y size of current image partition */
   n    = n;         /* x size of current image partition */
   m0   = grid->m0;  /* starting y window pixel of image */
   n0   = grid->n0;  /* starting x window pixel of image */
   m0   = (m0 - 1)/visual->sample + 1;
   n0   = (n0 - 1)/visual->sample + 1;

   
   /* display image on X window screen */
   if (par->node == 0) window(bytePS, 0, 0, m, n);
   for(i = 1; i < par->nnode; i++) 
      mpByteImage(par, grid, 1, m0, n0, m, n, bytePS, i);

   /* free memory for byte array */
   free(bytePS);

}



writeVisual3D(par, grid, data, visual)
Parameter          *par;
FdGrid            *grid;
VelocityStress    *data;
Image           *visual;
{
   int i, j, n, m, l, p, mdim, ndim, nx, ny, nz, m0, n0, pflag;
   int g, plane, mode;
   static int first = 1;
   float ***X, ***Y, ***Z;
   unsigned char *bytePS;

   /* get parameters */
   l   = grid->l;
   m   = grid->m;
   n   = grid->n;
   n   = (visual->d != YZPLANE) ? (grid->n) : (grid->l);
   m   = (visual->d != XYPLANE) ? (grid->m) : (grid->l);
   m   = (m - 1)/visual->sample + 1;
   n   = (n - 1)/visual->sample + 1;

   p   = visual->plane;

   /* determine if visual plane passes through node */
   pflag = 0;
   if (visual->d == XZPLANE) {
      p -= grid->l0;
      if (p >= 0 && p < grid->l) pflag = 1;
      }
   if (visual->d == XYPLANE) {
      p -= grid->m0;
      if (p >= 0 && p < grid->m) pflag = 1;
      }
   if (visual->d == YZPLANE) {
      p -= grid->n0;
      if (p >= 0 && p < grid->n) pflag = 1;
      }

   /* get memory for byte array */
   if (pflag) bytePS = (unsigned char *) malloc(n*m*sizeof(unsigned char));

   /* get memory for image plane */
/*
   if (visual->d == XZPLANE && pflag) {
      X = data->Vx[p];
      Y = data->Vz[p];
      }
   if ((visual->d == XYPLANE || visual->d == YZPLANE) && pflag) {
      X = (float **) malloc((m+2)*sizeof(float *)) + 1;
      Y = (float **) malloc((m+2)*sizeof(float *)) + 1;
      for(j = -1; j <= m; j++) X[j]=(float *) malloc((n+2)*sizeof(float *))+1;
      for(j = -1; j <= m; j++) Y[j]=(float *) malloc((n+2)*sizeof(float *))+1;
      }
*/

   /* copy data */
/*
   if (visual->d == XYPLANE && pflag) {
      for(j = -1; j <= m; j++) for(i = -1; i <= n; i++) {
         X[j][i] = data->Vx[j][p][i];
         Y[j][i] = data->Vy[j][p][i];
         } 
      }
   if (visual->d == YZPLANE && pflag) {
      for(j = -1; j <= m; j++) for(i = -1; i <= n; i++) {
         X[j][i] = data->Vy[i][j][p];
         Y[j][i] = data->Vz[i][j][p];
         } 
      }
*/

   /* get PS byte array */
   for(g = 0; g < par->g; g++) {
      X = data[g].Vx;
      Y = data[g].Vy;
      Z = data[g].Vz;
      nx = grid[g].n; ny = grid[g].l; nz = grid[g].m;
      mode  = visual->d;
      plane = p;
      if (pflag)
      getField(par, grid+g, X, Y, Z, nx, ny, nz, plane, mode, bytePS+0, visual);
      }

   /* open window on first pass (CAN'T INTEGRATE VDENSITY and PARALLEL) */
   mdim = (par->m - 1)/visual->sample + 1;
   ndim = (par->n - 1)/visual->sample + 1;   
   if (visual->d == XYPLANE) mdim = (par->l - 1)/visual->sample + 1;
   if (visual->d == YZPLANE) ndim = (par->l - 1)/visual->sample + 1;   
   if (par->node == 0 && first) {
      openWindow(ndim, mdim); 
      first = 0;
      }

   /* determine plotting coordinates (for multiple grids/nodes) */
   m    = m;         /* y size of current image partition */
   n    = n;         /* x size of current image partition */
   m0   = grid->m0;  /* starting y window pixel of image */
   n0   = grid->n0;  /* starting x window pixel of image */
   if (visual->d == XYPLANE) m0 = grid->l0;
   if (visual->d == YZPLANE) n0 = grid->l0;
   m0   = (m0 - 1)/visual->sample + 1;
   n0   = (n0 - 1)/visual->sample + 1;


   if (par->node == 0 && pflag) window(bytePS, 0, 0, m, n);
   for(i = 1; i < par->nnode; i++) {
      mpByteImage(par, grid, pflag, m0, n0, m, n, bytePS, i);
      }

   /* free memory for byte array and image plane */
   if (pflag) free(bytePS);
/*
   if (visual->d != XZPLANE && pflag) {
      for(j = -1; j <= m; j++) {free(X[j]-1); free(Y[j]-1);}
      free(X-1); free(Y-1);
      }
*/

}




/******************************************************************************
   getField: get P & S waveField in double byte format
******************************************************************************/
getField(par, grid, X, Y, Z, nx, ny, nz, plane, mode, bytePS, visual)
Parameter                  *par;
FdGrid                    *grid;
Image                   *visual;
float          ***X, ***Y, ***Z;
int     nx, ny, nz, plane, mode;
unsigned char           *bytePS;
{
   float **X2D, **Z2D, P, S, scale;
   float getP2D(), getS2D(), getP3D(), getS3D();
   int index, l, m, n, byteAmp, sample;
   unsigned char c;

   /* determine division factor (width of 58) */
   scale = (visual->scale) ? (((float) 58)*visual->scale) : (1.);
   sample = visual->sample << grid->g;

   /* output P & S combined wave field as an integrated byte array (2D) */
   if (mode == D2PLANE) {
      X2D = X[0];
      Z2D = Z[0];
      index = 0;
      for(m = 0; m < nz; m++) {
         for(n = 0; n < nx; n++) {
            if (m%sample + n%sample) continue;
            if ((m + n)%(2*sample) == 0 /*m%2 - n%2*/) {
               P = getP2D(X2D, Z2D, grid->dx, grid->dz, m, n);
               if (par->topo) if (getAirPoint(par, grid, 0, m, n)) P = 0;
               byteAmp = (int) (scale*P);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 127);
               if (par->topo) if (getAirPoint(par, grid, 0, m, n)) 
                  c = (unsigned char) 255;
               }
            else {
               S = getS2D(X2D, Z2D, grid->dx, grid->dz, m, n);
               if (par->topo) if (getAirPoint(par, grid, 0, m, n)) S = 0;
               byteAmp = (int) (scale*S);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 128);
               if (par->topo) if (getAirPoint(par, grid, 0, m, n)) 
                  c = (unsigned char) 255;
               }
            bytePS[index++] = c;
            }
         }
      }

   /* output P & S combined wave field as an integrated byte array (3D - XY) */
   if (mode == XYPLANE) {
      index = 0;
      m = plane;
      for(l = 0; l < ny; l++) {
         for(n = 0; n < nx; n++) {
            if (l%sample + n%sample) continue;
            if ((l + n)%(2*sample) == 0 /*l%2 - n%2*/) {
               P = getP3D(X, Y, Z, grid->dx, grid->dy, grid->dz, l, m, n);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) P = 0;
               byteAmp = (int) (scale*P);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 127);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) 
                  c = (unsigned char) 255;
               }
            else {
               S = getS3D(X, Y, Z, grid->dx, grid->dy, grid->dz, l, m, n);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) S = 0;
               byteAmp = (int) (scale*S);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 128);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) 
                  c = (unsigned char) 255;
               }
            bytePS[index++] = c;
            }
         }
      }

   /* output P & S combined wave field as an integrated byte array (3D - XZ) */
   if (mode == XZPLANE) {
      index = 0;
      l = plane;
      for(m = 0; m < nz; m++) {
         for(n = 0; n < nx; n++) {
            if (m%sample + n%sample) continue;
            if ((m + n)%(2*sample) == 0 /*m%2 - n%2*/) {
               P = getP3D(X, Y, Z, grid->dx, grid->dy, grid->dz, l, m, n);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) P = 0;
               byteAmp = (int) (scale*P);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 127);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) 
                  c = (unsigned char) 255;
               }
            else {
               S = getS3D(X, Y, Z, grid->dx, grid->dy, grid->dz, l, m, n);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) S = 0;
               byteAmp = (int) (scale*S);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 128);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) 
                  c = (unsigned char) 255;
               }
            bytePS[index++] = c;
            }
         }
      }

   /* output P & S combined wave field as an integrated byte array (3D - YZ) */
   if (mode == YZPLANE) {
      index = 0;
      n = plane;
      for(m = 0; m < nz; m++) {
         for(l = 0; l < ny; l++) {
            if (m%sample + l%sample) continue;
            if ((m + l)%(2*sample) == 0 /*m%2 - l%2*/) {
               P = getP3D(X, Y, Z, grid->dx, grid->dy, grid->dz, l, m, n);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) P = 0;
               byteAmp = (int) (scale*P);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 127);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) 
                  c = (unsigned char) 255;
               }
            else {
               S = getS3D(X, Y, Z, grid->dx, grid->dy, grid->dz, l, m, n);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) S = 0;
               byteAmp = (int) (scale*S);
               byteAmp = (byteAmp>58) ? 58 : ((byteAmp<-58) ? -58 : byteAmp);
               c = (unsigned char) (2*byteAmp + 128);
               if (par->topo) if (getAirPoint(par, grid, l, m, n)) 
                  c = (unsigned char) 255;
               }
            bytePS[index++] = c;
            }
         }
      }

}
 
 


putBytePS(bytePS, m0, n0, m, n, node)
unsigned char *bytePS;
int m0, n0, m, n, node;
{
   /*window(bytePS, m0, n0, m, n, node);*/
}



static Display *display;
static int       screen;
static Window     w = 0;
static XImage    *image;
static GC            gc;
static int displaydepth;

window(bytePS, m0, n0, m, n)
unsigned char *bytePS;
int m0, n0, m, n;
{
   int i, r[256], g[256], b[256], index;
   unsigned char *bytePS4;
   XImage    *subimage;
   /*
   image->data = (char *) bytePS;
   XPutImage(display, w, gc, image, 0, 0, n0, m0, n, m);
   */
   if (displaydepth ==  8) {
      subimage = XCreateImage(display, DefaultVisual(display, screen),
                              8, ZPixmap, 0, NULL, n, m,  8, 0);
      subimage->data = (char *) bytePS;
      }
   if (displaydepth == 24) {
      subimage = XCreateImage(display, DefaultVisual(display, screen),
                             24, ZPixmap, 0, NULL, n, m, 32, 0);
      bytePS4 = (unsigned char *) malloc(4*m*n*sizeof(unsigned char));
      for(i =   0; i < 256; i++) {r[i] = g[i] = b[i] = (unsigned char)   0;}
      for(i =  11; i < 245; i++) {
         r[i] = (!(i %   2)) ? (0) : (GETCOLORVALUE(i));
         g[i] = ( (i %   2)) ? (0) : (GETCOLORVALUE(i));
         b[i] = ( (i > 128)) ? (0) : (GETCOLORVALUE(i));
         }
      r[255] = g[255] = b[255] = 255;
      for(i = 0; i < m*n; i++) {
         index = bytePS[i];
         bytePS4[4*i + 0] = (unsigned char) b[index];
         bytePS4[4*i + 1] = (unsigned char) g[index];
         bytePS4[4*i + 2] = (unsigned char) r[index];
         bytePS4[4*i + 3] = (unsigned char)        0;
         }
      subimage->data = (char *) bytePS4;
      }
   XPutImage(display, w, gc, subimage, 0, 0, n0, m0, n, m);
   subimage->data = (char *) NULL;
   XDestroyImage(subimage);
   XFlush(display);
   if (displaydepth == 24) free(bytePS4);
}




openWindow(xdim, ydim)
int xdim, ydim;
{
   XSizeHints hints;
   XGCValues values;

   /* initialize */
   char *displayName = NULL;
   hints.flags       = 0;
   hints.width       = (xdim > MINWIDTH ) ? (xdim) : (MINWIDTH);
   hints.height      = (ydim > MINHEIGHT) ? (ydim) : (MINHEIGHT);
   hints.flags       = PSize | PMinSize;
   hints.min_width   = hints.width;
   hints.min_height  = hints.height;

   /* open display and screen */
   display = XOpenDisplay(displayName);
   if (display == NULL) {
      fprintf(stderr, "ERROR: can't open display; check env DISPLAY\n");
      exit(-1);
      }
   screen  = DefaultScreen(display);
   displaydepth = DisplayPlanes(display, screen);
   if (displaydepth != 8 && displaydepth != 24) {
      fprintf(stderr, "Error: bad display depth (needs to be 8 or 24)\n");
      exit(1);
      }

   /* open window */
   w = XCreateSimpleWindow(display, RootWindow(display, screen), 
       XINIT, YINIT, hints.width, hints.height, BORDER, 
       BlackPixel(display, screen), WhitePixel(display, screen));
   XSetStandardProperties(display, w, "E3D", "e3d",
                          None, NULL, 0, &hints);

   /* get graphics parameters */
   if (displaydepth == 8) setColorTable(PSCOLOR);
   gc = XCreateGC(display, w, 0, &values);
   image = XCreateImage(display, DefaultVisual(display, screen),
                        8, ZPixmap, 0, NULL, xdim, ydim, 8, 0);

   /* open window and plot */
   XMapWindow(display, w);
   XFlush(display);

}




setColorTable(mode)
int mode;
{
   Visual *v;
   static Colormap cmap;
   XColor colorcell[1];
   unsigned long index;
   unsigned short getColorValue();

   switch(mode) {
      case CLEANUP:
         XFreeColormap(display, cmap);
         XInstallColormap(display, DefaultColormap(display, screen));
         break;
      case PSCOLOR:
         v = DefaultVisual(display, screen);
         cmap  = XCreateColormap(display, w, v, AllocAll);
         colorcell->flags = (DoRed | DoGreen | DoBlue);
         for(index = 0; index <  7; index++) { 
            colorcell->pixel = index;
            XQueryColor(display, DefaultColormap(display, screen), colorcell);
            XStoreColor(display, cmap , colorcell);
            }
         for(index = 11; index < 245; index++) {
            colorcell->pixel = index;
            colorcell->red   = (!(index %   2)) ? (0) : (GETCOLORVALUE(index));
            colorcell->green = ( (index %   2)) ? (0) : (GETCOLORVALUE(index));
            colorcell->blue  = ( (index > 128)) ? (0) : (GETCOLORVALUE(index));
            XStoreColor(display, cmap, colorcell);
            } 
         colorcell->pixel =   255;
         colorcell->red   = 65535;
         colorcell->green = 65535;
         colorcell->blue  = 65535;
         XStoreColor(display, cmap, colorcell);
         XInstallColormap(display, cmap);
         XSetWindowColormap(display, w, cmap);
         break;
      case PSGRAY:
         v = DefaultVisual(display, screen);
         cmap  = XCreateColormap(display, w, v, AllocAll);
         colorcell->flags = (DoRed | DoGreen | DoBlue);
         for(index = 0; index <  7; index++) { 
            colorcell->pixel = index;
            XQueryColor(display, DefaultColormap(display, screen), colorcell);
            XStoreColor(display, cmap , colorcell);
            }
         for(index = 11; index < 245; index++) {
            colorcell->pixel = index;
            colorcell->red   = 256*(index + 1) - 1;
            colorcell->green = 256*(index + 1) - 1;
            colorcell->blue  = 256*(index + 1) - 1;
            XStoreColor(display, cmap, colorcell);
            } 
         colorcell->pixel =   255;
         colorcell->red   = 65535;
         colorcell->green = 65535;
         colorcell->blue  = 65535;
         XInstallColormap(display, cmap);
         XSetWindowColormap(display, w, cmap);
         break;
      default:
         fprintf(stderr, "Warning: bad color mode %d\n", mode);
         break;
      }

}



unsigned short
getColorValue(index)
unsigned long index;
{
   return((unsigned short) (65535/58)*abs((int) ((index + 1)/2) - 64));
}




closeWindow()
{

}




